/***************************************************************************/
/*       Lotos, an extensible overlay network simulator                    */
/*       MNG Group                                                         */
/*       University of Virginia                                            */
/*       July, 2002                                                        */
/***************************************************************************/

package lotos;

import lotos.*; 
import lotos.event.SimEventMAC;
import lotos.event.SimEventMsg;
/**
 * The interface for MAC layer protocol
 * 
 * @revised Guangyu Dong, Jan 6, 2004
 */
public interface I_MACInterface
{
	/* 
	 * Send a message to MAC layer
	 */
	boolean sendMsg(SimEventMsg msg);
	
	/* 
	 * Used to tell the MAC layer a MAC in transmission range
	 * is going to send out a packet
	 * @param ms_start_time the start time of sending in MS
	 * @param ms_end_time the end time of sending in MS 
	 */
	void recordMessage(long ms_start_time, long ms_end_time);
	/* 
	 * Used to tell the MAC layer a MAC in sensing range
	 * is going to send out a packet
	 * @param ms_start_time the start time of sending in MS
	 * @param ms_end_time the end time of sending in MS 
	 */
	void senseMessage(long ms_start_time, long ms_end_time);
	
	/*
	 * go forward a step and update the status
	 */
	void MACStep();
	
	/*
	 * Initiating
	 */
	void MACInit();
	/*
	 * @return size of mac layer header in Bytes
	 */
	int getHeaderSize();
	
	long getSendTime(int msgSize);
	
}
