/***************************************************************************/
/*       Lotos, an extensible overlay network simulator                    */
/*       MNG Group                                                         */
/*       University of Virginia                                            */
/*       July, 2002                                                        */
/***************************************************************************/
package lotos;

import lotos.*; 
import lotos.event.SimEventMsg;

import hypercast.I_Stats;

import java.awt.*;
import java.util.*;

/**
 * This interface abstracts the the services provided by the physical layer.
 * A logical node can only access services from the physical network from
 * this interface.
 */

public interface I_PhysicalInterface {

    //services to the logical layer
    /** the physical links of this node */
    Enumeration physicalLinks();
   
    /** <0 if the physical node is not a neighbor
     *  otherwise, the link weight
     */
    //float physicalWeight( physicalInterface );
    /**
     * @return the physical address of the node
     */
    XYAddress physicalAddress();
    //both the timer and the messages
    /**
     * send a message
     * @param msg the message
     * @return whether the message is sent successfully or not
     */
    boolean sendMsg( SimEventMsg msg);
    
    //the lotos adapter now handles its own timer events directly with the event processor.
    /**
     * set a timer event
     * @param interval the time after which this timer expires in microseconds
     * @param the type of the timer
     */
    //void setTimer( long interval, Object index );
    /**
     * @return the current simulation time
     */
    //long simTime( );
    
    /** added by Guangyu to provide support for Virtual Adapter to run
     *  hypercast node.
     */
    /**
     * Get the delay for the next specific timer event
     * @param index
     * @return
     */
    //long getTimer(Object index);
	/**
	 * Clear the specific timer events
	 * @param index
	 * @return
	 */
	//void clearTimer(Object index);
	
	/**
	 * registers a hypercast node for statistics
	 *
	 */
	//void registerHCNode(I_Stats hcnode);
    /**
     * @return the protocol running on the node.
     */
	public I_ProtocolInterface getProtocol();
	
}
