/***************************************************************************/
/*       Lotos, an extensible overlay network simulator                    */
/*       MNG Group                                                         */
/*       University of Virginia                                            */
/*       July, 2002                                                        */
/***************************************************************************/
package lotos;

import lotos.*; 
import lotos.event.SimEventMsg;
import hypercast.I_OverlaySocket;
import hypercast.I_Stats;
import hypercast.I_LogicalAddress;
import hypercast.I_PhysicalAddress;

import java.awt.*;
import java.util.Enumeration;
import java.util.LinkedList;
import java.util.Vector;

/**
 * This is the interfaces implemented by all the overlay protocols.
 * The physical network communiates with the protocol through this interface.
 * The interface abstracts two main functions: </p>
 * a. display related function, i.e. how to display the logial node</p>
 * b. protocol-related function, i.e. how to handle timer/packet event and
 * the start and reset of the protocol
 */
/**
 * revised by Guangyu Dong, Jan 6, 2004
 * add getName().
 */
public interface I_ProtocolInterface extends I_Stats, I_EventHandler {

    static final int shapeCircle = 0;
    static final int shapeTriangle = 1;
    static final int shapeSquare = 2;

	/** the logical links of this node */
    Enumeration logicalLinks();
    /** insert a logical edge
     *  @param n the other end of the edge
     *  @param c the color of the edge, for display purpose
     *  @param width the width of the edge, for display purpose
     */
    boolean insertLogicalEdge( PhysicalNode n, Color c, int width);
    /**
     * remove a logical edge
     * @param n the logical neighbor
     */
    boolean removeLogicalEdge( PhysicalNode n );

	public void clearLogicalEdges();
	public Vector getLogicalNeighbors();
	
	public Enumeration getLogicalNeighbor();

	/**
	 * @return the colors of the logical links
	 */
	public Enumeration getLogicalColor();

	/**
	 * @return the lengths of the logical links
	 */
	public Enumeration getLogicalWidth();
	public Vector getExternalNeighbors();
	
	/** the event list */
	public LinkedList getEventList();
	public void checkEventListSize();

	/* added by Guangyu */
	/** @return a label to be displayer on the GUI */
	public String getName();
	/** @return name for use in log file */
	public String getDetailedName();
    //*display properties
    /** the color of the logical node */
    public Color getColor();
    /** the shape of the logical node */
    public int getShape();

    /** handle message event */
    public void handleMsg( SimEventMsg msg );

    /** start the protocol at this logical node */
    public void startProtocol( );
    /** restart the protocol to its default state */
    public void restartProtocol();
    /** cleanly stop the protocol */
    public void stopProtocol();
    
    /** convert the protocol setting to a String */
    /** it's used to save the topology to a file */
    //String getConfigString();
    /** configurate the protocol with a String */
    /** it's used to read the topology from a file*/
    //void setConfigString(String config);
    /** the hypercast node */
    //public I_OverlaySocket getHCNode();
    public I_LogicalAddress getParent(I_LogicalAddress la);
    public I_LogicalAddress getLogicalAddress();
    public I_PhysicalAddress getPhysicalAddress();
    
    public void setNodeAdapter(LotosAdapter adpt);
    public void setScktAdapter(LotosAdapter adpt);
    
    public LotosAdapter getNodeAdapter();
    public LotosAdapter getScktAdapter();
}
