package lotos;

import lotos.*; 
import java.awt.Color;

import hypercast.*;
import hypercast.events.NOTIFICATION_EVENT;

/**
 * 
 * @author Guangyu, Dong
 *
 * User implements this interface to put a new HyperCast protocol into the 
 * LoTos simulator.
 * 
 * The instance of this interface will be dynamically loaded, so use constructor
 * without parameter only.
 *  
 */
public interface I_SimHyperCastAgent {
	
	/**
	 * Get the name of this protocol
	 * @return
	 */
	public String getName();
	/**
	 * Set the network object
	 * @param network
	 */
	public void setNetwork(simHCNetwork network);
	/**
	 * create the next I_Node object to be put into the simulator.
	 * 
	 * If needServers() return true, then this function create a server node
	 * that is needed to run the protocol.
	 * 
	 * @param coords	the coordinates of this node on the LoTos window
	 * @param info		if is not null, use this info to create a new node
	 * @return
	 */
	public I_OverlaySocket createSocket(HyperCastConfig config, String info, I_PhysicalAddress addr, I_ReceiveCallback callback, NotificationHandler handler, I_InterceptionCallback intercept);
	
	/**
	 * Check if need to create some servers before running the protocol.
	 * If the result is true, those servers can be created by calling createNode
	 * @return
	 */
	public boolean needServers(HyperCastConfig config);
	/**
	 * Get the information string of the current node, which can be used to restore it in another session
	 * @return
	 */
	public String getInfoForNode(I_OverlaySocket node);
	
	/**
	 * Get the labels of the menu items for this protocol.
	 * 
	 * There should not have duplicate menu names in this list
	 * 
	 * @return
	 */
	public String [] getMenuItems();
	
	/**
	 * Fires the menu item by the menu's name.
	 * 
	 * @param menu
	 */
	public void menuFired(String menu);
	
	/**
	 * Get the color for a I_Node object. The resulting color should depends on
	 * the current state of the node.
	 * @param node
	 * @return
	 */
	public Color getColorForNode(I_OverlaySocket node);
	
	/**
	 * Get the shape for a I_Node object. The resulting shape should depends on
	 * the type of the node.
	 * 
	 * @param node
	 * @return
	 */
	public int getShapeForNode(I_OverlaySocket node);
	
	/**
	 * Get the display label for a node. 
	 * @param node
	 * @return
	 */
	public String getLabelForNode(I_OverlaySocket node);
	
	
	/**
	 * Called when a protocol node is removed from LoTos
	 * @param node
	 * @return true if this protocol node can be removed
	 */
	public void nodeRemoved(I_OverlaySocket node);
	
	public I_LogicalAddress getLA();
	public I_PhysicalAddress getPA();
	/**
	 * finishes the configuration of a hypercast configuration, filling in any protocol dependent values
	 * @param config
	 */
	public void finishConfig(HyperCastConfig config, PhysicalNode node);
	
}
