package lotos;

import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.OutputStream;

import lotos.history.EventHistoryEntry;

public class LogFile {
	String filename;
	boolean logOpen;
	PrintWriter log;
	
	public LogFile(String name) {
		filename = name;
		logOpen = false;
		
	}
	public boolean logging() {
		return logOpen;
	}
	public void startLog(String postfix) {
		if(logOpen)
			endLog();
		try {
			log = new PrintWriter(new FileWriter(filename+postfix+".lotoslog"), true);
			logOpen = true;
		} catch(IOException ioe) {
			System.out.println("Error: Could not open log file");
			System.out.println(ioe.getMessage());
		}
		log.println("<lotoslog id=\""+postfix+"\">");
	}
	
	public void endLog() {
		if(logOpen) {
			log.println("</lotoslog>");
			log.close();
			logOpen = false;
		}
	}
	
	public void writeLine(String line) {
		if(!logOpen)
			return;
		log.println(line);
	}
	
	public void writeEvent(EventHistoryEntry event) {
		writeLine(event.logEntry());
	}
	
}

class SystemOutputStream extends OutputStream {

	String type;
	
	public SystemOutputStream(String typestr) {
		type = typestr;
	}
	
	public void close() {
		
	}
   
	public void flush() {
		
	}

	public void	write(byte[] b) {
		writeStr(new String(b));
	}

	public void	write(byte[] b, int off, int len) {
		writeStr(new String(b,off,len));
	}
	
	@Override
	public void write(int b) throws IOException {
		writeStr(String.valueOf(b));
		
	}
	
	private void writeStr(String output) {
		String logEntry = "<event node=\"System\" type=\""+type+"\">\n";
		logEntry += "\t<output>"+replaceSpecialChars(output)+"</output>\n";
		logEntry += "</event>";
		SimConfig.log.writeLine(logEntry);
	}
	
	private String replaceSpecialChars(String str) {
		String tmp = str.replaceAll("<", "&lt;");
		tmp = tmp.replaceAll(">", "&gt;");
		return tmp;
	}
}