

/**
 * This class implements a virtual adapter
 * used to run Hypercast Node in Lotos
 * 
 * @author Guangyu Dong, Feb 2, 2004
 */

 

package lotos;
import lotos.event.*;
import lotos.history.MessageHistoryEntry;
import lotos.history.TimerHistoryEntry;
import lotos.gateway.*;

import hypercast.*;
import hypercast.util.*;

import java.net.InetAddress;

import org.apache.xpath.XPath;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class LotosAdapter implements I_MulticastAdapter, I_EventHandler {

	/** the physical node using this adapter */
	private I_PhysicalInterface phy;
	/** the physical address of this adapter */
	private SimPhysicalAddress phy_addr;
	/** callback */
	public I_AdapterCallback callback;

	/** state */
	private boolean started;
	
	String statsName = "NodeAdapter";
	public static SimPhysicalAddress multicastAddress;
	
	/**
	 * A StatsProcessor instance which handles the statistics in this object.
	 */
	protected StatsProcessor statsPro;
	
	/**
	 *  Traffic Statistics 
	 */
	long UReceivedPackets = 0;
	long MReceivedPackets = 0;
	long USentPackets = 0;
	long MSentPackets = 0;
	long UReceivedBytes = 0;
	long MReceivedBytes = 0;
	long USentBytes = 0;
	long MSentBytes = 0;
	
	/** variables needed for hypercast socket implementation */
	String AdptIndicator;
	String PROPERTY_NAME_PREFIX;
	
	Object stopTimer;
	
	/** socket adapter. if null, this is the socket adapter. if not null, this is the node adapter */
	public I_UnicastAdapter socketAdpt;
	
	public LotosAdapter (HyperCastConfig config, I_UnicastAdapter adapter, String physicalAddressString, String prefix) {
		if (prefix.startsWith("NodeAdpt"))
		{
			AdptIndicator = "nodeadpt";
			PROPERTY_NAME_PREFIX = "/Public/NodeAdapter/"+prefix+"/";
		} 
		else
		{
			AdptIndicator = "scktadpt";
			PROPERTY_NAME_PREFIX = "/Public/SocketAdapter/"+prefix+"/";
		} 
		
		socketAdpt = adapter;
		InitStatisticsStructure();
		
		String paddr = config.getTextAttribute(XmlUtil.createXPath(PROPERTY_NAME_PREFIX + "PhysicalAddress"));
		if(SimConfig.hybrid)
			phy_addr = new SimPhysicalAddress(new INETv4TwoPorts(paddr));
		else
			phy_addr = new SimPhysicalAddress(new Integer(paddr).intValue());
		//lotosDebug.println("creating "+prefix+" with address "+phy_addr);
		phy = LotosAdapterManager.searchAddress(phy_addr);
		started = true;
		if(adapter==null)
			phy.getProtocol().setScktAdapter(this);
		else
			phy.getProtocol().setNodeAdapter(this);
		
		stopTimer = new Object();
	}
	private void InitStatisticsStructure()
	{
		
		/*
		 * Initial StatsProcessor instance.
		 */
		statsPro = new StatsProcessor(this/* I_Stats object */, true/* readable flag */, true/* writable flag */);
		
		statsPro.addStatsElement("UPacketsReceived", new UnicastPacketsReceived()/* I_Stats object */, 1, 1);

		statsPro.addStatsElement("UBytesReceived", new UnicastBytesReceived()/* I_Stats object */, 1, 1);

		statsPro.addStatsElement("UPacketsSent", new UnicastPacketsSent()/* I_Stats object */, 1, 1);

		statsPro.addStatsElement("UBytesSent", new UnicastBytesSent()/* I_Stats object */, 1, 1);
		
		statsPro.addStatsElement("MPacketsReceived", new MulticastPacketsReceived(), 1, 1);

		statsPro.addStatsElement("MBytesReceived", new MulticastBytesReceived(), 1, 1);

		statsPro.addStatsElement("MPacketsSent", new MulticastPacketsSent(), 1, 1);

		statsPro.addStatsElement("MBytesSent", new MulticastBytesSent(), 1, 1);
	}
	
	
	/**
	 * This class manages simple statistic "UPacketsReceived".
	 */
	class UnicastPacketsReceived extends SimpleStats
	{
		/**
		 * Gets the value of a simple statistic managed by the implementation of this class.
		 * 
		 * @see SimpleStats#getStats()
		 */
		protected String getStats()
		{
			return new String(""+UReceivedPackets);
		}
	
		/**
		 * Sets the value of a simple statistic managed by the implementation of this class.
		 * 
		 * @see SimpleStats#setStats(String newvalue)
		 */
		protected String setStats(String newValue) throws HyperCastStatsException
		{
			throw new HyperCastStatsException(statisticsName, HyperCastStatsException.WriteToAReadOnlyValue);
		}

		/** 
		 * @see I_Stats#getReadSchame
		 */
		public Element[] getReadSchema(Document doc, XPath xpath) throws HyperCastStatsException
		{
			return XmlUtil.createSchemaElement(doc/* document instance */, statisticsName/* element name */, "xsd:Long"/* type */, 
												null/* restriction base */, null/* pattern value */);
		}
	}

	/**
	 * This class manages simple statistic "UBytesReceived".
	 */
	class UnicastBytesReceived extends SimpleStats
	{
		/**
		 * Gets the value of a simple statistic managed by the implementation of this class.
		 * 
		 * @see SimpleStats#getStats()
		 */
		protected String getStats()
		{
			return new String(""+UReceivedBytes);
		}
	
		/**
		 * Sets the value of a simple statistic managed by the implementation of this class.
		 * 
		 * @see SimpleStats#setStats(String newvalue)
		 */
		protected String setStats(String newValue) throws HyperCastStatsException
		{
			throw new HyperCastStatsException(statisticsName, HyperCastStatsException.WriteToAReadOnlyValue);
		}

		/** 
		 * @see I_Stats#getReadSchame
		 */
		public Element[] getReadSchema(Document doc, XPath xpath) throws HyperCastStatsException
		{
			return XmlUtil.createSchemaElement(doc/* document instance */, statisticsName/* element name */, "xsd:Long"/* type */, 
												null/* restriction base */, null/* pattern value */);
		}
	}

	/**
	 * This class manages simple statistic "UPacketsSent".
	 */
	class UnicastPacketsSent extends SimpleStats
	{
		/**
		 * Gets the value of a simple statistic managed by the implementation of this class.
		 * 
		 * @see SimpleStats#getStats()
		 */
		protected String getStats()
		{
			return new String(""+USentPackets);
		}
	
		/**
		 * Sets the value of a simple statistic managed by the implementation of this class.
		 * 
		 * @see SimpleStats#setStats(String newvalue)
		 */
		protected String setStats(String newValue) throws HyperCastStatsException
		{
			throw new HyperCastStatsException(statisticsName, HyperCastStatsException.WriteToAReadOnlyValue);
		}

		/** 
		 * @see I_Stats#getReadSchame
		 */
		public Element[] getReadSchema(Document doc, XPath xpath) throws HyperCastStatsException
		{
			return XmlUtil.createSchemaElement(doc/* document instance */, statisticsName/* element name */, "xsd:Long"/* type */, 
												null/* restriction base */, null/* pattern value */);
		}
	}

	/**
	 * This class manages simple statistic "UBytesSent".
	 */
	class UnicastBytesSent extends SimpleStats
	{
		/**
		 * Gets the value of a simple statistic managed by the implementation of this class.
		 * 
		 * @see SimpleStats#getStats()
		 */
		protected String getStats()
		{
			return new String(""+USentBytes);
		}
	
		/**
		 * Sets the value of a simple statistic managed by the implementation of this class.
		 * 
		 * @see SimpleStats#setStats(String newvalue)
		 */
		protected String setStats(String newValue) throws HyperCastStatsException
		{
			throw new HyperCastStatsException(statisticsName, HyperCastStatsException.WriteToAReadOnlyValue);
		}

		/** 
		 * @see I_Stats#getReadSchame
		 */
		public Element[] getReadSchema(Document doc, XPath xpath) throws HyperCastStatsException
		{
			return XmlUtil.createSchemaElement(doc/* document instance */, statisticsName/* element name */, "xsd:Long"/* type */, 
												null/* restriction base */, null/* pattern value */);
		}
	}
	

	/**
	 * This class manages simple statistic "MPacketsReceived".
	 */
	class MulticastPacketsReceived extends SimpleStats
	{
		/**
		 * Gets the value of a simple statistic managed by the implementation of this class.
		 * 
		 * @see SimpleStats#getStats()
		 */
		protected String getStats()
		{
			return new String(""+MReceivedPackets);
		}
	
		/**
		 * Sets the value of a simple statistic managed by the implementation of this class.
		 * 
		 * @see SimpleStats#setStats(String newvalue)
		 */
		protected String setStats(String newValue) throws HyperCastStatsException
		{
			throw new HyperCastStatsException(statisticsName, HyperCastStatsException.WriteToAReadOnlyValue);
		}

		/** 
		 * @see I_Stats#getReadSchame
		 */
		public Element[] getReadSchema(Document doc, XPath xpath) throws HyperCastStatsException
		{
			return XmlUtil.createSchemaElement(doc/* document instance */, statisticsName/* element name */, "xsd:Long"/* type */, 
												null/* restriction base */, null/* pattern value */);
		}
	}

	/**
	 * This class manages simple statistic "MBytesReceived".
	 */
	class MulticastBytesReceived extends SimpleStats
	{
		/**
		 * Gets the value of a simple statistic managed by the implementation of this class.
		 * 
		 * @see SimpleStats#getStats()
		 */
		protected String getStats()
		{
			return new String(""+MReceivedBytes);
		}
	
		/**
		 * Sets the value of a simple statistic managed by the implementation of this class.
		 * 
		 * @see SimpleStats#setStats(String newvalue)
		 */
		protected String setStats(String newValue) throws HyperCastStatsException
		{
			throw new HyperCastStatsException(statisticsName, HyperCastStatsException.WriteToAReadOnlyValue);
		}

		/** 
		 * @see I_Stats#getReadSchame
		 */
		public Element[] getReadSchema(Document doc, XPath xpath) throws HyperCastStatsException
		{
			return XmlUtil.createSchemaElement(doc/* document instance */, statisticsName/* element name */, "xsd:Long"/* type */, 
												null/* restriction base */, null/* pattern value */);
		}
	}

	/**
	 * This class manages simple statistic "MPacketsSent".
	 */
	class MulticastPacketsSent extends SimpleStats
	{
		/**
		 * Gets the value of a simple statistic managed by the implementation of this class.
		 * 
		 * @see SimpleStats#getStats()
		 */
		protected String getStats()
		{
			return new String(""+MSentPackets);
		}
	
		/**
		 * Sets the value of a simple statistic managed by the implementation of this class.
		 * 
		 * @see SimpleStats#setStats(String newvalue)
		 */
		protected String setStats(String newValue) throws HyperCastStatsException
		{
			throw new HyperCastStatsException(statisticsName, HyperCastStatsException.WriteToAReadOnlyValue);
		}

		/** 
		 * @see I_Stats#getReadSchame
		 */
		public Element[] getReadSchema(Document doc, XPath xpath) throws HyperCastStatsException
		{
			return XmlUtil.createSchemaElement(doc/* document instance */, statisticsName/* element name */, "xsd:Long"/* type */, 
												null/* restriction base */, null/* pattern value */);
		}
	}

	/**
	 * This class manages simple statistic "MBytesSent".
	 */
	class MulticastBytesSent extends SimpleStats
	{
		/**
		 * Gets the value of a simple statistic managed by the implementation of this class.
		 * 
		 * @see SimpleStats#getStats()
		 */
		protected String getStats()
		{
			return new String(""+MSentBytes);
		}
	
		/**
		 * Sets the value of a simple statistic managed by the implementation of this class.
		 * 
		 * @see SimpleStats#setStats(String newvalue)
		 */
		protected String setStats(String newValue) throws HyperCastStatsException
		{
			throw new HyperCastStatsException(statisticsName, HyperCastStatsException.WriteToAReadOnlyValue);
		}

		/** 
		 * @see I_Stats#getReadSchame
		 */
		public Element[] getReadSchema(Document doc, XPath xpath) throws HyperCastStatsException
		{
			return XmlUtil.createSchemaElement(doc/* document instance */, statisticsName/* element name */, "xsd:Long"/* type */, 
												null/* restriction base */, null/* pattern value */);
		}
	}

	/**
	 * @return	the physical node
	 */
	public I_PhysicalInterface getPhyNode() {
		return phy;
	}
	/**
	 * Handle a message
	 * @param msg
	 */
	/*
	public void handleMsg(SimEventProtocolMsg msg) {
		//lotosDebug.errorln("msg=" + msg.getHCMessage());
		try {
			if(msg.dst==null){
				MReceivedPackets ++;
				MReceivedBytes += msg.msgSize;
			}else{
				UReceivedPackets ++;
				UReceivedBytes += msg.msgSize;
			}
			Protocol protocol = (Protocol)((PhysicalNode)phy).getProtocol();
			if(protocol!=null){


				String dststr = protocol.getDetailedName();
				if(msg.getDst()==null)
					dststr = "Broadcast";
				
				String srcstr = "@Unknown";
				PhysicalNode srcnode = msg.getSrc();
				if(srcnode !=null){
					if(!(srcnode.getProtocol() instanceof LotosGateway)) {
						Protocol srcprotocol = (Protocol)(srcnode).getProtocol();
						if(srcprotocol!=null){
							srcstr = srcprotocol.getDetailedName();
						}
					} else {
						srcstr = "LotosGateway";
					}
				}
				protocol.checkEventListSize();
				protocol.eventList.add(new MessageHistoryEntry(protocol.network.simTime(), false, msg, srcstr, dststr));
			}
			if(callback!=null)
				callback.messageArrivedFromAdapter(msg.getHCMessage());
			//lotosDebug.println("recieved msg from " + msg.srcaddr);
			
		} catch (Exception e) {
			lotosDebug.errorln("LotosAdapter#handleMsg: "+msg.getHCMessage().getClass().getName()+": "+e.toString());
			e.printStackTrace();
		}
	}
	*/
	/**
	 * Handle a timer
	 * @param timerID
	 */
	/*
	public void handleTimer(Object timerID) {
		try {
			
			Protocol protocol = (Protocol)((PhysicalNode)phy).getProtocol();
			if(protocol!=null){
				protocol.checkEventListSize();
				protocol.eventList.add(new TimerHistoryEntry(protocol.network.simTime(), false, timerID, 0.0f));
			}
			
			callback.timerExpired(timerID);
		} catch (Exception e) {
			lotosDebug.errorln("LotosAdapter#handleTimer: "+e.toString());
			e.printStackTrace();
		}
	}
	*/
	public void handleEvent(SimEvent e) {
		if(!started)
			return;
		if(e instanceof SimEventMsg) {
			try {
				SimEventMsg msg = (SimEventMsg)e;
				//handle statistics
				if(msg.dst==null){
					MReceivedPackets ++;
					MReceivedBytes += msg.msgSize;
				}else{
					UReceivedPackets ++;
					UReceivedBytes += msg.msgSize;
				}
				//log the message in the event history
				I_ProtocolInterface protocol = phy.getProtocol();
				if(protocol!=null){	
					String dststr = protocol.getName();
					if(msg.getDst()==null)
						dststr = "Broadcast";
					
					String srcstr = "@Unknown";
					PhysicalNode srcnode = msg.getSrc();
					if(srcnode !=null){
						I_ProtocolInterface srcprotocol = srcnode.getProtocol();
						if(srcprotocol!=null){
							srcstr = srcprotocol.getName();
						}
					}
					synchronized(msg.hc_message) {
						protocol.checkEventListSize();
						MessageHistoryEntry entry = new MessageHistoryEntry(SimEventProcessor.getSimTime(), false, msg, srcstr, dststr, protocol.getDetailedName());
						protocol.getEventList().add(entry);
						if(SimConfig.log != null) {
							SimConfig.log.writeEvent(entry);
						}
					}
				}
				//send the message to the callback.
				synchronized(msg.hc_message) {
					if(callback!=null)
						callback.messageArrivedFromAdapter(msg.getHCMessage());
				}
				//lotosDebug.println("recieved msg from " + msg.srcaddr);
				
			} catch (Exception err) {
				lotosDebug.errorln("LotosAdapter#handleMsg: "+e.toString());
				err.printStackTrace();
			}
		} else if(e instanceof SimEventTimer) {
			try {
				SimEventTimer timer = (SimEventTimer) e;
				Object timerID = timer.getIndex();
				
				I_ProtocolInterface protocol = phy.getProtocol();
				if(protocol!=null){
					protocol.checkEventListSize();
					TimerHistoryEntry entry = new TimerHistoryEntry(SimEventProcessor.getSimTime(), false, timerID, 0.0f, protocol.getDetailedName());
					protocol.getEventList().add(entry);
					if(SimConfig.log != null) {
						SimConfig.log.writeEvent(entry);
					}
				}
				if(timerID == stopTimer) {
					Stop();
				} else {
					callback.timerExpired(timerID);
				}
			} catch (Exception err) {
				lotosDebug.errorln("LotosAdapter#handleTimer: "+err.toString());
				err.printStackTrace();
			}
		}
	}
	/* (non-Javadoc)
	 * @see hypercast.I_MulticastAdapter#sendMulticastMessage(hypercast.I_Message)
	 */
	public void sendMulticastMessage(I_Message mp) {
		if (!started)
			return;
		
		MSentPackets ++;
		MSentBytes +=mp.toByteArray().length;
		
		I_ProtocolInterface protocol = phy.getProtocol();
		SimEventProtocolMsg msg = new SimEventProtocolMsg((PhysicalNode)phy, null, mp);
		String label = "unknown";

		if(protocol!=null){
			String src = protocol.getName();
			String dst = "Broadcast";
			protocol.checkEventListSize();
			MessageHistoryEntry entry = new MessageHistoryEntry(SimEventProcessor.getSimTime(), true, msg, src, dst, protocol.getDetailedName());
			protocol.getEventList().add(entry);
			if(SimConfig.log != null) {
				SimConfig.log.writeEvent(entry);
			}
			//label = protocol.agent.getName(); 
		}
		msg.srcaddr = phy_addr;
		msg.dstaddr = multicastAddress;
		phy.sendMsg(msg);
	}

	/* (non-Javadoc)
	 * @see hypercast.I_MulticastAdapter#getMulticastAddress()
	 */
	public I_PhysicalAddress getMulticastAddress() {
		return multicastAddress;
	}

	/* (non-Javadoc)
	 * @see hypercast.I_UnicastAdapter#Start()
	 */
	public void Start() {
		started = true;
	}

	/* (non-Javadoc)
	 * @see hypercast.I_UnicastAdapter#Stop()
	 */
	public void Stop() {
		started = false;
	}
	public void Stop(long delay) {
		setTimer(stopTimer, delay);
	}
	/* (non-Javadoc)
	 * @see hypercast.I_UnicastAdapter#sendUnicastMessage(hypercast.I_PhysicalAddress, hypercast.I_Message)
	 */
	public void sendUnicastMessage(I_NetworkAddress dst, I_Message mp) {
		if (!started)
			return;
		
		I_ProtocolInterface protocol = phy.getProtocol();
		
		USentPackets ++;
		USentBytes += mp.toByteArray().length;
		PhysicalNode dstnode = null;
		SimPhysicalAddress dstaddr;
		if(dst instanceof SimPhysicalAddress)
			dstaddr = (SimPhysicalAddress)dst;
		else if(dst instanceof INETv4TwoPorts)
			dstaddr = new SimPhysicalAddress((INETv4TwoPorts)dst);
		else
			throw new IllegalArgumentException("dst is of type " + dst);
			
		if(SimConfig.hybrid) {
			if(dstaddr.inetAddr.getScktAdptPort()==0)
				dstaddr.inetAddr = new INETv4TwoPorts(dstaddr.inetAddr.getInetAddress(), dstaddr.inetAddr.getNodeAdptPort(), dstaddr.inetAddr.getNodeAdptPort()+1);
				
		}
		//lotosDebug.println(phy_addr.toString()+ " unicast msg sent to "+ dst.toString());
		if(LotosAdapterManager.addr_list.containsKey(dstaddr.toString())) {
			//lotosDebug.println("no gateway or has key");
			dstnode=(PhysicalNode)LotosAdapterManager.addr_list.get(dstaddr.toString());
		}
		else if(((PhysicalNode)phy).net.gateway_nodes.size() > 0) {//destination is outside lotos, and a gateway exists
			/**
			 * this should route it to one of the gateway nodes (how, still being designed),
			 * but for simple testing, only 1 gateway node will be used.  
			 */
			dstnode = (PhysicalNode)((PhysicalNode)phy).net.gateway_nodes.get(0);
		}
		
		//lotosDebug.println("LotosAdapter - +"+phy_addr+" : sending unicast message to "+dstnode +" - " + dstaddr);
		
		/*
		byte[] bytes = mp.toByteArray();
		for(int i=0;i<bytes.length;i++)
			lotosDebug.print(ByteArrayUtility.toUnsignedShort(bytes, i)+" ");
		lotosDebug.println();
		*/
		//synchronized (mp)
		SimEventMsg msg;
		if(socketAdpt != null)
			msg = new SimEventProtocolMsg((PhysicalNode)phy,dstnode,mp);
		else
			msg = new SimEventApplMsg((PhysicalNode)phy,dstnode,mp);
		
			
		
		if(protocol!=null){
			String src = protocol.getName();
			String dststr = "@"+dstaddr;
			if(dstnode !=null){
				if(!(dstnode.getProtocol() instanceof LotosGateway)) {
					I_ProtocolInterface dstprotocol = (dstnode).getProtocol();
					if(dstprotocol!=null){	
						dststr = dstprotocol.getName();
					}
				}
			}
			synchronized(mp) {
				protocol.checkEventListSize();
				MessageHistoryEntry entry = new MessageHistoryEntry(SimEventProcessor.getSimTime(), true, msg, src, dststr, protocol.getDetailedName());
				protocol.getEventList().add(entry);
				if(SimConfig.log != null) {
					SimConfig.log.writeEvent(entry);
				}
			}
		}
		
		if (dstnode != null) {
			msg.srcaddr = phy_addr;
			msg.dstaddr = dstaddr;
			phy.sendMsg(msg);
		}
		//else lotosDebug.errorln("dst not found for "+dstnode);
	}

	/* (non-Javadoc)
	 * @see hypercast.I_UnicastAdapter#createPhysicalAddress(byte[], int)
	 */
	public I_PhysicalAddress createPhysicalAddress(byte[] byteaddr,	int offset) {
		/*
		byte[] withoutShift = new byte[INETv4TwoPorts.ADDRESS_SIZE];
		System.arraycopy(byteaddr, offset, withoutShift, 0, INETv4TwoPorts.ADDRESS_SIZE);

		INETv4TwoPorts newINETv4 = new INETv4TwoPorts(withoutShift);

		return new SimPhysicalAddress(newINETv4);
		*/
		return new SimPhysicalAddress(byteaddr, offset);
	}

	/** 
	 * not used by hypercast, used in handling adapters in lotos.
	 */
	public I_PhysicalAddress getPhysicalAddress() {
		return phy_addr;
	}

	/* (non-Javadoc)
	 * @see hypercast.I_UnicastAdapter#setCallBack(hypercast.I_AdapterCallBack)
	 */
	public void setCallback(I_AdapterCallback node) {
		callback = node;
	}

	/* (non-Javadoc)
	 * @see hypercast.I_UnicastAdapter#setTimer(int, long)
	 */
	public void setTimer(Object timer_id, long delay_ms) {
		if (!started)
			return;
		if(((PhysicalNode)phy).getProtocol() instanceof Protocol) {
			Protocol protocol = (Protocol)((PhysicalNode)phy).getProtocol();
			if(protocol!=null){
				protocol.checkEventListSize();
				TimerHistoryEntry entry = new TimerHistoryEntry(protocol.network.simTime(), true, timer_id, ((float)delay_ms) / 1000, protocol.getDetailedName());
				protocol.eventList.add(entry);
				if(SimConfig.log != null) {
					SimConfig.log.writeEvent(entry);
				}
			}
		}
		
		SimEventTimer timer = new SimEventTimer(SimEventProcessor.getSimTime() + delay_ms*1000, this, timer_id);
		SimEventProcessor.insertEvent(timer);
	}

	/* (non-Javadoc)
	 * @see hypercast.I_UnicastAdapter#clearTimer(int)
	 */
	public void clearTimer(Object timer_id) {
		//phy.clearTimer(timer_id);
		SimEventProcessor.clearEvent(new SimEventTimer(0, this, timer_id));
	}

	/* (non-Javadoc)
	 * @see hypercast.I_UnicastAdapter#getTimer(int)
	 */
	public long getTimer(Object timer_id) {
		//return (long)(phy.getTimer(timer_id)/1000);
		return SimEventProcessor.getEvent(new SimEventTimer(0, this, timer_id));
	}

	/* (non-Javadoc)
	 * @see hypercast.I_UnicastAdapter#getCurrentTime()
	 */
	public long getCurrentTime() {
		return (long) (SimEventProcessor.getSimTime() / 1000);
		//return System.currentTimeMillis();
	}

	/* (non-Javadoc)
	 * @see hypercast.I_UnicastAdapter#createPhysicalAddress(java.lang.String)
	 */
	public I_PhysicalAddress createPhysicalAddress(String addrString) {
		/*
		INETv4TwoPorts newINETv4 = new INETv4TwoPorts(addrString);
		if (newINETv4.getInetAddress() == null) return null;
		else return new SimPhysicalAddress(newINETv4);
		*/
		return new SimPhysicalAddress(addrString);
	}
	
	/* (non-Javadoc)
	 * @see hypercast.I_UnicastAdapter#createPhysicalAddress()
	 */
	public I_PhysicalAddress createPhysicalAddress() {
		/*
		if(socketAdpt != null)
			return new INETv4TwoPorts(phy_addr);
		else
			return new INETv4TwoPorts(phy_addr.getInetAddress(), phy_addr.getScktAdptPort(), 0);
		*/
		return phy_addr;
	}
	/* (non-Javadoc)
	 * @see hypercast.I_UnicastAdapter#getPhysicalAddressSize()
	 */
	public int getPhysicalAddressSize() {
		//return SimPhysicalAddress.HC_ADDR_SIZE;
		return phy_addr.getSize();
	}
	/* (non-Javadoc)
	 * @see hypercast.I_UnicastAdapter#createUnderlayAddress(byte[], int)
	 */
	public I_UnderlayAddress createUnderlayAddress(byte[] byteaddr, int offset) {
		/*
		byte[] withoutShift = new byte[INETv4OnePort.ADDRESS_SIZE];
		System.arraycopy(byteaddr, offset, withoutShift, 0, INETv4OnePort.ADDRESS_SIZE);
		return new INETv4OnePort(withoutShift);
		*/
		return new SimPhysicalAddress(byteaddr, offset);
	}
	/* (non-Javadoc)
	 * @see hypercast.I_UnicastAdapter#createUnderlayAddress(java.lang.String)
	 */
	public I_UnderlayAddress createUnderlayAddress(String addrString) {
		/*
		INETv4OnePort newINETv4 = new INETv4OnePort(addrString);
		if (newINETv4.getInetAddress() == null) return null;
		else return newINETv4;
		*/
		return new SimPhysicalAddress(addrString);
	}
	/* (non-Javadoc)
	 * @see hypercast.I_UnicastAdapter#createUnderlayAddress()
	 */
	public I_UnderlayAddress createUnderlayAddress() {			
		return phy_addr;
	}
	/* (non-Javadoc)
	 * @see hypercast.I_UnicastAdapter#getUnderlayAddressSize()
	 */
	public int getUnderlayAddressSize() {
		//return SimPhysicalAddress.HC_ADDR_SIZE;
		return phy_addr.getSize();
	}
	/* (non-Javadoc)
	 * @see hypercast.I_UnicastAdapter#Reset(java.lang.Object)
	 */
	public boolean Reset(Object socketObj) {
		UReceivedPackets = 0;
		MReceivedPackets = 0;
		USentPackets = 0;
		MSentPackets = 0;
		UReceivedBytes = 0;
		MReceivedBytes = 0;
		USentBytes = 0;
		MSentBytes = 0;
		return true;
	}
	/* (non-Javadoc)
	 * @see hypercast.I_UnicastAdapter#getInfo()
	 */
	public Object getInfo() {
		return phy_addr;
	}
	
	/* (non-Javadoc)
	 * @see hypercast.I_Stats#getStatsName()
	 */
	public String getStatsName() {
		return statsName;
	}
	/* (non-Javadoc)
	 * @see hypercast.I_Stats#setStatsName(java.lang.String)
	 */
	public void setStatsName(String name) {
		statsName = name;
	}
	/* (non-Javadoc)
	 * @see hypercast.I_Stats#getStats(org.w3c.dom.Document, org.apache.xpath.XPath)
	 */
	public Element[] getStats(Document doc, XPath xpath) throws HyperCastStatsException {
		return statsPro.getStatsResult(doc, xpath);
	}
	/* (non-Javadoc)
	 * @see hypercast.I_Stats#setStats(org.w3c.dom.Document, org.apache.xpath.XPath, org.w3c.dom.Element)
	 */
	public Element[] setStats(Document doc, XPath xpath, Element newValue) throws HyperCastStatsException {
		return statsPro.setStatsResult(doc, xpath, newValue);
	}
	/* (non-Javadoc)
	 * @see hypercast.I_Stats#getReadSchema(org.w3c.dom.Document, org.apache.xpath.XPath)
	 */
	public Element[] getReadSchema(Document doc, XPath xpath) throws HyperCastStatsException {
		return statsPro.getReadSchemaResult(doc, xpath);
	}
	/* (non-Javadoc)
	 * @see hypercast.I_Stats#getWriteSchema(org.w3c.dom.Document, org.apache.xpath.XPath)
	 */
	public Element[] getWriteSchema(Document doc, XPath xpath) throws HyperCastStatsException {
		return statsPro.getWriteSchemaResult(doc, xpath);
	}
	public void close() {
		// TODO Auto-generated method stub
		
	}
	

}