package lotos;

import java.util.Hashtable;
import hypercast.*;
import java.net.*;
import lotos.gateway.*;
import java.util.*;

/**
 * This class creates and manages virtual adapters
 * used to run Hypercast Node 
 * 
 * @author Guangyu Dong, Feb 3, 2004
 * 
 * this class forms the bridge between the hypercast socket and the lotos network.
 * @author Zhi An, 2006
 * 
 */

public class LotosAdapterManager {

	/** the list of all addresses 
	 * physical address.tostring -> physical interface 
	 */
	public static Hashtable addr_list;
	/** this hashtable contains physical addresses, with the corresponding logical address's string representation as a key *
	 * logical address.tostring -> physical address 
	 */
//	public static Hashtable addressPair;
	

	/** the Id number used to generate physical address */
	//private int current_id;
	
	/**
	 * create an address for the physical node. the returned address has the ip of the machine
	 * running lotos, with a port dependent on the physical node id.
	 * 
	 * @param phyNode the physical node the create the adapter on. used to determine port #
	 * @param local the ip address of the local host.
	 * @return the physical address of the node
	 */
	public static SimPhysicalAddress createAddress(int phyNode, InetAddress local) {
		if(SimConfig.hybrid) {
			INETv4TwoPorts addr = new INETv4TwoPorts(local, phyNode*2+LotosGateway.STARTING_PORT, phyNode*2+LotosGateway.STARTING_PORT+1);
			if(addr_list.containsKey(addr.toString())) {
				throw new IllegalArgumentException("PhysicalAddress ["+addr+"]already allocated to another node");
			}
			return new SimPhysicalAddress(addr);
		} else {
			return new SimPhysicalAddress(phyNode);
		}
		
	}
	/**
	 * create an address from an IP address and port
	 * @param id 4 bytes for the IP address
	 * @param p1
	 * @return
	 */
	/*
	public static SimPhysicalAddress createAddress(byte[] ip, int p1) {
		
		try {
			InetAddress inetaddr = InetAddress.getByAddress(ip);
			SimPhysicalAddress addr = new SimPhysicalAddress(new INETv4TwoPorts(inetaddr, p1, p1+1));
			
			return addr;
			
		} catch (UnknownHostException e) {
			lotosDebug.errorln("error in creating  InetAddress");
		}
		return new INETv4TwoPorts(null, p1, p1+1);
	}
	*/
	public synchronized static void reset(){
		if(addr_list == null)
			addr_list = new Hashtable();
		//if(addressPair == null)
			//addressPair = new Hashtable();
		addr_list.clear();
		//addressPair.clear();
	}
	
	/** 
	 * search the address by an address
	 * @param as	address string
	 * @return	address
	 */
	public synchronized static I_PhysicalInterface searchAddress(SimPhysicalAddress ha) {
		if(ha==null)
			return null;
		I_PhysicalInterface node = (I_PhysicalInterface)addr_list.get(ha.toString());
		return node;
	}
	/*
	public synchronized static I_PhysicalInterface searchAddress(INETv4OnePort pa) {
		if(pa==null)
			return null;
		int nodeport = pa.getPort();
		if(pa.getPort()%2==1) //all node adapter ports are even. if port is odd, then it is a socket port, and node port is one less
			nodeport--;
		
		return (I_PhysicalInterface)addr_list.get(new INETv4TwoPorts(pa.getInetAddress(), nodeport, nodeport+1).toString());
	}
	*/
	public synchronized static I_PhysicalInterface searchAddress(String pa) {
		return searchAddress(new SimPhysicalAddress(pa));
	}
	/**
	 * remove an adapter's address from the list
	 * @param adpt	the adapter
	 */
	public synchronized static void removeAdapter(LotosAdapter adpt) {
		Object target = addr_list.get(adpt.getPhysicalAddress().toString());
		Enumeration keys = addr_list.keys();
		while(keys.hasMoreElements()) {
			Object key = keys.nextElement();
			if(addr_list.get(key) == target) {
				addr_list.remove(key);
			}
		}
			
	}

}
