package lotos;

import hypercast.HyperCastStatsException;
import hypercast.I_LogicalAddress;
import hypercast.I_OverlaySocket;
import hypercast.I_PhysicalAddress;
import hypercast.SimpleStats;
import hypercast.util.XmlUtil;
import hypercast.StatsProcessor;

import java.awt.Color;
import java.io.CharArrayWriter;
import java.io.PrintWriter;
import java.util.Enumeration;
import java.util.LinkedList;
import java.util.Vector;

import org.apache.xpath.XPath;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import lotos.event.*;
import lotos.history.NotificationHistoryEntry;
import lotos.gateway.LotosGateway;

/**
 * This class implements a Protocol containing
 * a Hypercast Node.
 * 
 * @author Guangyu Dong, Feb 3, 2004
 */
public class Protocol implements I_ProtocolInterface{
	
	private static int MAXIMUM_SIZE_OF_EVENT_HISTORY = 300;
	
	/** the agent for the hypercast protocol */
	I_SimHyperCastAgent agent;
	
	public LinkedList eventList;
	String protocolName;
	
	String securitySetting = "None";
	
	/** the I_Node object to run the protocol */
	I_OverlaySocket hc_socket;
	
	/** the port numbers of the node and socket */
	int nodePort;
	int socketPort;
	
	/** the physical node */
	I_PhysicalInterface phy;

	/** the virtual adapter */
	public LotosAdapter nodeAdapter;
	public LotosAdapter scktAdapter;
	/** if started */
	boolean joined=false;
	
	/** the network object */
	simHCNetwork network;
	
	
	/** the output of protocol */
	public CharArrayWriter outputBuf;
	PrintWriter output;
	
	/** an application running at the node */
	//public A_NodeApplication application;
	
	public String toString(){
		return this.getDetailedName();
	}
	
	/** neighbors in the overlay */
	Vector logical_neighbors; // logical neighbors
	/** color used to display edges in the overlay */
	Vector logical_colors; // the link color, display purpose
	/** the width of the logical edges */
	Vector logical_width; // the link width, display purpose;
	/** neighbors which are external nodes */
	public Vector external_neighbors;
	
	/**
	 * Constructor
	 * @param p			the physical node
	 * @param adapter	the virtual adapter
	 * @param node		the hypercast node
	 */
	public Protocol(simHCNetwork network, I_PhysicalInterface p, I_SimHyperCastAgent agent) {
		//lotosDebug.println("instantiating protocol");		
		phy = p;
		//setHCNode(node);
		
		logical_neighbors = new Vector();
		logical_colors = new Vector();
		logical_width = new Vector();
		external_neighbors = new Vector();
		
		this.agent = agent;
		this.network = network;
		
		eventList = new LinkedList();
		outputBuf = new CharArrayWriter();
		output = new PrintWriter(outputBuf);
		
		joined=false;
		initStats();
		
	}
	/**
	 * called by lotosadapter when it is created inside a hypercast node.
	 */

	/**
	 * test if the node is its logical neighbor
	 */
	boolean hasLogicalEdge(PhysicalNode n) {
		if (logical_neighbors.contains(n))
			return true;
		else
			return false;
	}

	/**
	 * @return all the logical neighbors of this node
	 */
	public Enumeration getLogicalNeighbor() {
		return logical_neighbors.elements();
	}

	/**
	 * @return the colors of the logical links
	 */
	public Enumeration getLogicalColor() {
		return logical_colors.elements();
	}

	/**
	 * @return the lengths of the logical links
	 */
	public Enumeration getLogicalWidth() {
		return logical_width.elements();
	}

	/**
	 * @return the total number of logical neighbors
	 */
	int getLogicalNeighborNumber() {
		return logical_neighbors.size();
	}
	/**
	 * clear all the logical edges
	 */
	public void clearLogicalEdges() {
		this.logical_colors.clear();
		this.logical_neighbors.clear();
		this.logical_width.clear();
		this.external_neighbors.clear();
	}

	//physicalInterface
	/**
	 * part of the physicalInterface implemenation
	 * @return the logical neighbors of this node
	 */
	public Enumeration logicalLinks() {
		return getLogicalNeighbor();
	}

	/**
	 * insert a logical edge at this node
	 * @param n the other end of the logical edge
	 * @param color the color of the edge, for display purpose
	 * @param width the width of the edge, for dispaly purpose
	 */
	public boolean insertLogicalEdge(PhysicalNode n, Color color, int width) {
		if (logical_neighbors.contains(n))
			return false;
		else {
			logical_neighbors.addElement(n);
			logical_colors.addElement(color);
			logical_width.addElement(new Integer(width));
		}
		return true;
	}

	/**
	 * remove a logical neighbor
	 */
	public boolean removeLogicalEdge(PhysicalNode n) {
		if (logical_neighbors.contains(n)) {
			int index = logical_neighbors.indexOf(n);
			logical_neighbors.remove(index);
			logical_colors.remove(index);
			logical_width.remove(index);
			return true;
		} else {
			return false;
		}
	}
	public void setNodeAdapter(LotosAdapter adpt) {
		nodeAdapter = adpt;
		/*
		 * register the node with the gateways. since the gateways share a nat table,
		 * only one modification needs to be done.
		 */
		/*
		if(network.gateway_nodes.size() > 0) {
			PhysicalNode n = (PhysicalNode)network.gateway_nodes.get(0);
			LotosGateway gateway = (LotosGateway)n.protocol;
			gateway.registerAddress((INETv4TwoPorts)adpt.getPhysicalAddress());
		}
		*/
	}
	public void setScktAdapter(LotosAdapter adpt) {
		scktAdapter = adpt;
		/*
		 * register the node with the gateways. since the gateways share a nat table,
		 * only one modification needs to be done.
		 */
		/*
		if(network.gateway_nodes.size() > 0) {
			PhysicalNode n = (PhysicalNode)network.gateway_nodes.get(0);
			LotosGateway gateway = (LotosGateway)n.protocol;
			gateway.registerAddress((INETv4TwoPorts)adpt.getPhysicalAddress());
		}
		*/
	}
	
	public void setHCNode(I_OverlaySocket node){
		
		hc_socket = node;
		statsPro.addStatsElement("Socket", hc_socket, 1, 1);
		/*
		if(hc_socket!=null) {
			hc_socket.setNotificationHandler(this);
			//phy.registerHCNode(node);
		}
		*/
	}
	
	public PrintWriter getOutput(){
		return output;
	}
	
	/**
	 * @return	the hypercast node
	 */
	public I_OverlaySocket getHCNode() {
		return hc_socket;
	}
	public I_LogicalAddress getParent(I_LogicalAddress la) {
		return hc_socket.getParent(la);
	}
	public I_LogicalAddress getLogicalAddress() {
		return hc_socket.getLogicalAddress();
	}
	
	/**
	 * @return	the adapter
	 */
	public LotosAdapter getNodeAdapter() {
		return nodeAdapter;
	}
	public LotosAdapter getScktAdapter() {
		return scktAdapter;
	}
	/* (non-Javadoc)
	 * @see protocolInterface#getName()
	 */
	public String getName() {
		String label;
		I_LogicalAddress laddr = hc_socket.getLogicalAddress();
		label = "";
		if(network.displayNodeProtocol){
			label += agent.getLabelForNode(hc_socket);
			if(network.displayNodeLogicalAddress){
				I_LogicalAddress addr = hc_socket.getLogicalAddress();
				if(addr!=null && nodeAdapter.getPhysicalAddress()!=null)
					label += ":";
			}
		}
		if(network.displayNodeLogicalAddress){
			if(laddr!=null)
				label +=laddr.toString();
		}
		if(network.displayNodePhysicalAddress){
			//label +="@"+nodeAdapter.createPhysicalAddress();
			label+="@"+nodeAdapter.getPhysicalAddress();
		}
		if(network.displaySecuritySetting && !securitySetting.equals("None")){
			label +=":" + securitySetting;
		}
		return label;
		
	}
	public String getDetailedName(){
		String label = "";
		
		label += agent.getLabelForNode(hc_socket);
		
		if(hc_socket.getLogicalAddress()!=null){
			label += ":";
			label +=hc_socket.getLogicalAddress().toString();
		}
		label +="@"+nodeAdapter.createPhysicalAddress();
		
		return label;
	}

	/* (non-Javadoc)
	 * @see protocolInterface#getColor()
	 */
	public Color getColor() {
		return agent.getColorForNode(hc_socket);
	}

	/* (non-Javadoc)
	 * @see protocolInterface#getShape()
	 */
	public int getShape() {
		return agent.getShapeForNode(hc_socket);
	}
	public void checkEventListSize(){
		while(eventList.size()>=MAXIMUM_SIZE_OF_EVENT_HISTORY){
			eventList.removeFirst();
		}
	}
	/*
	public void handler (final NOTIFICATION_EVENT event){
		checkEventListSize();
		this.eventList.add(new NotificationHistoryEntry(network.simTime(), event));
		updateLogicalLinks();
	}
	*/
	private void updateLogicalLinks () {
		//lotosDebug.println("Updating logical links at node "+adapter.getPhysicalAddress());
		//update logical links
		//clear all links
		clearLogicalEdges();
		//update links
		try {
			//lotosDebug.println("updating logical edges for "+adapter.getPhysicalAddress());
			I_LogicalAddress[] aps = hc_socket.getNeighbors();
			//lotosDebug.println("number of neighbors " + aps.length);
			//lotosDebug.println(aps.length + " neighbors");
			if (aps != null) {
				for (int i = 0; i < aps.length; i++) {
					//INETv4TwoPorts ha = (INETv4TwoPorts)aps[i].getPhysicalAddress();
					//if(aps[i].toString()!="Unknown")
					//lotosDebug.print("Neighbor: "+aps[i].toString());
					SimPhysicalAddress pa = (SimPhysicalAddress)network.getPhysicalAddress(aps[i]);
					/*
					if(pa==null)
						continue;
					*/
					//lotosDebug.println(" | "+pa.toString());
					
					PhysicalNode neighbor =	(PhysicalNode)LotosAdapterManager.searchAddress(pa);
					if (neighbor != null) {
						insertLogicalEdge(neighbor, Color.yellow, 2);
						//neighbor.getProtocol().insertLogicalEdge((PhysicalNode)phy,Color.yellow,2);
					} else {
						external_neighbors.add(pa);
						if(network.gateway_nodes.size() > 0)
							((LotosGateway)((PhysicalNode)network.gateway_nodes.get(0)).getProtocol()).fromLA = aps[i];
							
					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	/* (non-Javadoc)
	 * @see protocolInterface#handleTimer(int)
	 */
	public void handleEvent(SimEvent e) {
		if(!joined)return;
		SimEventTimer timer = (SimEventTimer)e;
		Object index = timer.getIndex();
		if(index instanceof NeighborsUpdateTimer){
			if(SimConfig.calcLogical)
				this.updateLogicalLinks();
			e.setTime(SimEventProcessor.getSimTime() + SimConfig.neighborsUpdateTime);
			SimEventProcessor.insertEvent(e);
		}

	}

	/* (non-Javadoc)
	 * @see protocolInterface#handleMsg(SimEventMsg)
	 */
	public void handleMsg(SimEventMsg msg) {
		if(!joined)return;
		//check if there is a source
		PhysicalNode src = msg.getSrc();
		if(src == null )return;
		//check if the protocol is the same
		/* doesn't matter if the protocol is different, hypercast can handle/ignore it.
		if(!(src.getProtocol() instanceof LotosGateway)) {
			Protocol srcProtocol = (Protocol)src.getProtocol();
			if(srcProtocol==null)return;
			
			if(!srcProtocol.agent.getName().equals(agent.getName()))return;
		}
		*/
		//passes it to the adapter of the hypercast node
		//lotosDebug.println("sending msg to adapter");
		if(msg.msgType == SimEventMsg.PROTOCOL_MESSAGE)
			nodeAdapter.handleEvent(msg);
		else if (msg.msgType == SimEventMsg.APPLICATION_MESSAGE)
			scktAdapter.handleEvent(msg);

	}

	/** the timer to update the logical links */
	class NeighborsUpdateTimer {
		public boolean equals(Object o){
			if(o==null)
				return false;
			
			return o instanceof NeighborsUpdateTimer;
		}
	}

	/* (non-Javadoc)
	 * @see protocolInterface#startProtocol()
	 */
	public void startProtocol() {
		if(joined)return;
		try{
			joined=true;
			if(hc_socket!=null){
				//lotosDebug.println("Hypercast socket joining overlay");
				hc_socket.joinOverlay();
				SimEventTimer timer = new SimEventTimer(SimEventProcessor.getSimTime() + SimConfig.neighborsUpdateTime, this, new NeighborsUpdateTimer());
				SimEventProcessor.insertEvent(timer);
				/*
				if(application!=null) {
					application.startAppl();
					statsPro.addStatsElement(application.getStatsName(), application, 1, 1);
				}
				*/
				//application.run();
			}
			
		}catch(Exception e){
			
		}
	}

	/* (non-Javadoc)
	 * @see protocolInterface#resetProtocol()
	 */
	public void restartProtocol() {
		if(!joined)return;
		try{
			clearLogicalEdges();
			if(hc_socket!=null)
				hc_socket.leaveOverlay();
			this.nodeAdapter.Reset(null);
			this.scktAdapter.Reset(null);
			this.eventList.clear();
			this.outputBuf.reset();
			/*
			if(this.application != null) {
				this.application.stopAppl();
				statsPro.removeStatsElement(application);
			}
			*/
			joined=false;
		}catch(Exception e){
			e.printStackTrace();
		}
	}
	/** permanently stops  the protocol */
	public void stopProtocol() {
		try {
			clearLogicalEdges();
			if(hc_socket!=null) {
				hc_socket.leaveOverlay();
				hc_socket.closeSocket();
			}
			hc_socket = null;
			nodeAdapter.Stop();
			scktAdapter.Stop();
			eventList.clear();
			outputBuf.reset();
			/*
			if(this.application != null) {
				this.application.stopAppl();
				statsPro.removeStatsElement(application);
			}
			*/
			joined = false;
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	String statsName="Protocol";
	StatsProcessor statsPro;
	
	private void initStats() {
		statsPro = new StatsProcessor(this/* I_Stats object */, true/* readable flag */, true/* writable flag */);
		statsPro.addStatsElement("Type", new stat_protocol(), 1, 1);
		statsPro.addStatsElement("Address", new stat_phyAddr(), 1, 1);

	}
	
	class stat_phyAddr extends SimpleStats{
		protected String getStats()
		{
			return nodeAdapter.getPhysicalAddress().toString();
		}

		protected String setStats(String newValue) throws HyperCastStatsException
		{
			throw new HyperCastStatsException(statisticsName, HyperCastStatsException.WriteToAReadOnlyValue);
		}

		public Element[] getReadSchema(Document doc, XPath xpath) throws HyperCastStatsException
		{
			return XmlUtil.createSchemaElement(doc/* document instance */, statisticsName/* element name */, "xsd:String"/* type */, 
												null/* restriction base */, null/* pattern value */);
		}
	}
	
	class stat_protocol extends SimpleStats{
		protected String getStats()
		{
			return protocolName;
		}

		protected String setStats(String newValue) throws HyperCastStatsException
		{
			throw new HyperCastStatsException(statisticsName, HyperCastStatsException.WriteToAReadOnlyValue);
		}

		public Element[] getReadSchema(Document doc, XPath xpath) throws HyperCastStatsException
		{
			return XmlUtil.createSchemaElement(doc/* document instance */, statisticsName/* element name */, "xsd:String"/* type */, 
												null/* restriction base */, null/* pattern value */);
		}
	}
	
	public String getStatsName() {
		return statsName;
	}
	public void setStatsName(String name) {
		statsName = name;
	}
	public Element[] getStats(Document doc, XPath xpath) throws HyperCastStatsException {
		return statsPro.getStatsResult(doc, xpath);
	}
	public Element[] setStats(Document doc, XPath xpath, Element newValue) throws HyperCastStatsException {
		return statsPro.setStatsResult(doc, xpath, newValue);
	}
	public Element[] getReadSchema(Document doc, XPath xpath) throws HyperCastStatsException {
		return statsPro.getReadSchemaResult(doc, xpath);
	}
	public Element[] getWriteSchema(Document doc, XPath xpath) throws HyperCastStatsException {
		return statsPro.getWriteSchemaResult(doc, xpath);
	}
	public I_PhysicalAddress getPhysicalAddress() {
		return nodeAdapter.createPhysicalAddress();
	}
	public LinkedList getEventList() {
		return eventList;
	}
	public Vector getLogicalNeighbors() {
		return logical_neighbors;
	}
	public Vector getExternalNeighbors() {
		return external_neighbors;
	}

}
