package lotos;

import hypercast.*;
import hypercast.util.XmlUtil;

import java.io.IOException;
import java.io.InputStream;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.LinkedList;
import java.io.*;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import lotos.history.*;


/**
 * This class parse the configuration file and create a list of agent class
 * @author Guangyu Dong
 *
 */
public class SimConfig{
	
	//Static configuration variables
	/** Calculate the logical links */
	public static boolean calcLogical = true;
	
//	the time for updating the logical links for the gui
	public static long neighborsUpdateTime = 1000000;
	
	/** required by the multicast gateway for choosing the right interface to listen on */
	public static InetAddress interfaceAddr;
	/** link-state based routing for wireless networks */
	public static boolean AD_HOC_ROUTING = false;
	
	public static boolean hybrid = false;
	
	/** configuration variables for wire links */
	/** propagation speed */
	public static float WIRE_PACKET_SPEED = 100000.0f;
	/** default bandwidth in bytes/second **/
    public final static int WIRE_DEFAULT_BANDWIDTH = 12500000;
    /** default buffer size in bytes */
    public final static int WIRE_DEFAULT_BUFFER = 1024*128;
    
    /** propagation Speed in m/s */
 	public static float WIRELESS_PACKET_SPEED = 1000.0f;
 	/** default bandwidth in bytes/second */
    public final static int WIRELESS_DEFAULT_BANDWIDTH = 1375000; //11 MBit/s
    /* Transmission Range in m */
 	public final static float WIRELESS_DEFAULT_SIGNAL_RANGE = 100f;
    /* Signal Sensing Range in m */
    public final static float WIRELESS_DEFAULT_SENSE_RANGE = 200f;
    
    /** time granularity of the simulation */
    public static long SIM_STEP = 100;
    public static int SIM_SLEEP = 100;
    
	/** Underlay routing */
	public static boolean forwarding=true;
	
	/** log file */
	public static LogFile log = null;
	 
	private LinkedList protocols;
	
	private LinkedList securitySettings;
	
	private LinkedList applications;
	
	private LinkedList gateways;
	
	private simHCNetwork network;
	
	public SimConfig(simHCNetwork network, String config_xml) throws ClassNotFoundException, InstantiationException, IllegalAccessException, IOException {
		this.network = network;
		protocols = new LinkedList();
		securitySettings = new LinkedList();
		applications = new LinkedList();
		gateways = new LinkedList();
		
		InputStream configInput = FileResourceManager.openFile(config_xml);
		
		/*
		try {
			configInput = new FileInputStream(config_xml);
		} catch (Exception e) {
			System.out.println("Couldn't find simulation config file in local directory, use the embedded config instead.");
			configInput = getClass().getResourceAsStream(config_xml);
		}
		*/
		parseConfig(XmlUtil.createDocument(configInput, config_xml));
		
	}
	public int getProtocolCount(){
		return protocols.size();
	}
	public int getSecuritySettingCount(){
		return this.securitySettings.size();
	}
	public int getApplicationCount(){
		return applications.size();
	}
	public int getGatewayCount() {
		return gateways.size();
	}
	public ProtocolEntry getProtocolByName(String protocolName){
		for(int i=0;i<protocols.size();i++){
			ProtocolEntry agent = (ProtocolEntry)protocols.get(i);
			if(agent.name.equals(protocolName))
				return agent;
		}
		return null;
	}
	public SecuritySettingEntry getSecuritySettingByName(String name){
		for(int i=0;i<securitySettings.size();i++){
			SecuritySettingEntry sse = (SecuritySettingEntry)securitySettings.get(i);
			if(sse.name.equals(name))
				return sse;
		}
		return null;
	}
	public ApplicationEntry getApplicationByName(String Name){
		for(int i=0;i<protocols.size();i++){
			ApplicationEntry agent = (ApplicationEntry)applications.get(i);
			if(agent.name.equals(Name))
				return agent;
		}
		return null;
	}
	public GatewayEntry getGatewayByName(String Name){
		for(int i=0;i<protocols.size();i++){
			GatewayEntry agent = (GatewayEntry)gateways.get(i);
			if(agent.name.equals(Name))
				return agent;
		}
		return null;
	}
	public String [] getProtocolNames(){
		String []names = new String[protocols.size()];
		for(int i=0;i<protocols.size();i++){
			ProtocolEntry entry = (ProtocolEntry)protocols.get(i);
			names[i] = entry.getName();
		}
		return names;
	}
	public String [] getSecuritySettingNames(){
		String []names = new String[securitySettings.size()];
		for(int i=0;i<securitySettings.size();i++){
			SecuritySettingEntry sse = (SecuritySettingEntry)securitySettings.get(i);
			names[i] = sse.name;
		}
		return names;
	}
	public String [] getApplicationNames(){
		String []names = new String[applications.size()];
		for(int i=0;i<applications.size();i++){
			ApplicationEntry entry = (ApplicationEntry)applications.get(i);
			names[i] = entry.getName();
		}
		return names;
	}
	public String [] getGatewayNames(){
		String []names = new String[gateways.size()];
		for(int i=0;i<gateways.size();i++){
			GatewayEntry entry = (GatewayEntry)gateways.get(i);
			names[i] = entry.getName();
		}
		return names;
	}
	private void parseConfig(Document config_doc) throws ClassNotFoundException, InstantiationException, IllegalAccessException{
		if(log!=null)
			log.endLog();
		
		Node simHypercast = config_doc.getElementsByTagName("SimHypercast").item(0);
		
		/** get the global lotos settings */
		Node lotosSets = ((Element)simHypercast).getElementsByTagName("LotosSettings").item(0);
		if(lotosSets!=null) {
			Node nHybrid = ((Element)lotosSets).getElementsByTagName("Hybrid").item(0);
			if(nHybrid!=null) {
				String mHybrid = nHybrid.getFirstChild().getNodeValue();
				SimConfig.hybrid = mHybrid.equalsIgnoreCase("true");
			}
			
			Node nMulticastAddr = ((Element)lotosSets).getElementsByTagName("MulticastAddress").item(0);
			if(nMulticastAddr!=null) {
				if(hybrid) {
					String mAddr = nMulticastAddr.getFirstChild().getFirstChild().getNodeValue();
					INETv4TwoPorts addr = new INETv4TwoPorts(mAddr);
					LotosAdapter.multicastAddress = new SimPhysicalAddress(addr);
				} else {
					String mAddr = nMulticastAddr.getFirstChild().getFirstChild().getNodeValue();
					LotosAdapter.multicastAddress = new SimPhysicalAddress(new Integer(mAddr).intValue());
				}
			}
			if(hybrid) {
				Node iface = ((Element)lotosSets).getElementsByTagName("InterfaceAddress").item(0);
				try {
					if(iface!=null) {
						interfaceAddr = InetAddress.getByName(iface.getFirstChild().getNodeValue());
					} else
						interfaceAddr = InetAddress.getLocalHost();
				} catch (UnknownHostException uhe) {
					uhe.printStackTrace();
				}
			}
			
			Node nAdHocRouting = ((Element)lotosSets).getElementsByTagName("AdHocRouting").item(0);
			if(nAdHocRouting != null) {
				String val = nAdHocRouting.getFirstChild().getNodeValue();
				SimConfig.AD_HOC_ROUTING = val.equalsIgnoreCase("true");
			}
			
			Node nSimStep =  ((Element)lotosSets).getElementsByTagName("SimStep").item(0);
			if(nSimStep != null) {
				String val = nSimStep.getFirstChild().getNodeValue();
				SimConfig.SIM_STEP = Long.valueOf(val).longValue();
			}
			
			Node nSimSleep =  ((Element)lotosSets).getElementsByTagName("SimSleep").item(0);
			if(nSimSleep != null) {
				String val = nSimSleep.getFirstChild().getNodeValue();
				SimConfig.SIM_SLEEP = Integer.valueOf(val).intValue();
			}
			Node nLogFile =  ((Element)lotosSets).getElementsByTagName("LogFileName").item(0);
			if(nLogFile != null) {
				String val = nLogFile.getFirstChild().getNodeValue();
				log = new LogFile(val);
			} else {
				log = null;
			}
				
		}
		
		/** get protocol configs */
		NodeList hypercastProtocols = ((Element)simHypercast).getElementsByTagName("HypercastProtocol");
		
		for(int i=0;i<hypercastProtocols.getLength();i++){
			String name = ((Element)hypercastProtocols.item(i)).getAttribute("name");
			Node agentClass = ((Element)hypercastProtocols.item(i)).getElementsByTagName("AgentClass").item(0);
			Node properties = ((Element)hypercastProtocols.item(i)).getElementsByTagName("Properties").item(0);
			Node configNode = properties.getChildNodes().item(0);
			String agentClassName = "lotos.agent."+agentClass.getChildNodes().item(0).getNodeValue();
						
			lotosDebug.println("Loading HyperCast protocol agent '"+agentClassName);
			
			Document hc_config_doc = XmlUtil.createDocument();
			Node newConfigNode = hc_config_doc.importNode(configNode, true);
			hc_config_doc.appendChild(newConfigNode);
			
			/* dynamically load agent class and set the config */
			Class dynamicAgentClass = Class.forName(agentClassName);
			//I_SimHyperCastAgent agentObject = (I_SimHyperCastAgent)dynamicAgentClass.newInstance();
			//agentObject.setNetwork(network);
			//ProtocolEntry entry = new ProtocolEntry(name, hc_config, agentObject);
			ProtocolEntry entry = new ProtocolEntry(name, hc_config_doc, dynamicAgentClass);
			protocols.add(entry);
		}
		
		/** get application settings */
		NodeList applSettings = ((Element)simHypercast).getElementsByTagName("NodeApplication");
		for(int i=0;i<applSettings.getLength();i++) {
			String name = ((Element)applSettings.item(i)).getAttribute("name");
			Node properties = ((Element)applSettings.item(i)).getElementsByTagName("Properties").item(0);
			Node applClass = ((Element)applSettings.item(i)).getElementsByTagName("ClassName").item(0);
			String applClassName = "lotos.nodeappl."+applClass.getChildNodes().item(0).getNodeValue();
			lotosDebug.println("Loading node application: "+applClassName);
			
			Document configDoc = XmlUtil.createDocument();
			Node newconfignode = configDoc.importNode(properties, true);
			configDoc.appendChild(newconfignode);
			
			Class dynamicApplClass = Class.forName(applClassName);
			ApplicationEntry entry = new ApplicationEntry(name, configDoc, dynamicApplClass);
			applications.add(entry);
			
		}
		
		/** get security configs */
		NodeList securitySettings = ((Element)simHypercast).getElementsByTagName("SecuritySetting");
		
		for(int i=0;i<securitySettings.getLength();i++){
			String name = ((Element)securitySettings.item(i)).getAttribute("name");
			Node properties = ((Element)securitySettings.item(i)).getElementsByTagName("Properties").item(0);
			NodeList privateProperties = ((Element)securitySettings.item(i)).getElementsByTagName("PrivateProperty");
			Node configNode = properties.getChildNodes().item(0);
			
			lotosDebug.println("Loading security setting '"+name+"'");
			
			Document hc_config_doc = XmlUtil.createDocument();
			Node newConfigNode = hc_config_doc.importNode(configNode, true);
			hc_config_doc.appendChild(newConfigNode);
			
			HyperCastConfig hc_config = new HyperCastConfig(hc_config_doc);
			
			for(int j=0;j<privateProperties.getLength();j++){
				String ppName = ((Element)privateProperties.item(j)).getAttribute("XPath");
				String ppValue = ((Element)privateProperties.item(j)).getAttribute("value");
				hc_config.setPrivateTextAttribute(XmlUtil.createXPath(ppName), ppValue);
				//lotosDebug.println(ppName + " = " + ppValue);
			}
			
			SecuritySettingEntry sse = new SecuritySettingEntry(name, hc_config);
						
			this.securitySettings.add(sse);
		}
		
		/**get the gateway adapter settings */
		NodeList adapterSettings = ((Element)simHypercast).getElementsByTagName("GatewayAdapter");
		for(int i=0;i<adapterSettings.getLength();i++) {
			String name = ((Element)adapterSettings.item(i)).getAttribute("name");
			lotosDebug.println("Loading gateway adapter setting: "+name);
			Node config = ((Element)adapterSettings.item(i).getFirstChild());
			Document configdoc = XmlUtil.createDocument();
			Node newConfigNode = configdoc.importNode(config, true);
			configdoc.appendChild(newConfigNode);
			//lotosDebug.println(config.toString());
			HyperCastConfig hcConfig = new HyperCastConfig(configdoc);
			GatewayEntry entry = new GatewayEntry(name, hcConfig);
			this.gateways.add(entry);
		}
		
	}
	
	
}

class SecuritySettingEntry{
	
	String name;
	HyperCastConfig config;
	
	public SecuritySettingEntry(String name, HyperCastConfig config){
		this.name = name;
		this.config = config;
	}
}

class ProtocolEntry{
	String name;
	Document config;
	Class agentClass;
	
	public ProtocolEntry(String name ,Document config, Class agent){
		this.name = name;
		this.config = config;
		this.agentClass = agent;
	}
	/**
	 * @return Returns the agent.
	 */
	public Class getAgentClass() {
		return agentClass;
	}
	/**
	 * @return Returns the config.
	 */
	public Document getConfig() {
		return config;
	}
	/**
	 * @return Returns the name.
	 */
	public String getName() {
		return name;
	}
	
}
class ApplicationEntry{
	String name;
	Document config;
	Class applClass;
	
	public ApplicationEntry(String name ,Document config, Class applclass){
		this.name = name;
		this.config = config;
		this.applClass = applclass;
	}
	/**
	 * @return Returns the application class.
	 */
	public Class getApplClass() {
		return applClass;
	}
	/**
	 * @return Returns the config.
	 */
	public Document getConfig() {
		return config;
	}
	/**
	 * @return Returns the name.
	 */
	public String getName() {
		return name;
	}
	
}
class GatewayEntry{
	String name;
	HyperCastConfig config;
	
	public GatewayEntry(String name ,HyperCastConfig config){
		this.name = name;
		this.config = config;
	}
	/**
	 * @return Returns the config.
	 */
	public HyperCastConfig getConfig() {
		return config;
	}
	/**
	 * @return Returns the name.
	 */
	public String getName() {
		return name;
	}
	
}

