package lotos;

import hypercast.*;
/**
 * This class implements a virtual physical address
 * used to run Hypercast Node in Lotos
 * 
 * @author Guangyu Dong, Feb 2, 2004
 */
public class SimPhysicalAddress implements I_PhysicalAddress, I_UnderlayAddress {
	/** the ID */
	public int ID;
	public INETv4TwoPorts inetAddr;
	static final int HC_ADDR_SIZE=4;
	
	/** 
	 * Constructor
	 * @param _id	the Id
	 * @param p		the physical node
	 */
	public SimPhysicalAddress(int _id){
		ID=_id;
	}
	public SimPhysicalAddress(INETv4TwoPorts addr) {
		inetAddr = addr;
	}
	public SimPhysicalAddress(INETv4OnePort addr) {
		inetAddr = new INETv4TwoPorts(addr.getInetAddress(), addr.getPort()-addr.getPort()%2, addr.getPort()-addr.getPort()%2+1);
	}
	public SimPhysicalAddress(String addr) {
		if(SimConfig.hybrid) {
			inetAddr = new INETv4TwoPorts(addr);
			if(inetAddr.getScktAdptPort()==0)
				inetAddr = new INETv4TwoPorts(inetAddr.getInetAddress(), inetAddr.getNodeAdptPort(), inetAddr.getNodeAdptPort()+1);
		}
		else
			ID = new Integer(addr).intValue();
		
	}
	
	public SimPhysicalAddress(byte []array, int offset){
		if(SimConfig.hybrid) {
			byte[] withoutShift = new byte[INETv4TwoPorts.ADDRESS_SIZE];
			System.arraycopy(array, offset, withoutShift, 0, INETv4TwoPorts.ADDRESS_SIZE);

			inetAddr = new INETv4TwoPorts(withoutShift);
		} else {
			ID=ByteArrayUtility.toInteger(array, offset);
		}
	}
	
	/**
	 * @return	string value
	 */
	public String toString(){
		if(SimConfig.hybrid) {
			return inetAddr.toString();
		} else {
			return ""+ID;
		}
	}
	
	/**
	 * @return	the physical node using this address
	 */
	/*
	public physicalInterface getPhyNode(){
		return phy;
	}
	*/
	/* (non-Javadoc)
	 * @see hypercast.I_Address#greaterThan(hypercast.I_Address)
	 */
	public boolean greaterThan(I_Address a) {
		if (a == null || !(a instanceof SimPhysicalAddress))
			throw new IllegalArgumentException("Attempted to compare LotosAddress with something of a different type!");
		if(SimConfig.hybrid)
			return inetAddr.greaterThan(((SimPhysicalAddress)a).inetAddr);
		else
			return ID > ((SimPhysicalAddress)a).ID;
	}

	/* (non-Javadoc)
	 * @see hypercast.I_Address#toByteArray()
	 */
	public byte[] toByteArray() {
		if(SimConfig.hybrid) {
			return inetAddr.toByteArray();
		} else {
			byte[] array = new byte[4];
			System.arraycopy(ByteArrayUtility.toByteArray(ID), 0, array, 0, 4);        
			return array;
		}
	}

	/* (non-Javadoc)
	 * @see hypercast.I_Address#getSize()
	 */
	public int getSize() {
		if(SimConfig.hybrid)
			return INETv4TwoPorts.ADDRESS_SIZE;
		else
			return 4;
	}
	/**
	 * compare two addresses
	 */
	public boolean equals(Object o){
		if(o instanceof INETv4TwoPorts) {
			return ((INETv4TwoPorts)o).equals(inetAddr);
		}else if(!(o instanceof SimPhysicalAddress))
			throw new IllegalArgumentException("Attempted to compare LotosAddress with something of a different type!");
		if(SimConfig.hybrid){
			return ((SimPhysicalAddress)o).inetAddr.equals(inetAddr);
		}else 
			return (ID == ((SimPhysicalAddress)o).ID);
		
	}
	/* (non-Javadoc)
	 * @see hypercast.I_PhysicalAddress#addressCheck(java.lang.Object)
	 */
	public boolean addressCheck(Object o) {
		return this.equals(o);
	}
		
	
	
}
