/***************************************************************************/
/*       Lotos, an extensible overlay network simulator                    */
/*       MNG Group                                                         */
/*       University of Virginia                                            */
/*       July, 2002                                                        */
/***************************************************************************/
package lotos;

import lotos.*; 
import lotos.event.SimEvent;
import lotos.event.SimEventMsg;

import java.util.*;
/*
 * A wireless physical layer implementation
 * 
 * @author Guangyu Dong, Jan 6, 2004
 */
public class WirelessPhysicalLayer implements I_PhysicalLayer
{
   
   /* the network */
   PhysicalNetwork net;
   
	/*
	 * Constructor
	 */
	WirelessPhysicalLayer(PhysicalNetwork olNet)
	{
		net = olNet;
	}
	/*
	 * @see PhysicalLayer#sendMsg(SimEventMsg)
	 */
	public boolean sendMsg(SimEventMsg msg)
	{
		PhysicalNode srcNode = msg.getSender();
		long time_start = net.simTime();
		//long time_end = time_start + (long)((msg.getSize()+getPreamble())*SimEvent.TIME_PER_SEC/PACKET_SPEED);
		long time_end = time_start+srcNode.wl_MAC.getSendTime(msg.getSize() + getPreamble());

		
		/* Detect collisions with previous transmissions from neighbors */
		{
			Vector neighbors = srcNode.getWirelessNeighbors();
			if(neighbors!=null){
	   	   		for(int i=0;i<neighbors.size();i++) {
		      	    PhysicalNode onode = (PhysicalNode) neighbors.elementAt(i);
					onode.wl_MAC.recordMessage(time_start, time_end);
		      	}
		    }
		    neighbors = srcNode.getSensingNeighbors();
			if(neighbors!=null){
	   	   		for(int i=0;i<neighbors.size();i++) {
		      	    PhysicalNode onode = (PhysicalNode) neighbors.elementAt(i);
					onode.wl_MAC.senseMessage(time_start, time_end);
		      	}
		    }
		
         	msg.setTime( time_end );
         	net.insertEvent(msg);
         	
         	
         	srcNode.wl_MAC.recordMessage(time_start, time_end);
         	
			
			return true;
		}
		
	}
	/*
	 * @return the preamble in Bytes
	 */
	int getPreamble(){
		return 0;	
	}
}
