/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/

package lotos.agent;

import hypercast.HyperCastConfig;
import hypercast.I_InterceptionCallback;
import hypercast.I_LogicalAddress;
import hypercast.I_MulticastAdapter;
import hypercast.I_Node;
import hypercast.I_OverlaySocket;
import hypercast.I_PhysicalAddress;
import hypercast.I_ReceiveCallback;
import hypercast.I_UnicastAdapter;
import hypercast.NotificationHandler;
import hypercast.BC.BC_Container_Node;
import hypercast.BC.BC_Node;
import hypercast.CT.CT_AddressPair;
import hypercast.CT.CT_Node;
import hypercast.util.XmlUtil;
import java.awt.Color;

import lotos.*;

/**
 * @author HyperCast Team
 * @author Wittawat Tantisiriroj
 * @version Jul. 03, 2005 
*/
public class BCAgent implements I_SimHyperCastAgent {

	simHCNetwork network;

	I_OverlaySocket socket;
	I_PhysicalAddress paddr;
	String prefix = "/Public/Node/BC_Container/";
	String type;

	/*
	 * (non-Javadoc)
	 * 
	 * @see I_SimHyperCastAgent#getName()
	 */
	public String getName() {
		return "BC_Container";
	}

	public I_OverlaySocket createSocket(HyperCastConfig config, String info, I_PhysicalAddress phyaddr, I_ReceiveCallback callback, NotificationHandler handler, I_InterceptionCallback intercept) {

		I_OverlaySocket sock = config.createOverlaySocket(callback, handler, intercept);
		try {
			type = config.getTextAttribute(XmlUtil.createXPath(prefix+"/ClusterNode/CT/Type"));
		}
		catch (Exception e){
			type = "";
		}
		paddr = phyaddr;
		socket = sock;

		//LotosAdapterManager.addressPair.put(laddr.toString(), paddr);
		
		return sock;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see I_SimHyperCastAgent#getLabelForNode(hypercast.I_Node)
	 */
	public String getLabelForNode(I_OverlaySocket node) {
		return "bc";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see I_SimHyperCastAgent#setNetwork(simHCNetwork)
	 */
	public void setNetwork(simHCNetwork network) {
		this.network = network;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see I_SimHyperCastAgent#getInfoForNode(hypercast.I_Node)
	 */
	public String getInfoForNode(I_OverlaySocket node) {
		return getLabelForNode(node) + ":"
				+ getNameForNode(node);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see I_SimHyperCastAgent#getNameForNode(hypercast.I_Node)
	 */
	public String getNameForNode(I_OverlaySocket node) {
		return node.getLogicalAddress().toString();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see I_SimHyperCastAgent#getMenuItems(java.lang.String)
	 */
	public String[] getMenuItems() {
		return new String[0];
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see I_SimHyperCastAgent#menuFired(java.lang.String)
	 */
	public void menuFired(String menu) {
		//no menu
		//do nothing
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see I_SimHyperCastAgent#getColorForNode(hypercast.I_Node)
	 */
	public Color getColorForNode(I_OverlaySocket node) {
		Color nodeColor;
		try {
			String state = node.getStats(XmlUtil.createDocument(), XmlUtil.createXPath("/Socket/Node/State"))[0].getFirstChild().getNodeValue();
			if(state.equals("Stopped"))
				nodeColor = Color.BLACK;
			else if(state.equals("Unknown"))
				nodeColor = Color.GRAY;
			else if(state.equals("MemberWithoutBBoneID"))
				nodeColor = Color.YELLOW;
			else if(state.equals("MemberWithBBoneID"))
				nodeColor = Color.ORANGE;
			else if(state.equals("Head"))
				nodeColor = Color.RED;
			else
				nodeColor = Color.BLACK;
		}
		catch (Exception e)
		{
			nodeColor = Color.BLACK;
		}
		return nodeColor;
	}


	/*
	 * (non-Javadoc)
	 * 
	 * @see I_SimHyperCastAgent#getShapeForNode(hypercast.I_Node)
	 */
	public int getShapeForNode(I_OverlaySocket node) {
		return I_ProtocolInterface.shapeCircle;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see I_SimHyperCastAgent#nodeRemoved(hypercast.I_Node)
	 */
	public void nodeRemoved(I_OverlaySocket node) {

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see I_SimHyperCastAgent#needServers()
	 */
	public boolean needServers(HyperCastConfig config) {
		return false;
	}

	public I_LogicalAddress getLA() {
		return socket.getLogicalAddress();
	}
	public I_PhysicalAddress getPA() {
		return paddr;
	}

	public void finishConfig(HyperCastConfig config, PhysicalNode node) {
		XYAddress coords = node.getAddress();
		String ModeStr = config.getTextAttribute(XmlUtil.createXPath(prefix
				+ "ClusterNode/CT/Type"));
		config.setTextAttribute(XmlUtil.createXPath(prefix+"ClusterNode/CT/Type/"+ModeStr+"/Criteria/Location/Coordinate"), ""+(float)coords.getX()/10+","+(float)coords.getY()/10);
	}
}