
package lotos.agent;

import lotos.*; 
import hypercast.HyperCastConfig;
import hypercast.I_Node;
import hypercast.I_PhysicalAddress;
import hypercast.I_UnicastAdapter;
import hypercast.DT.GNP_Landmark;
import hypercast.util.XmlUtil;

/**
 * This class tries to assign a DT address to a protocol node according to 
 * the configuration file 
 * @author Guangyu Dong
 *
 */
public class DTAddressManager {
	private simHCNetwork network;
	String prefix;
	
	GNP_Landmark landmarks[];
	I_PhysicalAddress landmarkAddrs[];
	
	public DTAddressManager(simHCNetwork network, String prefix){
		this.network = network;
		this.prefix = prefix;
	}
	public I_Node createServer(HyperCastConfig config, I_UnicastAdapter adapter){
		String coordMode = config.getTextAttribute(XmlUtil.createXPath(prefix +"Coords"));
		
		if(!coordMode.equals("USE_LM"))
			return null;
		
		if(landmarks==null){
			int numOfLandmark = Integer.parseInt(config.getTextAttribute(XmlUtil.createXPath(prefix +"Coords/USE_LM/LandmarkNum")));
			landmarks = new GNP_Landmark[numOfLandmark];
			landmarkAddrs = new I_PhysicalAddress[numOfLandmark];
		}
		
		GNP_Landmark newLandmark = null;
		int index = -1;
		
		for(int i=0;i<landmarks.length;i++){
			if(landmarks[i]==null){
				index = i;
				break;
			}
		}
		
		if(index>=0){
		
			newLandmark = new GNP_Landmark(config, adapter, prefix);
			landmarks[index] = newLandmark;
			landmarkAddrs[index] = adapter.createPhysicalAddress();
		}
		
		return newLandmark;
	}
	public void processConfig(XYAddress coords, HyperCastConfig config){
		String coordMode = config.getTextAttribute(XmlUtil.createXPath(prefix +"Coords"));
		
		if(coordMode.equals("FIXED")){
			config.setTextAttribute(XmlUtil.createXPath(prefix+"Coords/FIXED/coordinate"), ""+coords.getX()+","+coords.getY());
		
		}else{
			//TODO
		}
	}
	public boolean needServers(HyperCastConfig config){
		
		String coordMode = config.getTextAttribute(XmlUtil.createXPath(prefix +"Coords"));
		
		if(coordMode.equals("USE_LM")){
			if(landmarks==null){
				int numOfLandmark = Integer.parseInt(config.getTextAttribute(XmlUtil.createXPath(prefix +"Coords/USE_LM/LandmarkNum")));
				landmarks = new GNP_Landmark[numOfLandmark];
				landmarkAddrs = new I_PhysicalAddress[numOfLandmark];
			}
			for(int i=0;i<landmarks.length;i++){
				if(landmarks[i] == null)
					return true;
			}
		}
		return false;
	}
	public void nodeRemoved(I_Node node){
		for(int i=0;i<landmarks.length;i++){
			if(node == landmarks[i])
				landmarks[i] =null;
		}
	}
	
	
}
