package lotos.agent;

import lotos.*; 
import hypercast.*;
import hypercast.DT.DT_LogicalAddress;
import hypercast.Pastry.Pastry_LogicalAddress;
import hypercast.Pastry.Pastry_Node_BuddyList;
import hypercast.SPT.SPT_Node;
import hypercast.events.NODE_ISSTABLE;
import hypercast.events.NODE_LEAVEOVERLAY;
import hypercast.events.NODE_LOGICALADDRESSCHANGED;
import hypercast.events.NODE_NEIGHBORHOODCHANGED;
import hypercast.events.NOTIFICATION_EVENT;
import hypercast.util.XmlUtil;

import java.awt.Color;
import java.util.StringTokenizer;

import javax.xml.transform.TransformerException;


public class PastryAgent implements I_SimHyperCastAgent {

	simHCNetwork network;
	//I_LogicalAddress laddr;
	I_OverlaySocket socket;
	I_PhysicalAddress paddr;
	String prefix = "/Public/Node/Pastry/";
	
	/* (non-Javadoc)
	 * @see I_SimHyperCastAgent#getName()
	 */
	public String getName() {
		return "Pastry";
	}
	/* (non-Javadoc)
	 * @see I_SimHyperCastAgent#getLabelForNode(hypercast.I_OverlaySocket)
	 */
	public String getLabelForNode(I_OverlaySocket node) {
		return "pastry";
	}
	/* (non-Javadoc)
	 * @see I_SimHyperCastAgent#setNetwork(simHCNetwork)
	 */
	public void setNetwork(simHCNetwork network) {
		this.network = network;
	}
	public I_OverlaySocket createSocket(HyperCastConfig config, String info, I_PhysicalAddress phyaddr, I_ReceiveCallback callback, NotificationHandler handler, I_InterceptionCallback intercept) {
		I_OverlaySocket sock = config.createOverlaySocket(callback, handler, intercept);
		//laddr = sock.getLogicalAddress();
		paddr = phyaddr;
		socket = sock;
		
		//LotosAdapterManager.addressPair.put(laddr.toString(), paddr);
		
		return sock;
	}
	
	/* (non-Javadoc)
	 * @see I_SimHyperCastAgent#getInfoForNode(hypercast.I_OverlaySocket)
	 */
	public String getInfoForNode(I_OverlaySocket node) {
		String laStr = node.getLogicalAddress().toString();
		return this.getName()+":"+laStr.substring(3, laStr.length()-1);
	}
	/* (non-Javadoc)
	 * @see I_SimHyperCastAgent#getMenuItems(java.lang.String)
	 */
	public String[] getMenuItems() {
		return new String[0];
	}

	/* (non-Javadoc)
	 * @see I_SimHyperCastAgent#menuFired(java.lang.String)
	 */
	public void menuFired(String menu) {
		//no menu
		//do nothing
	}

	/* (non-Javadoc)
	 * @see I_SimHyperCastAgent#getColorForNode(hypercast.I_OverlaySocket)
	 */
	public Color getColorForNode(I_OverlaySocket node) {
		
		String stateString = ""+Pastry_Node_BuddyList.STATE_STOPPED;
		try {
			stateString = XmlUtil.getScalarStatsResult(node, "/Socket/Node/State");
		} catch (HyperCastStatsException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (TransformerException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		int state = Integer.parseInt(stateString);
		Color color;
		
		switch (state)
        {
            case Pastry_Node_BuddyList.STATE_STOPPED:
                color = Color.lightGray;
                break;
            case Pastry_Node_BuddyList.STATE_BUDDY_SEARCH:
                color = Color.red;
                break;
            case Pastry_Node_BuddyList.STATE_BUDDY_FOUND:
                color = Color.pink;
                break;
            case Pastry_Node_BuddyList.STATE_JOIN_INITIATED:
                color = Color.orange;
                break;
            case Pastry_Node_BuddyList.STATE_JOINED:
                color = Color.yellow;
                break;
            default:
                // TODO: error?
            	color = Color.GRAY;
        }
		return color;
	}

	/* (non-Javadoc)
	 * @see I_SimHyperCastAgent#getShapeForNode(hypercast.I_OverlaySocket)
	 */
	public int getShapeForNode(I_OverlaySocket node) {
		return I_ProtocolInterface.shapeCircle;
	}

	/* (non-Javadoc)
	 * @see I_SimHyperCastAgent#nodeRemoved(hypercast.I_OverlaySocket)
	 */
	public void nodeRemoved(I_OverlaySocket node) {
		
	}


	/* (non-Javadoc)
	 * @see I_SimHyperCastAgent#needServers()
	 */
	public boolean needServers(HyperCastConfig config) {
		return false;
	}
	
	public I_LogicalAddress getLA() {
		return socket.getLogicalAddress();
	}
	
	public I_PhysicalAddress getPA() {
		return paddr;
	}
	public void finishConfig(HyperCastConfig config, PhysicalNode node) {
		// TODO Auto-generated method stub
		
	}
}
