package lotos.agent;

import lotos.*; 
import hypercast.*;
import hypercast.SPT.SPT_LogicalAddress;
import hypercast.SPT.SPT_Node;
import hypercast.events.NODE_ISSTABLE;
import hypercast.events.NODE_LEAVEOVERLAY;
import hypercast.events.NODE_LOGICALADDRESSCHANGED;
import hypercast.events.NODE_NEIGHBORHOODCHANGED;
import hypercast.events.NOTIFICATION_EVENT;
import hypercast.util.XmlUtil;

import java.awt.Color;
import java.util.StringTokenizer;


public class SPTAgent implements I_SimHyperCastAgent {

	simHCNetwork network;

	I_OverlaySocket socket;
	I_PhysicalAddress paddr;
	String prefix = "/Public/Node/SPT/";
	
	/* (non-Javadoc)
	 * @see I_SimHyperCastAgent#getName()
	 */
	public String getName() {
		return "SPT";
	}
	
	public I_OverlaySocket createSocket(HyperCastConfig config, String info, I_PhysicalAddress phyaddr, I_ReceiveCallback callback, NotificationHandler handler, I_InterceptionCallback intercept) {
		I_OverlaySocket sock = config.createOverlaySocket(callback, handler, intercept);
		paddr = phyaddr;
		socket = sock;

		//LotosAdapterManager.addressPair.put(laddr.toString(), paddr);
		
		return sock;
	}
	/* (non-Javadoc)
	 * @see I_SimHyperCastAgent#getLabelForNode(hypercast.I_OverlaySocket)
	 */
	public String getLabelForNode(I_OverlaySocket node) {
		return "spt";
	}
	/* (non-Javadoc)
	 * @see I_SimHyperCastAgent#setNetwork(simHCNetwork)
	 */
	public void setNetwork(simHCNetwork network) {
		this.network = network;
	}

	/* (non-Javadoc)
	 * @see I_SimHyperCastAgent#getInfoForNode(hypercast.I_OverlaySocket)
	 */
	public String getInfoForNode(I_OverlaySocket node) {
		return "spt:"+node.getLogicalAddress();
	}

	/* (non-Javadoc)
	 * @see I_SimHyperCastAgent#getNameForNode(hypercast.I_OverlaySocket)
	 */
	public String getNameForNode(I_OverlaySocket node) {
		return node.getLogicalAddress().toString();
	}

	/* (non-Javadoc)
	 * @see I_SimHyperCastAgent#getMenuItems(java.lang.String)
	 */
	public String[] getMenuItems() {
		return new String[0];
	}

	/* (non-Javadoc)
	 * @see I_SimHyperCastAgent#menuFired(java.lang.String)
	 */
	public void menuFired(String menu) {
		//no menu
		//do nothing
	}

	/* (non-Javadoc)
	 * @see I_SimHyperCastAgent#getColorForNode(hypercast.I_OverlaySocket)
	 */
	public Color getColorForNode(I_OverlaySocket node) {
		try {
			String root = node.getStats(XmlUtil.createDocument(), XmlUtil.createXPath("/Socket/Node/Root"))[0].getFirstChild().getNodeValue();
			if(root.equals(node.getLogicalAddress().toString()))
				return Color.red;
		} catch (Exception e) {}
		return Color.orange;
	}

	/* (non-Javadoc)
	 * @see I_SimHyperCastAgent#getShapeForNode(hypercast.I_OverlaySocket)
	 */
	public int getShapeForNode(I_OverlaySocket node) {
		return I_ProtocolInterface.shapeCircle;
	}

	/* (non-Javadoc)
	 * @see I_SimHyperCastAgent#nodeRemoved(hypercast.I_OverlaySocket)
	 */
	public void nodeRemoved(I_OverlaySocket node) {
		
	}

	/* (non-Javadoc)
	 * @see I_SimHyperCastAgent#needServers()
	 */
	public boolean needServers(HyperCastConfig config) {
		return false;
	}
	
	public I_LogicalAddress getLA() {
		return socket.getLogicalAddress();
	}
	public I_PhysicalAddress getPA() {
		return paddr;
	}

	public void finishConfig(HyperCastConfig config, PhysicalNode node) {
		// TODO Auto-generated method stub
		
	}
}
