/***************************************************************************/
/*       Lotos, an extensible overlay network simulator                    */
/*       MNG Group                                                         */
/*       University of Virginia                                            */
/*       July, 2002                                                        */
/***************************************************************************/
package lotos.agent;

import lotos.*; 

import java.lang.Math;
import java.io.*;
import java.util.*;
/**
 * 
 * This class encapsulates the algorithm of Spanning Tree by hiding
 * the details of the algorithm and providing a simple interface for the outside
 * classes to invoke the algorithm
 * 
 * @author Guangyu Dong, Jan 7, 2004
 */
class SpanningTree{
	static int []roots=null;
	/**
	 * @return	the root list as an result of the last calculation
	 */
	static int [] getLastRootList(){
		return roots;
	}
	/**
	 * Calculate the spanning tree with regular algorithm
	 * @param spt_ids	a list of SPT Ids
	 * @param distances	the distances between each two nodes
	 * @param range		wireless transmission range
	 * @return	a list edges
	 */
	static DTEdge [] RegularSPT(int []spt_ids, float [][]distances){
		roots=new int[spt_ids.length];
		DTEdge [] edges=new DTEdge[spt_ids.length-1];
		roots=new int[spt_ids.length];
		int root_cnt=0;
		
		int remained=spt_ids.length;
		int edge_cnt=0;
		boolean handled[]=new boolean[spt_ids.length];
		int cost[]=new int[spt_ids.length];
		
		for(int i=0;i<handled.length;i++)
			handled[i]=false;
			
		while(remained>0){
			//find the minumum id
			int min_id=-1, min_index=-1;
			for(int i=0;i<spt_ids.length;i++){
				if(!handled[i] && (spt_ids[i]<min_id||min_id<0)){
					min_id=spt_ids[i];
					min_index=i;
				}
			}
			lotosDebug.println("Spanning Tree Root "+min_id);
			roots[root_cnt++]=min_index;
			
			handled[min_index]=true;
			cost[min_index]=0;
			remained--;
			
			//build the tree rooted at the minimum id
			int current_cost=0;
			while(true){
				boolean new_edge=false;
				for(int i=0;i<spt_ids.length;i++){
					if(!handled[i])
					{
						//search a parent
						int parent_index=-1;
						int parent_id=-1;
						int parent_cost=-1;
						for(int j=0;j<spt_ids.length;j++){
							if(handled[j] && i!=j && distances[i][j]>0 && cost[j]<=current_cost){
								//find a possible parent
								//lotosDebug.println(""+spt_ids[i]+"finds a possible parent "+spt_ids[j]+" cost="+cost[j]);
								if(parent_index<0 || (parent_cost>cost[j]) ||((parent_cost==cost[j]) && (parent_id>spt_ids[j]) )){
									//find a better parent
									//lotosDebug.println("it's better");
									parent_index=j;
									parent_id=spt_ids[j];
									parent_cost=cost[j];
								}
									
							}
						}
						if(parent_index>=0){
							//find a parent
							//lotosDebug.println("edge "+spt_ids[i]+","+spt_ids[parent_index]);
							edges[edge_cnt++]=new DTEdge(i,parent_index);
							handled[i]=true;
							cost[i]=parent_cost+1;
							remained--;
							new_edge=true;
							
						}
					}
				}
				if(!new_edge){break;}
				current_cost++;
				//lotosDebug.println("current_cost="+current_cost);					
			}
		}
		for(int i=(edge_cnt);i<edges.length;i++)
			edges[i]=new DTEdge(-1,-1);
		for(int i=root_cnt;i<roots.length;i++)
			roots[i]=-1;
		
		return edges;
		
	}
	
	
}
