
package lotos.gateway;

import hypercast.*;
import hypercast.util.XmlUtil;

import java.awt.Color;
import java.net.*;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.Vector;
import java.io.*;
import java.util.*;

import org.apache.xpath.XPath;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import lotos.*;
import lotos.event.*;
import lotos.history.MessageHistoryEntry;


/**
 * LotosGateway is a virtual NAT for LoToS. It sits on a physical node like a normal
 * protocol. Messages which are routed to it and are either multicast or destined for
 * an IP address outside of lotos, are forwarded to the destination
 * through a UDP datagram socket mapped to the source physical node.
 * 
 * @author Zhi An
 * @version 1 - 16/05/06
 *
 */

public class LotosGateway implements I_ProtocolInterface {
	/**The port where the LotosGateways starts assigning ports for overlay nodes. it starts
	 * at this port and increments by 2's (for future socket adapter port support).
	 */
	public final static int STARTING_PORT = 5000;
	
	/**
	 * a hashtable mapping a lotos private IP address to a thread that reads
	 * the socket, which is a blocking method. the thread class encapsulates
	 * the datagram socket. the thread class handles the reverse traversal.
	 * all the gateways share the same set of real sockets and mappings.
	 */
	public static Hashtable NAT;
	/**
	 * the reverse NAT contains lotos addresses with gateway ports as keys.
	 */
	public static Hashtable revNAT;
	
	public final static Color COLOR = Color.WHITE;
	public final static Color EXTERNAL_EDGE_COLOR = Color.GREEN;
	
	public String name = "LoToS Gateway";
	
	/** log to prevent the gateway to pick up the multicast messages that it sends */
	//LinkedList msgLog;
	//final static int MAX_LOG_SIZE = 30;
	LinkedList eventList;
	final static int MAXIMUM_SIZE_OF_EVENT_HISTORY = 300;
	
	/**
	 * the gateway agent translates messages through the NAT.
	 */
	//public I_GatewayAgent translator;
	/** thread for reading a multicast socket */
	MulticastThread multiNode;
	MulticastThread multiSckt;
	
	/** data about the simulation */
	PhysicalNetwork lotosNet;
	PhysicalNode lotosNode;
	LotosAdapterManager manager;
	
	public I_NetworkAddress fromPA;
	public I_LogicalAddress fromLA;
	
	/** the type of external adapters contained in the gateway */
	public String type;
	
	HyperCastConfig adapterConfig;
	
	public LotosGateway(PhysicalNetwork net, PhysicalNode node, LotosAdapterManager mngr, HyperCastConfig settings, String adptType) {
		lotosNet = net;
		lotosNode = node;
		manager = mngr;
		if(NAT == null)
			NAT = new Hashtable();
		if(revNAT == null)
		revNAT = new Hashtable();

		type = adptType;
		//msgLog = new LinkedList();
		eventList = new LinkedList();
		
		adapterConfig = settings;
		
		for(int i=0;i<lotosNet.active_nodes.size();i++) {
			registerAdapters(((PhysicalNode)lotosNet.active_nodes.get(i)).getProtocol());
		}
		
		/*
		Enumeration addrs = mngr.addr_list.keys();
		while(addrs.hasMoreElements()) {
			registerAddress(new INETv4TwoPorts((String)addrs.nextElement()));
		}
		*/
		multiNode = new MulticastThread(true);
		multiNode.start();
		multiSckt = new MulticastThread(false);
		multiSckt.start();
		
		initStats();
		
	}
	
	/**
	 * register a lotos address with the gateway to bind an outside socket and port.
	 * when the overlay node is destroyed, closeAddress should be called to close the
	 * socket.
	 * @param addr the lotos address of the node
	 * @return
	 */
	/*
	public boolean registerAddress(INETv4TwoPorts addr) {
		INETv4OnePort nodeAddr = new INETv4OnePort(addr.getInetAddress(), addr.getNodeAdptPort());
		INETv4OnePort scktAddr = new INETv4OnePort(addr.getInetAddress(), addr.getScktAdptPort());
		if(!NAT.containsKey(nodeAddr) && !NAT.containsKey(scktAddr)) {	
			SocketThread newSock = new SocketThread(nodeAddr);
			newSock.start();
			NAT.put(nodeAddr, newSock);
			//lotosDebug.println("reg "+addr.toString());
			while(!newSock.active);		
			revNAT.put(new Integer(newSock.sock.getLocalPort()), nodeAddr);
	
			newSock = new SocketThread(scktAddr);
			newSock.start();
			NAT.put(scktAddr, newSock);
			//lotosDebug.println("reg "+addr.toString());
			while(!newSock.active);		
			revNAT.put(new Integer(newSock.sock.getLocalPort()), scktAddr);
			return true;
						
		}
		return false;
	}
	*/
	public boolean registerAdapters(I_ProtocolInterface prot) {
		if(NAT.containsKey(((SimPhysicalAddress)prot.getNodeAdapter().getPhysicalAddress()).toString()))
			return false;
		lotosDebug.println("creating adapters for "+prot.getScktAdapter().createPhysicalAddress().toString());
		AdapterWrapper hcScktAdapter = new AdapterWrapper(null, prot.getScktAdapter(), adapterConfig);
		AdapterWrapper hcNodeAdapter = new AdapterWrapper(hcScktAdapter.adapter, prot.getNodeAdapter(), adapterConfig);
		WrapWrapper wrap = new WrapWrapper();
		wrap.node = hcNodeAdapter;
		wrap.socket = hcScktAdapter;
		NAT.put(prot.getScktAdapter().createPhysicalAddress().toString(), wrap);
		
		return true;
	}
	/**
	 * remove the binding of a lotos address and real outside socket.
	 * @param addr the lotos address of the node
	 * @return
	 */
	public boolean closeAddress(SimPhysicalAddress paddr) {
		INETv4TwoPorts addr = paddr.inetAddr;
		lotosDebug.println("close " + addr.toString());
		if(NAT.containsKey(addr.toString())) {
			
			WrapWrapper aWrapper = (WrapWrapper)NAT.get(addr.toString());
			aWrapper.node.close();
			aWrapper.socket.close();
			NAT.remove(addr.toString());
			return true;
		}
		return false;
	}
	/**
	 * resets the gateway when the user resets the simulation. this closes
	 * all the addresses bound to the gateway
	 *
	 */
	public void reset() {
		lotosDebug.println("resetting gateway");
		
		Enumeration keys = NAT.keys();
		while(keys.hasMoreElements()) {
			Object key = keys.nextElement();
			WrapWrapper aWrapper = (WrapWrapper)NAT.get(key);
			lotosDebug.println("Closing gateway address " + key.toString());
			aWrapper.node.close();
			aWrapper.socket.close();
		}
		fromPA = null;
		fromLA = null;
		multiNode.close();
		multiSckt.close();
		
		NAT.clear();
		revNAT.clear();
		//msgLog.clear();
	}
	public String getName() {
		String val = name;
		if(fromLA!=null)
			val += "/"+fromLA.toString();
		if(fromPA!=null)
			val += "@"+fromPA.toString();
		return val;
	}
	public String getDetailedName() {
		return getName();
	}
	public Color getColor() {
		return COLOR;
	}

	public int getShape() {
		
		return shapeTriangle;
	}
	public LotosAdapter getNodeAdapter() {
		return null;
	}
	public LotosAdapter getScktAdapter() {
		return null;
	}
	public void setNodeAdapter(LotosAdapter ad) {
		
	}
	public void setScktAdapter(LotosAdapter ad) {
		
	}
	public I_LogicalAddress getParent(I_LogicalAddress la) {
		return null;
	}
    public I_LogicalAddress getLogicalAddress() {
    	return null;
    }
    
    
	/**
	 * handle message takes a message, passes it through the gateway agent which translates the
	 * message for outside lotos, and sends the message using the socket bound to the message's
	 * source.
	 */
	public void handleMsg(SimEventMsg msg) {
			//System.out.print("a");
		//if(msg.dst != null) {
			INETv4TwoPorts srcaddr = msg.srcaddr.inetAddr;
			INETv4TwoPorts dstaddr = msg.dstaddr.inetAddr;
			
			//check if the message is destined for outside lotos
			if(LotosAdapterManager.searchAddress(msg.dstaddr) != null)
				return;
			if(!NAT.containsKey(srcaddr.toString())) {
				lotosDebug.errorln("LotosGateway: could not locate external socket");
				return;
			}
			byte[] bytes = msg.hc_message.toByteArray();
			//System.out.print("b");
			//lotosDebug.println("LotosGateway: sending external message from "+srcaddr+" for "+dstaddr+" size:"+bytes.length);
			/*
			for(int i=0;i<bytes.length;i++)
				lotosDebug.print(ByteArrayUtility.toUnsignedShort(bytes, i)+" ");
			lotosDebug.println();
			*/
			//SocketThread sock = (SocketThread)NAT.get(srcaddr);
			//bytes = translator.outgoing(bytes, NAT, sock.sock.getLocalAddress());
			/*
			for(int i=0;i<bytes.length;i++)
				lotosDebug.print(ByteArrayUtility.toUnsignedShort(bytes, i)+" ");
			lotosDebug.println();
			*/
			//sock.transmit(bytes, dstaddr);
			if(dstaddr.getInetAddress().isMulticastAddress()) {
				multiNode.send(dstaddr.getInetAddress(), msg.hc_message);
			} else {
				WrapWrapper wrap = (WrapWrapper)NAT.get(srcaddr.toString());
				//if(!dstaddr.getInetAddress().isMulticastAddress())
				//lotosDebug.println("dst: "+dstaddr.toString());
				if(msg instanceof SimEventProtocolMsg) {
					INETv4OnePort dst = new INETv4OnePort(dstaddr.getInetAddress(), dstaddr.getNodeAdptPort());
					wrap.node.sendUnicastMessage(dst, msg.hc_message);
				} else {
					INETv4OnePort dst = new INETv4OnePort(dstaddr.getInetAddress(), dstaddr.getScktAdptPort());
					wrap.socket.sendUnicastMessage(dst, msg.hc_message);
				}
			}
			//System.out.println("c");
			//else
				//aWrapper.sendMulticastMessage(msg.hc_message);
		//}
			checkEventListSize();
			eventList.add(new MessageHistoryEntry(SimEventProcessor.getSimTime(), true, msg, msg.sender.getProtocol().getName(), dstaddr.toString(), getDetailedName()));
			
	}

	public void startProtocol() {// this isn't a real protocol
	}
	public void restartProtocol() {// not a real protocol
	}
	public void stopProtocol() {
		
	}
	public void handleTimer(Object index) {	// this should never happen
	}
	public I_OverlaySocket getHCNode() { //not a real protocol
		return null;
	}
	public void setAdapter(LotosAdapter la) {
		
	}
	/*
	private void checkLogSize() {
		while(msgLog.size() > MAX_LOG_SIZE) {
			msgLog.removeFirst();
		}
	}
	private boolean inLog(byte[] msg) {
		ListIterator iterator = msgLog.listIterator();
		while(iterator.hasNext()) {
			byte[] log = (byte[])iterator.next();
			if(log.length != msg.length)
				continue;
			for(int i=0;i<log.length;i++)
				if(log[i]!=msg[i])
					continue;
			return true;
		}
		return false;
	}
	*/
	public class WrapWrapper {
		AdapterWrapper node;
		AdapterWrapper socket;
	}
	/**
	 * the adapterwrapper holds an adapter from hypercast. it handles the sending and recieving of messages,
	 * passing them to and from the adapter.
	 * @author anz
	 *
	 * TODO: add a sendbuffer and a thread to send the messages. the entire simulation blocks on the send, which
	 * could take a long time if the TCP adapter is used, and cannot make a connection.
	 */
	public class AdapterWrapper implements I_AdapterCallback {
		LotosAdapter target;
		I_UnicastAdapter adapter;
		LinkedList sendBuffer;
		boolean running;
		
		class sendEntry {
			I_NetworkAddress dst;
			I_Message msg;
			sendEntry(I_NetworkAddress dst, I_Message msg) {
				this.dst = dst;
				this.msg = msg;
			}			
		}
		/*
		 * In some adapters, like the TCP unicast adapter, opening a tcp connection causes blocking, which can
		 * last a long time, especially with longer timeouts. The main lotos thread which handles passing
		 * messages back and forth should not send messages using the adapters, which would pause the simulation
		 * for the duration of the sending/timeout. The messages should be buffered, and then sent to the adapters
		 * by a seperate thread.
		 */
		class sendThread implements Runnable{
			public void run() {
				while(running) {
					sendEntry entry;
					synchronized(sendBuffer) {
						while(sendBuffer.size()==0 && running) {
							try {sendBuffer.wait();} catch(InterruptedException ign) {}
						}
						if(!running)
							break;
						entry = (sendEntry)sendBuffer.removeFirst();
					}
					if(!running)
						break;
					if(entry==null)
						continue;
					adapter.sendUnicastMessage(entry.dst, entry.msg);
				}					
			}
		}
		
		
		public AdapterWrapper(I_UnicastAdapter scktAdpt, LotosAdapter dstAdapter, HyperCastConfig config) {
			lotosDebug.println("Creating wrapper for adapter " + dstAdapter.createUnderlayAddress());
			sendBuffer = new LinkedList();
			target = dstAdapter;
			AdapterFactory factory = new AdapterFactory(true);
			if(scktAdpt == null)
				adapter = factory.createAdapter(config, null, "SocketAdapter", null);
			else 
				adapter = factory.createAdapter(config, scktAdpt, "NodeAdapter", target.createPhysicalAddress().toString());
			
			adapter.setCallback(this);
			if(scktAdpt != null) {
				adapter.Start();
				scktAdpt.Start();
			}			
			running = true;
			new Thread(new sendThread(), "gateway sender").start();
			
		}
		
		public void sendUnicastMessage(I_NetworkAddress dst, I_Message msg) {
			synchronized(sendBuffer) {
				sendBuffer.add(new sendEntry(dst, msg));
				sendBuffer.notifyAll();
			}
			fromPA = dst;
		}
		public void sendMulticastMessage(I_Message msg) {
			//checkLogSize();
			//lotosDebug.println("send multicast");			
			//msgLog.add(msg.toByteArray());
			if(adapter instanceof I_MulticastAdapter)
				((I_MulticastAdapter)adapter).sendMulticastMessage(msg);
			else
				lotosDebug.errorln("LotosGateway: Attempted to send a multicast message through a unicast adapter");			
		}
	
		public I_Message restoreMessage (byte[] receiveBuffer, int[] validBytesStart, int validBytesEnd) {
			//use an active node's callback feature to handle the message restoration.
			//this only supports one type of protocol running at a time.
			if(lotosNet.active_nodes.size()<=0)
				return null;
			
			if(target.callback == null) {
				lotosDebug.error("LotosGateway: Could not locate receiving node");
				return null;
			}
			try {
				I_Message hcmsg = target.callback.restoreMessage(receiveBuffer, validBytesStart, validBytesEnd);
				return hcmsg;
			} catch(IllegalArgumentException e) {
				lotosDebug.println("LotosGateway: discarding corrupt message");
				lotosDebug.println(e.toString());
				return null;
			}
		}
		public void messageArrivedFromAdapter(I_Message hcmsg) {
			//if(inLog(hcmsg.toByteArray())) {
				//lotosDebug.println("echo message dropped");
			//	return;
			//}
			
			SimEventMsg message;
			if(target.socketAdpt != null)
				message = new SimEventProtocolMsg(lotosNode, null, hcmsg);
			else
				message = new SimEventApplMsg(lotosNode, null, hcmsg);
			message.dstaddr = (SimPhysicalAddress)target.createUnderlayAddress();
			message.srcaddr = null;
			message.dst = (PhysicalNode)target.getPhyNode();
			//how to seperate unicast (infinite hop limit) vs broadcast (limited hops)
			message.setTTL(lotosNet.multicast_hop_limit);
			
			bytesRec +=message.msgSize;			
			lotosNode.sendMsg(message);
			
			//lotosDebug.println("outside msg for "+target.createUnderlayAddress());
			
		}
		public void timerExpired (Object timerID) {
			
		}
		public void close() {
			running = false;
			synchronized(sendBuffer) {
				sendBuffer.notifyAll();
			}
			adapter.Stop();
			adapter.close();
		}
	}

	/**
	 * the multicast thread does reading from a multicast socket.
	 * the reason for a seperate multicast socket, rather than using the UDP_Multicast_Adapter's multicast send
	 * function: The adapters inside the lotos gateway will recieve messages sent other adapters inside the gateway.
	 * So in essence, the gateway becomes a relay. This is an unwanted behavior, so with multicast messages being sent
	 * and being received from one socket, it is possible to filter out these unwanted messages.
	 * @author anz
	 *
	 */
	public class MulticastThread extends Thread {
		public MulticastSocket sock;
		public INETv4TwoPorts addr;
		boolean active;
		public InetAddress group;
		/** true if node adapter port, false if socket adapter port */
		boolean node;
		
		MulticastThread(boolean isnode) {
			super("GatewayMulticast");
			
			try {
				group = LotosAdapter.multicastAddress.inetAddr.getInetAddress();
				node = isnode;
				sock = new MulticastSocket(isnode?LotosAdapter.multicastAddress.inetAddr.getNodeAdptPort():LotosAdapter.multicastAddress.inetAddr.getScktAdptPort());
				sock.setInterface(SimConfig.interfaceAddr);
				sock.setLoopbackMode(true);
				sock.joinGroup(group);
				lotosDebug.println("Creating multicast socket at " + LotosAdapter.multicastAddress.inetAddr);
			} catch (Exception e) {
				e.printStackTrace();
			}
			
			active = true;
			
		}
		public void send(InetAddress dst, I_Message msg) {
			byte[] bytes = msg.toByteArray();
			DatagramPacket packet = new DatagramPacket(bytes, bytes.length, group, sock.getLocalPort());
			try {
				sock.send(packet);
			} catch (IOException ioe) {
				lotosDebug.errorln("LotosGateway: could not transmit message");
			}
		}		
		public void run() {
			byte[] buf = new byte[1024*128];
			DatagramPacket packet = new DatagramPacket(buf, 1024*5);
			while(active) {
				try {
					sock.receive(packet);
					InetAddress senderaddr = packet.getAddress();
					int senderport = packet.getPort();
										
					INETv4TwoPorts sender = new INETv4TwoPorts(senderaddr, senderport-senderport%2, senderport-senderport%2+1);
					fromPA = sender;
					//lotosDebug.println(sender.toString());
					if(NAT.containsKey(sender.toString()))
						continue;
					byte[] msg = new byte[packet.getLength()];
					System.arraycopy(buf, 0, msg, 0, msg.length);
					
					//lotosDebug.errorln("LotosGateway: External multicast message recieved from "+sender+"  size:"+packet.getLength());
					
					/**
					 * since this is a multicast message, it doesn't have a definite destination.
					 * therefore, there is no good choice for which node to use to recompose the
					 * byte array back into a message. as such, if there are more than one protocol
					 * running in lotos, this will have to be changed.
					 */ 
					if(lotosNet.active_nodes.size()==0)
						continue;
					
					LotosAdapter target;
					
					if(node) {
						//lotosDebug.println("protocol message");
						target = ((PhysicalNode)lotosNet.active_nodes.get(0)).getProtocol().getNodeAdapter();
					}
					else
						target = ((PhysicalNode)lotosNet.active_nodes.get(0)).getProtocol().getScktAdapter();
					if(target.callback == null) {
						lotosDebug.errorln("LotosGateway: Could not locate receiving node");
						continue;
					}
					int[] validmsg = {0};
					try {
						
						I_Message hcmsg = target.callback.restoreMessage(msg, validmsg, msg.length);
						SimEventMsg message;
						if(node)
							message = new SimEventProtocolMsg(lotosNode, null, hcmsg);
						else
							message = new SimEventApplMsg(lotosNode, null, hcmsg);
						message.dstaddr = LotosAdapter.multicastAddress;
						message.srcaddr = new SimPhysicalAddress(sender);
						//lotosDebug.println("LotosGateway: relaying multicast message");
						lotosNode.sendMsg(message);
						bytesRec += msg.length;
						
						checkEventListSize();
						
						eventList.add(new MessageHistoryEntry(SimEventProcessor.getSimTime(), true, message, sender.toString(), "Broadcast", getDetailedName()));
						
					} catch(Exception e) {
						lotosDebug.errorln("LotosGateway: discarding corrupt message");
						lotosDebug.errorln(e.toString());
						continue;
					}
								
				} catch (IOException e) {
					/*
					 * if active is false, then the socket was closed, and this
					 * error was thrown to force the thread out of its block read.
					 */
					if(active)
						lotosDebug.errorln("LotosGateway: Error recieving packet");					
				}
			}
		}
		public void close() {
			/*
			try {
				sock.leaveGroup(group);
			} catch (Exception e) {
				e.printStackTrace();
			}
			*/
			active = false;
			/*
			 * turning active to false may not actually cause the socket thread to
			 * immediately exit, since the thread does block reading on the socket.
			 * so the socket is closed here, and the block reading exits with an
			 * IOException, thus allowing the thread to stop.
			 * 
			 */
			
			sock.close();
			
		}
	}
	
	String statsName="Gateway";
	StatsProcessor statsPro;
	long bytesSent;
	long bytesRec;
	
	private void initStats() {
		bytesSent = 0;
		bytesRec = 0;
		statsPro = new StatsProcessor(this/* I_Stats object */, true/* readable flag */, true/* writable flag */);
		statsPro.addStatsElement("BytesSent", new stat_bytesSent(), 1,1);
		statsPro.addStatsElement("BytesRecieved", new stat_bytesRec(), 1, 1);
		statsPro.addStatsElement("ExternalPorts", new stat_ports(), 1, 1);
	}
	class stat_bytesSent extends SimpleStats{
		protected String getStats()
		{
			return Long.toString(bytesSent);
		}

		protected String setStats(String newValue) throws HyperCastStatsException
		{
			throw new HyperCastStatsException(statisticsName, HyperCastStatsException.WriteToAReadOnlyValue);
		}

		public Element[] getReadSchema(Document doc, XPath xpath) throws HyperCastStatsException
		{
			return XmlUtil.createSchemaElement(doc/* document instance */, statisticsName/* element name */, "xsd:Long"/* type */, 
												null/* restriction base */, null/* pattern value */);
		}
	}
	class stat_bytesRec extends SimpleStats{
		protected String getStats()
		{
			return Long.toString(bytesRec);
		}

		protected String setStats(String newValue) throws HyperCastStatsException
		{
			throw new HyperCastStatsException(statisticsName, HyperCastStatsException.WriteToAReadOnlyValue);
		}

		public Element[] getReadSchema(Document doc, XPath xpath) throws HyperCastStatsException
		{
			return XmlUtil.createSchemaElement(doc/* document instance */, statisticsName/* element name */, "xsd:Long"/* type */, 
												null/* restriction base */, null/* pattern value */);
		}
	}
	class stat_ports extends SimpleStats{
		protected String getStats()
		{
			String ports ="";
			Enumeration penum = revNAT.keys();
			while(penum.hasMoreElements()) {
				Integer p = (Integer) penum.nextElement();
				ports = ports + p.toString();
				if(penum.hasMoreElements()) {
					ports = ports+",";
				}
			}
			return ports;
		}

		protected String setStats(String newValue) throws HyperCastStatsException
		{
			throw new HyperCastStatsException(statisticsName, HyperCastStatsException.WriteToAReadOnlyValue);
		}

		public Element[] getReadSchema(Document doc, XPath xpath) throws HyperCastStatsException
		{
			return XmlUtil.createSchemaElement(doc/* document instance */, statisticsName/* element name */, "xsd:String"/* type */, 
												null/* restriction base */, null/* pattern value */);
		}
	}
	
	public String getStatsName() {
		return statsName;
	}
	public void setStatsName(String name) {
		statsName = name;
	}
	public Element[] getStats(Document doc, XPath xpath) throws HyperCastStatsException {
		return statsPro.getStatsResult(doc, xpath);
	}
	public Element[] setStats(Document doc, XPath xpath, Element newValue) throws HyperCastStatsException {
		return statsPro.setStatsResult(doc, xpath, newValue);
	}
	public Element[] getReadSchema(Document doc, XPath xpath) throws HyperCastStatsException {
		return statsPro.getReadSchemaResult(doc, xpath);
	}
	public Element[] getWriteSchema(Document doc, XPath xpath) throws HyperCastStatsException {
		return statsPro.getWriteSchemaResult(doc, xpath);
	}

	public void handleEvent(SimEvent event) {
		// TODO Auto-generated method stub
		
	}

	public I_PhysicalAddress getPhysicalAddress() {
		return null;
	}

	public LinkedList getEventList() {
		return eventList;
	}

	public void checkEventListSize(){
		while(eventList.size()>=MAXIMUM_SIZE_OF_EVENT_HISTORY){
			eventList.removeFirst();
		}
	}

	public Enumeration logicalLinks() {
		// TODO Auto-generated method stub
		return null;
	}

	public boolean insertLogicalEdge(PhysicalNode n, Color c, int width) {
		// TODO Auto-generated method stub
		return false;
	}

	public boolean removeLogicalEdge(PhysicalNode n) {
		// TODO Auto-generated method stub
		return false;
	}

	public void clearLogicalEdges() {
		// TODO Auto-generated method stub
		
	}

	public Vector getLogicalNeighbors() {
		// TODO Auto-generated method stub
		return null;
	}

	public Enumeration getLogicalNeighbor() {
		// TODO Auto-generated method stub
		return null;
	}

	public Enumeration getLogicalColor() {
		// TODO Auto-generated method stub
		return null;
	}

	public Enumeration getLogicalWidth() {
		// TODO Auto-generated method stub
		return null;
	}

	public Vector getExternalNeighbors() {
		// TODO Auto-generated method stub
		return null;
	}

}
