package lotos.gui;

/***********************************************************************/
/* The University of Virginia specifically disclaims any warranties,   */
/* including but not limited to the implied warranties of              */
/* merchantability and fitness for a particular purpose.  The software */
/* provided hereunder is on an "as is" basis, and the University of    */
/* Virginia has no obligation to provide maintenance, support,         */
/* updates, enhancements, or modifications.                            */
/*                                                                     */
/* Copyright (c) 1998-2004 by the Rector and Visitors of the           */
/* University of Virginia.                                             */
/* All Rights Reserved.                                                */
/***********************************************************************/

import lotos.*;

import hypercast.util.XmlUtil;

import java.io.FileOutputStream;
import java.net.URL;

import java.awt.*; 
import java.awt.event.*;
import java.util.*;

import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import javax.swing.border.*;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Text;

import org.apache.xpath.XPath;

import hypercast.HyperCastStatsException;
import hypercast.I_Stats;

public class Evan extends JPanel
implements ActionListener, TreeSelectionListener, MouseListener
{
	/**
	 * Debug flag.
	 */
	public final static boolean debug = false;
	private I_Stats statistics;
	
	/**
	 * The menu bar of the configuration GUI.
	 */
	//private JMenuBar menu;

	/** The items under the File menu	 */
	//private JMenuItem loaditem, saveitem, saveasitem;//, newitem, openitem, exititem;

	/** the varaibles used to save or open files */
	//private JFileChooser filechooser;
	//private File file;

	/**
	 * A popup menu which is displayed when right clicking
	 * on the jtree.
	 */
	private JPopupMenu popupmenu;	 

	/**The item under the popup menu	 */
	private JMenuItem expanditem, collapseitem;

	/** The Document instance created from the opened schema file. */
	private Document statsDoc = null;

	/**
	 * The JScrollPane instance for displaying tree.
	 */
	private JScrollPane treePanel;

	/**
	 * The JPanel for editing a tree node when it is selected.
	 */
	private JPanel editPanel;

	/**
	 * The JTextField instance for displaying opreation related information.
	 */
	private JTextField infoField;

	/**
	 * The JTextField instance for displaying if the schema file is loaded.
	 */
	private JTextField schemafileLoadInfoField;

	/**
	 * The tree structure for displaying and editing properties.
	 */
	private JTree tree;
	private DefaultMutableTreeNode selectedNode;
	private DefaultMutableTreeNode root;
	private JTextField textBox;
	private JButton updateButton;
	private JButton reloadButton;
	private JButton saveButton;
	private DefaultTreeModel model;
	/**
	 * Flag indicating if the JTree object is empty or not.
	 * True--empty; False--not empty or a valid tree.
	 */
	private boolean isTreeEmpty = true;
	
	private Frame parent;

	/**
	 * Tree node which is selected currently.
	 */

	/**
	 * static constants for GUI components.
	 */
	public static final int FRAME_WIDTH = 620; 
	public static final int FRAME_HEIGHT = 620; 
	public static final int LABEL_WIDTH = 300; 
	public static final int LABEL_HEIGHT = 20; 
	public static final int COMBOX_WIDTH = 300; 
	public static final int COMBOX_HEIGHT = 20; 
	public static final int SPINNER_WIDTH = 300; 
	public static final int SPINNER_HEIGHT = 20; 
	public static final int SPINNERFORNUMBER_WIDTH = 60; 
	public static final int TEXTFIELD_WIDTH = 300; 
	public static final int TEXTFIELD_HEIGHT = 20; 

	/** The root element name in the schema file. */
	public static final String ROOT_ATTRIBUTE_ELEMENT_NAME = "Public";

	/**
	 * Contructor.
	 */
    public Evan(Frame p, I_Stats inputstats, String rootstr)
    {
    	parent = p;
    	
    	statistics = inputstats;

		root = new DefaultMutableTreeNode (rootstr);
		model = new DefaultTreeModel (root);
		tree = new JTree (model);
		
    	initializeDisplay();
    }
    
    public void initializeDisplay()
	{
		

		/*
		 * Create a JPopupMenu instance. It is displayed 
		 * when right clicking on the jtree.
		 */
		popupmenu = new JPopupMenu();
		expanditem = new JMenuItem("Expand Subtree");
		collapseitem = new JMenuItem("Collapse Subtree");
		popupmenu.add(expanditem);
		popupmenu.add(collapseitem);
		expanditem.addActionListener(this);
		collapseitem.addActionListener(this);

		/*
		 * Make a nice border for this JPanel.
		 */
		EmptyBorder eb = new EmptyBorder(5,5,5,5);
		BevelBorder bb = new BevelBorder(BevelBorder.LOWERED);
		CompoundBorder cb = new CompoundBorder(eb,bb);
		this.setBorder(new CompoundBorder(cb,eb));

		/*
		 * Build bottomt-side view. It's layout is set to null, so absolute
		 * position for displaying Swing units is supported.
		 */
		editPanel = new JPanel();
		editPanel.setLayout(new BorderLayout());
		//editPanel.setLayout(null);

		loadStats();
		
		tree.addTreeSelectionListener(this);
		tree.addMouseListener(this);

		tree.setEditable (true);
		
		tree.setVisible(true);

		/* 
		 * Build top-side view. Set an invisible tree in the treePanel
		 * when the treePanel is created. If we don't pass this tree 
		 * when initializing treePanel, no way is found to display
		 * a tree added later in the treePanel.
		 */
		treePanel = new JScrollPane(tree);
		
		/*
		 * Build split-pane view which contains tree panel and edit panel.
		 */
		JSplitPane splitPane = 
		   //new JSplitPane( JSplitPane.HORIZONTAL_SPLIT,
			new JSplitPane( JSplitPane.VERTICAL_SPLIT,
						   treePanel,
						   editPanel );
		splitPane.setContinuousLayout( true );
		//splitPane.setDividerLocation( FRAME_WIDTH/3 );
		splitPane.setDividerLocation( (FRAME_HEIGHT-155) );

		//Add split panel to this panel.
		this.setLayout(new BorderLayout());
		this.add(splitPane, BorderLayout.CENTER);

		JPanel infoPanel = new JPanel();
		infoPanel.setLayout(new BorderLayout());
		this.add(infoPanel, BorderLayout.SOUTH);

		/*
		 * Create a JTextField instance which is located at the bottom of the
		 * this JPanel object and is used to show the process and status information.
		 */		
		schemafileLoadInfoField = new JTextField("");
		schemafileLoadInfoField.setHorizontalAlignment(JTextField.CENTER);
		schemafileLoadInfoField.setEditable(false);
		infoPanel.add(schemafileLoadInfoField, BorderLayout.CENTER);

		/*
		 * Create a JTextField instance which is located at the bottom of the
		 * this JPanel object and is used to show the process and status information.
		 */		
		infoField = new JTextField("");
		infoField.setHorizontalAlignment(JTextField.CENTER);
		infoField.setEditable(false);
		infoPanel.add(infoField, BorderLayout.SOUTH);
		
		textBox = new JTextField("");
		textBox.setHorizontalAlignment(JTextField.CENTER);
		textBox.setEditable(false);
		textBox.setVisible(true);
		editPanel.add(textBox, BorderLayout.CENTER);
		
		updateButton = new JButton("Update");
		updateButton.addActionListener(this);
		editPanel.add(updateButton, BorderLayout.EAST);
		updateButton.setEnabled(true);
		
		reloadButton = new JButton("Reload");
		reloadButton.addActionListener(this);
		editPanel.add(reloadButton, BorderLayout.WEST);
		reloadButton.setEnabled(true);

		saveButton = new JButton("Save");
		saveButton.addActionListener(this);
		editPanel.add(saveButton, BorderLayout.SOUTH);
		saveButton.setEnabled(true);
		/*
		 * By default the schema file hypercast3.0.xsd contained in hypercast3.0.jar is loaded.
		 */
		//loadDefaultSchemaFile();		
		

	}
    
    public void loadStats() {    	
		lotosDebug.println("Evan: Loading stats");
    	statsDoc = XmlUtil.createDocument();
		XPath path = XmlUtil.createXPath("/*");
		try {
			Element[] statelements = statistics.getStats(statsDoc, path);
			for(int i=0;i<statelements.length;i++)
				statsDoc.appendChild(statelements[i]);
		} catch (HyperCastStatsException e) {
			lotosDebug.println("Evan: can not read statistics");
		}
		XmlUtil.removeComments(statsDoc, statsDoc);
		root.removeAllChildren();
		buildTree(root,statsDoc.getFirstChild());
		model.nodeStructureChanged(root);
    }
    
    public void actionPerformed(ActionEvent e) 
	{
    	if(e.getSource().equals(reloadButton)) {
    		loadStats();
    	} 
    	else if (e.getSource().equals(updateButton))
    	{    		
    		if(textBox.isEditable()) {
    			TreeNode[] path = selectedNode.getPath();
    			String xstr = "";
    			int index=1;
    			int total=1;
    			for(int i=0;i<path.length-1;i++){
    				xstr += "/" + path[i].toString();
    				if(total>1)
    					xstr += "["+index+"]";
    				
    				Enumeration kids = path[i].children();
    				String next = (String)((DefaultMutableTreeNode)path[i+1]).getUserObject();
    				index = 1;
    				total = 1;
    				//lotosDebug.print(next+" ");
    				while(kids.hasMoreElements()) {
    					DefaultMutableTreeNode tmpnode = (DefaultMutableTreeNode)kids.nextElement();
    					if(tmpnode == (DefaultMutableTreeNode)path[i+1]) {
    						index = total;
    					} else if(((String)tmpnode.getUserObject()).equals(next))
    						total++;
    				}
    				//lotosDebug.print(""+total); 				
    			}
    			lotosDebug.println(xstr);   
    			textBox.setEditable(false);
    			
    			XPath xpath = XmlUtil.createXPath(xstr);
    			String newVal = textBox.getText();
    			String oldVal = selectedNode.toString();
    			if(newVal != oldVal) {
    				try {
	    				String parent = selectedNode.getParent().toString();
	    				Document doc;
	    				if(newVal != "")
	    					doc = XmlUtil.createDocumentFromString("<"+parent+">"+newVal+"</"+parent+">");
	    				else
	    					doc = XmlUtil.createDocumentFromString("<"+parent+"/>");
	    				
	    				if(doc.getChildNodes().getLength() > 0) {
	    					lotosDebug.println(((Element)doc.getChildNodes().item(0)).toString());
	    					Element results [] = statistics.setStats(doc,xpath,(Element)doc.getChildNodes().item(0));

		    				((DefaultMutableTreeNode)selectedNode).setUserObject(newVal);
		    	    		((DefaultTreeModel) tree.getModel()).reload(selectedNode);
	    				}
    				} catch (Exception ee) {
    					lotosDebug.println(ee.toString());
    				}
    			}
    		}
    	}
    	else if (e.getSource().equals(saveButton))
    	{
        	String filename = getFileName();
            if( filename != null )
            {
            	filename = filename.trim();
            	if(!filename.toLowerCase().endsWith(".xml")){
            		filename += ".xml";
            	}
        		try {
        			FileOutputStream output = new FileOutputStream(filename);
        			XmlUtil.writeXml(statsDoc, output);
        		} catch (Exception ex) {
        			lotosDebug.errorln("Error when saving statistics to file: "+filename);
        		}
            }
    	}
	}
    
    public void buildTree( DefaultMutableTreeNode currentNode, Node docNode )
    {
    	NodeList buildList = docNode.getChildNodes();
    	for(int i=0; i < buildList.getLength(); i++)
		{
    		if(buildList.item(i).getNodeName().charAt(0) != '#')
    		{
    			DefaultMutableTreeNode child = new DefaultMutableTreeNode (buildList.item(i).getNodeName());
    			currentNode.add(child);
    			buildTree(child,buildList.item(i));
    		}
    		else
    		{
    			DefaultMutableTreeNode child = new DefaultMutableTreeNode (buildList.item(i).getNodeValue());
    			currentNode.add(child);
    		}
		}
    }
    
    public void valueChanged(TreeSelectionEvent e)
	{
    	if(tree.getLastSelectedPathComponent() != null)
    	{
    		if(((DefaultMutableTreeNode)tree.getLastSelectedPathComponent()).getChildCount()==0)
    		{
    			selectedNode = (DefaultMutableTreeNode) tree.getLastSelectedPathComponent();
    			textBox.setEditable(true);
    			//updateButton.setEnabled(true);
    			textBox.setText(tree.getLastSelectedPathComponent().toString());
    		}
    		else
    		{
    			textBox.setEditable(false);
    			//updateButton.setEnabled(false);
    			textBox.setText("");
    		}
    	}
	}
    
    public void mouseClicked(MouseEvent e)
	{
	}
    
    public void mouseEntered(MouseEvent e)
	{
	}
    
    public void mouseExited(MouseEvent e)
	{
	}
    
    public void mousePressed(MouseEvent e)
	{
	}
    
    public void mouseReleased(MouseEvent e)
	{
	}
    
    private Document createDocument(JTree jtree)
	{
		DefaultMutableTreeNode treeroot = (DefaultMutableTreeNode) ((DefaultTreeModel) jtree.getModel()).getRoot();
		
		Document doc = XmlUtil.createDocument();
		
		Element xmltreeRoot = (Element) CreateSubXmlTreeFromJTree(doc, jtree, treeroot);
		
		doc.appendChild(xmltreeRoot);
		return doc;
	}
    
    private Node CreateSubXmlTreeFromJTree(Document doc, JTree jtree, DefaultMutableTreeNode treeNode)
	{
		DefaultTreeModel treeModel = (DefaultTreeModel) jtree.getModel();
		
		Node xmlNode = null;
		
		/*
		 * Here we use treeNode.toString() as the name of XML node name
		 * because in jtree each tree node is assigned a used-defined object,
		 * in this case treeNode.toString() represents the identifier of
		 * the node in the tree.
		 */
		if (treeModel.isLeaf(treeNode))
		{
			DefaultMutableTreeNode parent = null;
			if (treeNode.getParent() != null)
			{
				parent = (DefaultMutableTreeNode) treeNode.getParent();
			}

			
			/*
			 * If a treenode is leaf node and its parent represents a simpletype
			 * attribute, create a Text node for this tree node.
			 */
			if (treeNode.getChildCount() == 0)
			{
				xmlNode = (Text) doc.createTextNode(treeNode.toString());
				
				//System.out.println("CreateSubXmlTreeFromJTree: text node for parent "+parent.toString()+" has value "+xmlNode.getNodeValue()+".");

				//if (xmlNode == null)
				//	System.out.println("CreateSubXmlTreeFromJTree: text node for parent "+parent.toString()+" is null");
				//else
				//	System.out.println("CreateSubXmlTreeFromJTree: text node for parent "+parent.toString()+" is "+xmlNode.toString()+".");
				
			}
			/*
			 * If a treenode is leaf node and but its parent doesn't represent a 
			 * simpletype attribute, create an Element for it.
			 */
			else
			{
				xmlNode = (Element) doc.createElement(treeNode.toString());
			}
		}
		else
		{
			xmlNode = (Element) doc.createElement(treeNode.toString());

			/*
			 * Recursively create XMl subtree for each jtree child and add them
			 * to this XML node.
			 */
			for (int i = 0; i<treeNode.getChildCount(); i++)
			{
				DefaultMutableTreeNode child = (DefaultMutableTreeNode) treeNode.getChildAt(i);
				Node xmlChild = CreateSubXmlTreeFromJTree(doc, jtree, child);
				xmlNode.appendChild(xmlChild);
			}
		}

		return xmlNode;
	}
    private String getFileName()
    {
        FileDialog fd = new FileDialog(parent, "Save File", FileDialog.SAVE);
        fd.show();

        String result;
        if (fd.getFile() == null)   // Cancel button was pressed.
            result = null;
        else
            result = fd.getDirectory() + fd.getFile();

        fd.dispose();
        return result;
    }
}