package lotos.gui;

import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.List;
import java.awt.Panel;
import java.awt.TextArea;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.LinkedList;

import lotos.I_ProtocolInterface;
import lotos.history.EventHistoryEntry;
import lotos.LotosHyperCastAppl;

/**
*
* * This dialog show the timer and messages that have been sent and received by a node
* @author Guangyu Dong
*
*/
public class EventWindow extends Dialog implements ActionListener, ItemListener{
	I_ProtocolInterface hcNode;
	List lt_mt;
	TextArea ta_details;
	TextArea ta_output;
	Button b_update;
	Button b_clear;
	Button b_close;
	LinkedList entries;
	LinkedList console;
	
	public EventWindow(I_ProtocolInterface hcNode){
		super(new Frame(), "Event History", false);
		
		this.setSize(800,600);
		this.setLocation(300, 100);
		
		this.setLayout(new BorderLayout());
		
		Panel bottomPanel = new Panel();
		
		b_update = new Button("Update");
		b_update.addActionListener(this);
		bottomPanel.add(b_update);
		
		
		b_clear = new Button("Clear");
		b_clear.addActionListener(this);
		bottomPanel.add(b_clear);
		
		b_close = new Button("Close");
		b_close.addActionListener(this);
		bottomPanel.add(b_close);
		
		this.add(bottomPanel, BorderLayout.SOUTH);
		
		Panel centerPanel = new Panel();
		centerPanel.setLayout(new BorderLayout());
		
		lt_mt = new List();
		lt_mt.addItemListener(this);
		centerPanel.add(lt_mt, BorderLayout.WEST);
		
		Panel displayPanel = new Panel();
		displayPanel.setLayout(new BorderLayout());
		
		Panel detailPanel = new Panel();
		detailPanel.setLayout(new BorderLayout());
		
		Panel outputPanel = new Panel();
		outputPanel.setLayout(new BorderLayout());
		
		displayPanel.add(detailPanel, BorderLayout.NORTH);
		displayPanel.add(outputPanel, BorderLayout.CENTER);
		
		detailPanel.setSize(200, 100);
		
		ta_details = new TextArea("<Empty>", 15 , 80 , TextArea.SCROLLBARS_VERTICAL_ONLY);
		detailPanel.add(ta_details, BorderLayout.CENTER);
		
		ta_output = new TextArea("<no output>", 25 , 80 , TextArea.SCROLLBARS_VERTICAL_ONLY);
		ta_output.setEditable(false);
		outputPanel.add(ta_output, BorderLayout.CENTER);
		
		centerPanel.add(displayPanel, BorderLayout.CENTER);
		
		this.add(centerPanel, BorderLayout.CENTER);
		
		setHCNode(hcNode);
	}
	public void setHCNode(I_ProtocolInterface hcnode){
		hcNode = hcnode;
		if(hcNode!=null){
			this.setTitle("Event History: "+hcnode.getName());
			ta_details.setText("<Empty>");
			lt_mt.clear();
			entries = (LinkedList)hcNode.getEventList().clone();
			for(int i=0;i<entries.size();i++){				
				EventHistoryEntry entry = (EventHistoryEntry)entries.get(i);
				String timestr = "" +entry.getTimestamp();
				while(timestr.length()<9)
					timestr+="0";
				
				lt_mt.add(timestr+"s : " +entry.getSummary());
			}
			if(hcNode instanceof LotosHyperCastAppl) {
				ta_output.setText("");
				LotosHyperCastAppl appl = (LotosHyperCastAppl) hcNode;
				console = (LinkedList)appl.getConsoleList().clone();
				for(int i=0;i<console.size();i++) {
					ta_output.append(console.get(i)+"\n");
				}
			}
			//ta_output.setText(hcNode.outputBuf.toString());
		}
	}
	
	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent arg0) {
		if (arg0.getSource() == b_close) {
			this.dispose();
		} else if(arg0.getSource() == b_update){
			if(hcNode!=null){
				setHCNode(hcNode);
			}
		}
		else if(arg0.getSource() == b_clear){
			if(hcNode!=null){
				hcNode.getEventList().clear();
				//hcNode.outputBuf.reset();
				setHCNode(hcNode);
			}
		}
	}

	/* (non-Javadoc)
	 * @see java.awt.event.ItemListener#itemStateChanged(java.awt.event.ItemEvent)
	 */
	public void itemStateChanged(ItemEvent arg0) {
		if(arg0.getSource()==lt_mt){
			int index = lt_mt.getSelectedIndex();
			if(index>=0){
				EventHistoryEntry entry = (EventHistoryEntry)entries.get(index); 
				ta_details.setText(entry.getDetails());
			}else{
				ta_details.setText("<Empty>");
				
			}
		}
		
	}
	
}