/***************************************************************************/
/*       Lotos, an extensible overlay network simulator                    */
/*       MNG Group                                                         */
/*       University of Virginia                                            */
/*       July, 2002                                                        */
/***************************************************************************/
package lotos.gui;

import lotos.*; 
import lotos.gateway.*;
import hypercast.util.HypercastConfigurator;
import hypercast.util.XmlUtil;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.io.IOException;
import java.io.InputStream;
import java.net.*;
import java.util.*;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.xml.transform.TransformerException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * The framework of the GUI interface
 */


public class GUI extends Applet implements ActionListener, ItemListener, KeyEventDispatcher
{
    // Global Definition of the GUI Interface
    final static Color bkColor = new Color(223, 223, 223);
    final static Color dwColor = new Color(160, 160, 223);
    final static Dimension winSize = new Dimension(900, 750);
    final static Color ndColor = Color.black;
    final static int ndDiameter = 8;
    final static Color edgeColor = Color.black;
    //static float PACKET_SPEED = 1000.0f;
    final static Color packetColor = Color.green;
    final static Color sendingColor = Color.BLUE;
    final static int packetSRadius = 6;
    final static int packetLRadius = 60;
    final static int packetAngle = 24;
    static Dimension packetSize = new Dimension(10, 5);
    static Dimension imageButtonSize = new Dimension( 20, 20 );
    static float zoominFactor = 1.25f;
    static float zoomoutFactor = 0.75f;
    	
   
    Animation animation;

    /** a handle to the simulated network */
    //OverlayNetwork simulation.network;
    //simHCNetwork simulation.network;
    SimControl simulation;

    float advance_time ;
    boolean animationRunning; //indicate if the packet-simulation is running

    //applet mode or application mode
    public boolean isApplet;

    //GUI components
    Frame theApplicationWindow;

    /**the main display window */
    public GUI_DisplayWindow dw;

    /* statistics reporting window deprecated 08/05/2006 - zhi*/
    //reportDialog reportDia;
  
    //menus
    PopupMenu pm_file;
    PopupMenu pm_demo;
    MenuItem mi_loadall;
    MenuItem mi_saveall;
    MenuItem mi_loadLogical;
    MenuItem mi_saveLogical;
    MenuItem mi_loadPhysical;
    MenuItem mi_savePhysical;
    
    MenuItem mi_print;
    //MenuItem mi_report;
    //MenuItem mi_graph;
    MenuItem mi_quit;

    //View menu and contained items
    PopupMenu pm_packet_view;
    CheckboxMenuItem cbmi_display_on;
    CheckboxMenuItem cbmi_display_logical_net;
    CheckboxMenuItem cbmi_display_physical_net;
    CheckboxMenuItem cbmi_display_node_labels;
    CheckboxMenuItem cbmi_display_any_messages;
    CheckboxMenuItem cbmi_display_message_labels;
    CheckboxMenuItem cbmi_display_wireless_range;
    CheckboxMenuItem cbmi_display_protocol;
    CheckboxMenuItem cbmi_display_logical_addr;
    CheckboxMenuItem cbmi_display_physical_addr;
    CheckboxMenuItem cbmi_display_security;
    
    
    //** menu items to zoo in and out the display
    MenuItem mi_zoomin;
    MenuItem mi_zoomout;
    MenuItem mi_bestfit;
    MenuItem mi_restore;
    //CheckboxMenuItem cbmi_display_multicast_messages;

    //Packet-based protocol;
    PopupMenu pm_packet_protocol;
    PopupMenu pm_security;
	PopupMenu pm_application;
    PopupMenu pm_gateway;
    
	//mobile mode 
	PopupMenu pm_mobile_mode;
	CheckboxMenuItem[] cbmi_mobile_mode;

    // Buttons which activate the File and View menues
	Button b_demo;
    Button b_filemenu;
	Button b_demomenu;
    Button b_viewmenu;
    Button b_protocolmenu;
    Button b_securitymenu;
    Button b_applicationmenu;
    Button b_gatewaymenu;
	Button b_mobilemodemenu;
	TextField tf_speed;
	TextField tf_hoplimit;
	//Button b_changespeed;

    // Run control items
    Panel run_palette;
    //Button b_stop;
    Button b_step;
    Button b_advance;
    TextField tf_advance_time;
    Label l_sec;
    Button b_run;
    Button b_restart;
    Button b_reset;
	Button b_stats;
	
    Panel edit_palette;
	
	Checkbox cb_st_wr;
	Checkbox cb_st_wl;
	Checkbox cb_st_both;
	Checkbox cb_mb_wl;
	
	Checkbox cb_move;
	Checkbox cb_drag;
	Checkbox cb_add_nodes;
	Checkbox cb_delete_nodes;
	Checkbox cb_add_links;
	Checkbox cb_delete_links;
    Checkbox cb_start_or_stop_protocol;
    Checkbox cb_show_routes;
    Checkbox cb_src;
    Checkbox cb_dst;

    // for warning message at bottom
    Label l_warning_message;


    /**construct function. Not used for the compatibility with Applet
     *
     */
    public GUI()
    {
         // only used in main().
         // real initialization done in init()
    }

    /**whether the packet simulation is running
     *@return   whether the animuation is running
     */
    public boolean animated() { return animationRunning; }


    /**
     * print a short message at the bottom of the GUI interface
     */
    public void printWarning(String s)
    {
        l_warning_message.setText(s);

    }
    
    String []demoFiles;
    MenuItem mi_demos[];
    private void initDemoMenu(String democonfig){
    	InputStream input = FileResourceManager.openFile(democonfig);
    	
    	if(input==null){
    		demoFiles = new String[0];
    		mi_demos = new MenuItem[0];
    		return;
    	}
    	Document demoxml = null;
    	try {
			demoxml = XmlUtil.createDocument(input, democonfig);
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		Element demoMenu = (Element)demoxml.getElementsByTagName("DemoMenu").item(0);
		NodeList demos = demoMenu.getElementsByTagName("Demo");
		
		mi_demos = new MenuItem[demos.getLength()];
		demoFiles = new String [demos.getLength()];
		
		for(int i=0;i<demos.getLength();i++){
			Element demo = (Element)demos.item(i);
			String menuName = demo.getAttribute("name");
			Element fileName = (Element)demo.getElementsByTagName("FileName").item(0);
			demoFiles[i] = fileName.getChildNodes().item(0).getNodeValue();
			mi_demos[i] = new MenuItem(menuName);
			mi_demos[i].addActionListener(this);
			pm_demo.add(mi_demos[i]);
		}
		
    }

    /**
     * The initialization function, responsible for setup initial states and
     * components layout. Used both in applet and application.
     */
    public void init()
    {
        if( this.theApplicationWindow == null ){
        	isApplet = true;
        	
        }
        else {
        	isApplet = false;
        	this.enableWindowListener();
        }
        KeyboardFocusManager.getCurrentKeyboardFocusManager().addKeyEventDispatcher(this);

        
        
        //initialize the componetns states
        setBackground(bkColor);
        simulation = new SimControl();
        simulation.initNetwork();
        simulation.network.the_gui = this;
        
        animationRunning = false; 
		
        
        pm_demo = new PopupMenu("Demo");
        //generate demo menu
        initDemoMenu("DemoMenuConfig.xml");
        this.add(pm_demo);
        
        
        
		if(this.isApplet==true){
			//not 
		}
        else{
        	pm_file = new PopupMenu("File");
			mi_loadPhysical = new MenuItem("Load Substrate Network");
			mi_loadPhysical.addActionListener(this);
            pm_file.add(mi_loadPhysical);

            mi_loadLogical = new MenuItem("Load Overlay Network");
            mi_loadLogical.addActionListener(this);
            pm_file.add(mi_loadLogical);
            
            mi_loadall = new MenuItem("Load All");
            mi_loadall.addActionListener(this);
            pm_file.add(mi_loadall);
            
            pm_file.addSeparator();
            
            mi_savePhysical = new MenuItem("Save Substrate Network");
            mi_savePhysical.addActionListener(this);
            pm_file.add(mi_savePhysical);

            mi_saveLogical = new MenuItem("Save Overlay Network");
            mi_saveLogical.addActionListener(this);
            pm_file.add(mi_saveLogical);
            
            mi_saveall = new MenuItem("Save All");
            mi_saveall.addActionListener(this);
            pm_file.add(mi_saveall);

       		pm_file.addSeparator();
            mi_print = new MenuItem("Print");
            mi_print.addActionListener(this);
            pm_file.add(mi_print);
        
	        pm_file.addSeparator();
	
	        //mi_report = new MenuItem("Report");
	        //mi_report.addActionListener(this);
	        //pm_file.add(mi_report);
	
	        //mi_graph = new MenuItem("Static Analysis");
	        //mi_graph.addActionListener(this);
	        //pm_file.add(mi_graph);
	
	        //pm_file.addSeparator();
	
	
	        mi_quit = new MenuItem("Quit");
	        mi_quit.addActionListener(this);
	        pm_file.add(mi_quit);
			
	        this.add(pm_file);
		}
		
        // create view menu
        pm_packet_view = new PopupMenu("View");

        mi_zoomin = new MenuItem( "zoomin" );
        pm_packet_view.add(mi_zoomin);
        mi_zoomin.addActionListener(this);

        mi_zoomout = new MenuItem( "zoomout" );
        pm_packet_view.add(mi_zoomout);
        mi_zoomout.addActionListener(this);

        mi_restore = new MenuItem( "restore" );
        pm_packet_view.add(mi_restore);
        mi_restore.addActionListener(this);

        mi_bestfit = new MenuItem( "best fit" );
        pm_packet_view.add(mi_bestfit);
        mi_bestfit.addActionListener(this);

        pm_packet_view.addSeparator();

        cbmi_display_on = new CheckboxMenuItem("Display On/Off", true);
        pm_packet_view.add(cbmi_display_on);
        cbmi_display_on.addItemListener(this);
        
        cbmi_display_physical_net = new CheckboxMenuItem("Show Physical Links", true);
        pm_packet_view.add(cbmi_display_physical_net);
        cbmi_display_physical_net.addItemListener(this);

        cbmi_display_logical_net = new CheckboxMenuItem("Show Logical Links", true);
        pm_packet_view.add(cbmi_display_logical_net);
        cbmi_display_logical_net.addItemListener(this);

        cbmi_display_node_labels = new CheckboxMenuItem("Show Node Labels", true);
        pm_packet_view.add(cbmi_display_node_labels);
        cbmi_display_node_labels.addItemListener(this);

        cbmi_display_any_messages = new CheckboxMenuItem("Show Messages", true);
        pm_packet_view.add(cbmi_display_any_messages);
        cbmi_display_any_messages.addItemListener(this);

        cbmi_display_message_labels = new CheckboxMenuItem("Show Message Labels", true);
        pm_packet_view.add(cbmi_display_message_labels);
        cbmi_display_message_labels.addItemListener(this);
        
        cbmi_display_wireless_range = new CheckboxMenuItem("Show Wireless Range", true);
        pm_packet_view.add(cbmi_display_wireless_range);
        cbmi_display_wireless_range.addItemListener(this);
        
        cbmi_display_protocol = new CheckboxMenuItem("Show Protocols", true);
        pm_packet_view.add(cbmi_display_protocol);
        cbmi_display_protocol.addItemListener(this);
        
        cbmi_display_logical_addr = new CheckboxMenuItem("Show Logical Addresses", true);
        pm_packet_view.add(cbmi_display_logical_addr);
        cbmi_display_logical_addr.addItemListener(this);
        
        cbmi_display_physical_addr = new CheckboxMenuItem("Show Physical Addresses", true);
        pm_packet_view.add(cbmi_display_physical_addr);
        cbmi_display_physical_addr.addItemListener(this);
        
        cbmi_display_security = new CheckboxMenuItem("Show Security Settings", true);
        pm_packet_view.add(cbmi_display_security);
        cbmi_display_security.addItemListener(this);

        this.add(pm_packet_view);
        
        /*
        pm_packet_protocol = new PopupMenu("Protocol");
        cbmi_packet_protocol = new CheckboxMenuItem[OverlayNetwork.getProtocols().length];
        for( int i=0; i< cbmi_packet_protocol.length; i++){
            cbmi_packet_protocol[i] = new CheckboxMenuItem( (OverlayNetwork.getProtocols())[i], false);
            pm_packet_protocol.add(cbmi_packet_protocol[i]);
            cbmi_packet_protocol[i].addItemListener(this);
        }

        cbmi_packet_protocol[0].setState(true);
        this.add(pm_packet_protocol);
        */
        pm_packet_protocol = simulation.network.getProtocolMenu();
        this.add(pm_packet_protocol);
        
        pm_security = simulation.network.getSecurityMenu();
        this.add(pm_security);
        
        pm_application = simulation.network.getApplMenu();
        this.add(pm_application);
        
        if(SimConfig.hybrid) {
        pm_gateway = simulation.network.getGatewayMenu();
        this.add(pm_gateway);
        }
        
        simulation.network.chooseProtocol("None");
        simulation.network.chooseSecurity("None");
		simulation.network.chooseApplication("None");
        
		pm_mobile_mode = new PopupMenu("MobileMode");
        cbmi_mobile_mode = new CheckboxMenuItem[PhysicalNetwork.getMobilemode().length];
        for( int i=0; i< cbmi_mobile_mode.length; i++){
            cbmi_mobile_mode[i] = new CheckboxMenuItem( (PhysicalNetwork.getMobilemode())[i], false);
            pm_mobile_mode.add(cbmi_mobile_mode[i]);
            cbmi_mobile_mode[i].addItemListener(this);
        }

        cbmi_mobile_mode[0].setState(true);
        this.add(pm_mobile_mode);


        // create layout
        setLayout( new BorderLayout() );

        // create upper display options
        // (buttons to active File and View popup menus
        //Panel north_container = new Panel();
        //north_container.setLayout(new GridLayout(1, 2) );

        Panel north_panel = new Panel();
        //north_panel.setLayout(new GridLayout(1, 4, 4, 0));
		north_panel.setLayout(new FlowLayout(FlowLayout.LEFT));
		
		b_demo = new Button("Demo");
        north_panel.add(b_demo);
        b_demo.addActionListener(this);
		
		if(!this.isApplet){
	        b_filemenu = new Button("File");
	        north_panel.add(b_filemenu);
	        b_filemenu.addActionListener(this);
		}


        b_viewmenu = new Button("View");
        north_panel.add(b_viewmenu);
        b_viewmenu.addActionListener(this);

        b_protocolmenu = new Button("Protocol");
        north_panel.add(b_protocolmenu);
        b_protocolmenu.addActionListener(this);
        
        b_securitymenu = new Button("Security");
        north_panel.add(b_securitymenu);
        b_securitymenu.addActionListener(this);
		
        b_applicationmenu = new Button("Application");
        north_panel.add(b_applicationmenu);
        b_applicationmenu.addActionListener(this);
        
        if(SimConfig.hybrid) {
	        b_gatewaymenu = new Button("Gateway");
	        north_panel.add(b_gatewaymenu);
	        b_gatewaymenu.addActionListener(this);
        }
        
		b_mobilemodemenu = new Button("Mobile Mode");
        north_panel.add(b_mobilemodemenu);
        b_mobilemodemenu.addActionListener(this);
        
        north_panel.add(new Label("Speed(m/s):"));
		tf_speed = new TextField("100");
		north_panel.add(tf_speed);
		tf_speed.addActionListener(this);
		
		north_panel.add(new Label("Multicast Hop Limit:"));
		tf_hoplimit = new TextField("255");
		north_panel.add(tf_hoplimit);
		tf_hoplimit.addActionListener(this);
		
		/*
		b_changespeed = new Button("Change Speed");
		north_panel.add(b_changespeed);
		b_changespeed.addActionListener(this);
		*/

		add(BorderLayout.NORTH,  north_panel);
        // create lower left controls
        Panel southwest_panel = new Panel();

        // create lower right display options
        // e.g. stop, advance, play
        Panel southeast_panel = new Panel();

        //Image loading is different for applet-based case and application
       


		run_palette = new Panel();
        run_palette.setLayout(new GridLayout(0,1));

        b_run = new Button("Run");
        run_palette.add(b_run);
        b_run.addActionListener(this);

        // initialize speed to real time
        Panel advance_time_panel = new Panel();
        advance_time = (float) SimConfig.SIM_STEP;
        tf_advance_time = new TextField(Float.toString((float) 1.0));
        advance_time_panel.add(tf_advance_time);
        tf_advance_time.addActionListener(this);

        l_sec = new Label("Sim. Sec. per Real Sec.");
        advance_time_panel.add(l_sec);
        run_palette.add(advance_time_panel);

        /*
        b_stop = new Button("Stop");
        run_palette.add(b_stop);
        b_stop.addActionListener(this);
        */
        
        b_step = new Button("Go To Next Event");
        run_palette.add(b_step);
        b_step.addActionListener(this);

        b_advance = new Button("Go To Next Frame");
        run_palette.add(b_advance);
        b_advance.addActionListener(this);

            
        b_restart = new Button("Restart");
        run_palette.add(b_restart);
        b_restart.addActionListener(this);
        
        b_reset = new Button("Reset");
        run_palette.add(b_reset);
        b_reset.addActionListener(this);

        b_stats = new Button("Stats");
        run_palette.add(b_stats);
        b_stats.addActionListener(this);
        
        edit_palette = new Panel();
        edit_palette.setLayout(new GridLayout(0,1));
        
        edit_palette.add(new Label("--Node Type--"));

		cb_st_wr = new Checkbox("Static Wired", true);
        edit_palette.add(cb_st_wr);
        cb_st_wr.addItemListener(this);
        
        cb_st_wl = new Checkbox("Static Wireless", false);
        edit_palette.add(cb_st_wl);
        cb_st_wl.addItemListener(this);
        
        cb_st_both = new Checkbox("Static Both", false);
        edit_palette.add(cb_st_both);
        cb_st_both.addItemListener(this);
        
        cb_mb_wl = new Checkbox("Mobile Wireless", false);
        edit_palette.add(cb_mb_wl);
        cb_mb_wl.addItemListener(this);
        
		edit_palette.add(new Label("--Mouse Mode--"));

		cb_move = new Checkbox("Select/Move", true);
        edit_palette.add(cb_move);
        cb_move.addItemListener(this);
        
		cb_drag = new Checkbox("Drag", true);
		//edit_palette.add(cb_drag);
		cb_drag.addItemListener(this);
        

        cb_add_nodes = new Checkbox("Add Nodes", false);
        edit_palette.add(cb_add_nodes);
        cb_add_nodes.addItemListener(this);
        
        cb_delete_nodes = new Checkbox("Delete Nodes", false);
        edit_palette.add(cb_delete_nodes);
        cb_delete_nodes.addItemListener(this);
        
        cb_add_links = new Checkbox("Add Links", true);
        edit_palette.add(cb_add_links);
        cb_add_links.addItemListener(this);
        
        cb_delete_links = new Checkbox("Delete Links", false);
        edit_palette.add(cb_delete_links);
        cb_delete_links.addItemListener(this);

        
        cb_start_or_stop_protocol = new Checkbox("Join/Leave Overlay", false);
        edit_palette.add(cb_start_or_stop_protocol);
        cb_start_or_stop_protocol.addItemListener(this);

        cb_show_routes = new Checkbox("Multicast Tree", false);
        edit_palette.add(cb_show_routes);
        cb_show_routes.addItemListener(this);
        
        cb_src = new Checkbox("Select Src", false);
        edit_palette.add(cb_src);
        cb_src.addItemListener(this);
        
        cb_dst = new Checkbox("Select Dst", false);
        edit_palette.add(cb_dst);
        cb_dst.addItemListener(this);

       
		Panel west_panel = new Panel();
        west_panel.setLayout(new BorderLayout());

        west_panel.add(BorderLayout.NORTH, run_palette);
        west_panel.add(BorderLayout.SOUTH, edit_palette);


        add(BorderLayout.WEST, west_panel);

        l_warning_message = new Label("Lotos Started.");
        l_warning_message.setBackground(new Color(160, 160, 160));
        add(BorderLayout.SOUTH, l_warning_message);


        // create DisplayWindow
        dw = new GUI_DisplayWindow(this);
        add(BorderLayout.CENTER, dw);
        setMode( GUI_DisplayWindow.EDIT_NODE );
        setNodeType( GUI_DisplayWindow.NODE_STATIC_WIRED);
        
        // set the edit mode buttons to the right state
        this.repaint();
        dw.repaint();
    }


/**
 *  Called when the simulation runs as an application.
 */
    public static void main(String argv[])
    {
    	System.out.println("Starting LoToS");
        GUI gui = new GUI();
        gui.theApplicationWindow = new Frame( "LOTOS: Overlay Protocol Simulator" );
        
        gui.theApplicationWindow.add(BorderLayout.CENTER, gui);
        gui.theApplicationWindow.setSize(winSize);
        gui.theApplicationWindow.addWindowListener(new LotosFrameListener());
        gui.init();
        gui.start();
       // Toolkit tk = gui.theApplicationWindow.getToolkit();
       // gui.theApplicationWindow.setIconImage( tk.createImage("hcast.gif") );

        gui.theApplicationWindow.show();

    }


    /** Called when the applet starts running or becomes visible */
    public void start()
    {
		//load("default.txt");
    }


    /**Called when the applet is no longer visible */
    public void stop()
    {
    }
    private void loadNetwork(boolean logical, boolean physical, String filename){
    	if(filename==null)
    		filename = getFileName(FileDialog.LOAD);
        if( null != filename )
        {
        	synchronized(simulation.network){
            	printWarning("Simulation reseted");
                if( animation != null )  { 
                	animation.stopAnimation();
                	b_run.setLabel("Run");
                }
                //if( simulation.network.isRunning() ) simulation.network.reset();
                /*
                 * reset the gateways so that the ports are cleared
                 */
                if(simulation.network.gateway_nodes.size()>0) {
                	PhysicalNode gnode = (PhysicalNode)simulation.network.gateway_nodes.get(0);
                	((LotosGateway)gnode.getProtocol()).reset();
                }
                animationRunning = false;
                
            	//simulation.network = createCurrentProtocol(null);
                String sec = simulation.network.getCurrentSecurity();
                String prot = simulation.network.getCurrentProtocolName();
                //simulation.network = new simHCNetwork();
                simulation.resetNetwork();
                
                simulation.network.the_gui = this;
                this.remove(pm_packet_protocol);
                pm_packet_protocol = simulation.network.getProtocolMenu();
                simulation.network.chooseProtocol(prot);
                simulation.network.chooseSecurity(sec);
                this.add(pm_packet_protocol);
            	
                this.remove(pm_security);
                pm_security = simulation.network.getSecurityMenu();
                this.add(pm_security);
                
                this.remove(pm_application);
                pm_application = simulation.network.getApplMenu();
                this.add(pm_application);
                
                this.remove(pm_gateway);
                pm_gateway = simulation.network.getGatewayMenu();
                this.add(pm_gateway);
                
                //simulation.network.numnode = 0;
		    	dw.repaint();
		    }
        	simulation.network.setLoadSaveFlag(logical, physical);
            if( simulation.network.protocolReader(filename) == false)
                this.printWarning("load file fails");
            dw.repaint();
        }
    }
    
    private void saveNetwork(boolean logical, boolean physical){
    	String filename = getFileName(FileDialog.SAVE);
        if( filename != null )
        {
        	filename = filename.trim();
        	if(!filename.toLowerCase().endsWith(".xml")){
        		filename += ".xml";
        	}
        	simulation.network.setLoadSaveFlag(logical, physical);
            if( simulation.network.protocolWriter(filename)==false)
                this.printWarning("save file fails");
        }
    }
    /** handles buttons, menu items, returns entered in textfield */
    public synchronized void actionPerformed(ActionEvent ae)
    {
        // synchronizing to prevent bad interaction from animation thread
    	if (ae.getSource() == b_demo){
    		if( animationRunning ){
            	this.printWarning("Stop the simulation first to show this menu");
            	return;
            }
    		 Point p = b_demo.getLocation();
             pm_demo.show(this, p.x, p.y);
    	}
    	else if (ae.getSource() == b_filemenu)
        {
            if( animationRunning ){
            	this.printWarning("Stop the simulation first to show this menu");
            	return;
            }
            Point p = b_filemenu.getLocation();
            pm_file.show(this, p.x, p.y);
        }
        else if (ae.getSource() == b_viewmenu)
        {
            //if( animationRunning ) return;
            Point p = b_viewmenu.getLocation();
            pm_packet_view.show(this, p.x, p.y);
        }
        else if (ae.getSource() == b_protocolmenu)
        {
            //if( animationRunning ) return;
            Point p = b_protocolmenu.getLocation();
		    pm_packet_protocol.show(this, p.x, p.y);
        } else if(ae.getSource() == b_applicationmenu) {
        	Point p = b_applicationmenu.getLocation();
        	pm_application.show(this, p.x, p.y);
        } else if(ae.getSource() == b_gatewaymenu) {
        	Point p = b_gatewaymenu.getLocation();
        	pm_gateway.show(this, p.x, p.y);
        }        
        else if (ae.getSource() == b_securitymenu)
        {
            //if( animationRunning ) return;
            Point p = b_securitymenu.getLocation();
		    pm_security.show(this, p.x, p.y);
        }
		else if (ae.getSource() == b_mobilemodemenu)
        {
            //if( animationRunning ) return;
            Point p = b_mobilemodemenu.getLocation();
		    pm_mobile_mode.show(this, p.x, p.y);
        }
		else if(ae.getSource() == tf_speed){
			//change all nodes to this speed
			float speed;
			try{
				speed=Float.parseFloat(tf_speed.getText());
			}catch(Exception e){
				speed=0;
			}
			simulation.network.setSpeed(speed);
		}
		else if(ae.getSource() == tf_hoplimit){
			int hoplimit;
			try{
				hoplimit=Integer.parseInt(tf_hoplimit.getText());
			}catch(Exception e){
				hoplimit=255;
				tf_hoplimit.setText("255");
			}
			simulation.network.setMulticastHopLimit(hoplimit);
		}
        else if (ae.getSource() == mi_loadLogical)
        {
            this.loadNetwork(true, false, null);

        }
        else if (ae.getSource() == mi_saveLogical)
        {
           this.saveNetwork(true, false);
        }
        else if (ae.getSource() == mi_loadPhysical){
        	this.loadNetwork(false, true, null);
        }
        else if (ae.getSource() == mi_savePhysical){
        	this.saveNetwork(false, true);
        }
        
        else if (ae.getSource() == mi_loadall){
        	this.loadNetwork(true, true, null);
        }
        else if (ae.getSource() == mi_saveall){
        	this.saveNetwork(true, true);
        }
        	
	    else if (ae.getSource() == mi_quit)
        {
                System.exit(0);
        }
    	
        else if (ae.getSource() == mi_print)  {
            Toolkit tk = dw.getToolkit();
            PrintJob pj = tk.getPrintJob(theApplicationWindow, "Lotus", null);
            if (pj != null) // user did not cancel (SUN)
            {
                    Graphics page_graphics = pj.getGraphics();
                    if (page_graphics != null)  // user did not cancel (Windows)
                    {
                        dw.print(page_graphics);  // print() just calls paint()
                        page_graphics.dispose();
                    }
                    pj.end();
            }
		}
        else if( ae.getSource() == mi_zoomin ) {
            dw.scaleBy( zoominFactor );
            dw.repaint();
        }else if( ae.getSource() == mi_zoomout ) {
            dw.scaleBy( zoomoutFactor );
            dw.repaint();
        }else if( ae.getSource() == mi_bestfit ) {
            dw.bestfit();
            dw.repaint();
        }else if( ae.getSource() == mi_restore ) {
            dw.restore();
            dw.repaint();
        }
        else if( ae.getSource() == tf_advance_time ){
             try
            {
                advance_time = (Float.valueOf(tf_advance_time.getText())).floatValue()*SimConfig.SIM_SLEEP/1000;
            }
            catch (NumberFormatException nfe)
            {
                printWarning("incorrect numeric format");
            }
        }else if ( ae.getSource() == b_run)
        {
        	if(!animationRunning){
        		/* Run */
	            float ad_time = 0;
		      	try {
		            ad_time = (Float.valueOf(tf_advance_time.getText())).floatValue();
		        }catch( Exception e){
		            printWarning(" invalid advance time format");
		            return;
		        }
		        if( ad_time <= 0 ){
		            printWarning(" can not go backward" );
		            return;
		        }
		        advance_time=ad_time*SimConfig.SIM_SLEEP/1000;
		
	            boolean started = true;
	            if( simulation.network.isRunning() == false ) {
	                started = simulation.start();
	            }
	
	            if( false == started ) {
	                printWarning("simulation can not be started");
	                return;
	            }
	            printWarning("start the simulation");
	            animationRunning = true;
	            animation = new Animation();
	            animation.start();
	            b_run.setLabel("Pause");
        	}else{
        		/* Pause */
        		 printWarning("Simulation stops");
                 if( animation != null )  { animation.stopAnimation(); }
                 //if( simulation.network.isRunning() ) simulation.network.reset();

                 animationRunning = false;
                 dw.repaint();
        		
                 b_run.setLabel("Run");
        	}
            
        }else if ( ae.getSource() == b_restart)
        {
            synchronized(simulation.network){
            	//simulation.network = createCurrentProtocol(null);
            	simulation.restartNetwork();
            	
            	if(animationRunning){
	        		boolean started=true;
	        		if( simulation.network.isRunning() == false ) {
	                	started = simulation.network.start();
	            	}
	
	            	if( false == started ) {
	                	printWarning("simulation can not be started");
	                	return;
	            	}
            	}
		    	dw.repaint();
		    }
        }
        else if ( ae.getSource() == b_reset)
        {
            synchronized(simulation.network){
            	printWarning("Simulation reseted");
                if( animation != null )  { 
                	animation.stopAnimation();
                	b_run.setLabel("Run");
                }
                //if( simulation.network.isRunning() ) simulation.network.reset();
                /*
                 * reset the gateways so that the ports are cleared
                 */
                if(simulation.network.gateway_nodes.size()>0) {
                	PhysicalNode gnode = (PhysicalNode)simulation.network.gateway_nodes.get(0);
                	((LotosGateway)gnode.getProtocol()).reset();
                }
                animationRunning = false;
                
            	//simulation.network = createCurrentProtocol(null);
                String sec = simulation.network.getCurrentSecurity();
                String prot = simulation.network.getCurrentProtocolName();
                String appl = simulation.network.getCurrentApplicationName();
                //simulation.network = new simHCNetwork();
                simulation.resetNetwork();
                
                simulation.network.the_gui = this;
                this.remove(pm_packet_protocol);
                pm_packet_protocol = simulation.network.getProtocolMenu();
                simulation.network.chooseProtocol(prot);
                simulation.network.chooseSecurity(sec);
                simulation.network.chooseApplication(appl);
                this.add(pm_packet_protocol);
            	
                this.remove(pm_security);
                pm_security = simulation.network.getSecurityMenu();
                this.add(pm_security);
                
                this.remove(pm_application);
                pm_application = simulation.network.getApplMenu();
                this.add(pm_application);
                
                if(SimConfig.hybrid) {
	                this.remove(pm_gateway);
	                pm_gateway = simulation.network.getGatewayMenu();
	                this.add(pm_gateway);
                }
                //simulation.network.numnode = 0;
		    	dw.repaint();
		    }
        }
        else if (ae.getSource() == b_stats) {
        	/*
        	StatsWindow statswin = new StatsWindow(simulation);
    		statswin.show();
    		*/
        	JFrame frame = new JFrame ("HyperCast XML viewer");
    		
    		/*
    		 * Create a new HypercastConfigurator instance, which is
    		 * an implementation of JPanel.
    		 */
    		Evan hc = new Evan(frame, simulation, simulation.getStatsName());
    		
    		frame.getContentPane().add (hc, BorderLayout.CENTER);
    		frame.setSize(Evan.FRAME_WIDTH,Evan.FRAME_HEIGHT);
    		frame.setVisible (true);
        }
        else if ( ae.getSource() == b_step )
        {
            
            if( simulation.network.isRunning() == false ) {
                if( simulation.start() == false ) {
                    printWarning("simulation can not be started");
                    return;
                }
            }
            //simulation.network.step();
            simulation.stepToNext();

            dw.repaint();
            
           
        }else if( ae.getSource() == b_advance )
        {
        	 if( simulation.network.isRunning() == false ) {
                 if( simulation.start() == false ) {
                     printWarning("simulation can not be started");
                     return;
                 }
             }
            if(animationRunning){
        		printWarning("Simulation is running");	
        	}else{
            	long ad_time = 0;
	            try {
	                ad_time = (long)((Float.valueOf(tf_advance_time.getText())).floatValue() * SimEventProcessor.TIME_PER_SEC);
	            }catch( Exception e){
	                printWarning(" invalid advance time format");
	                return;
	            }
	            if( ad_time <= 0 ){
	                printWarning(" can not go backward" );
	                return;
	            }
	
	            simulation.advance(ad_time*SimConfig.SIM_SLEEP/1000);
	            dw.repaint();
	        }
        }
        else{
        	
        	for(int i=0;i<mi_demos.length;i++){
        		if(ae.getSource() == mi_demos[i]){
        			this.loadNetwork(true, true, demoFiles[i]);
        			break;
        		}
        	}
        	
        }

    }


/**
 * hanle the events from checked menu items
 */
public synchronized void itemStateChanged(ItemEvent ie){
	
	if ( ie.getSource() == cbmi_display_on  )
		dw.setDisplayOn(cbmi_display_on.getState());
    if ( ie.getSource() == cbmi_display_logical_net  ) {
    	dw.setDisplayLogical(cbmi_display_logical_net.getState());
    	SimConfig.calcLogical = cbmi_display_logical_net.getState();
    }
    else if ( ie.getSource() == cbmi_display_physical_net  )
	dw.setDisplayPhysical(cbmi_display_physical_net.getState());
    else if ( ie.getSource() == cbmi_display_node_labels  )
	dw.setDisplayNodeLabels(cbmi_display_node_labels.getState());
    else if ( ie.getSource() == cbmi_display_any_messages  )
	dw.setDisplayMessages(cbmi_display_any_messages.getState());
    else if ( ie.getSource() == cbmi_display_message_labels  )
	dw.setDisplayMessageLabels(cbmi_display_message_labels.getState());
    else if( ie.getSource() == cbmi_display_wireless_range)
    	dw.setDisplayWirelessRange(cbmi_display_wireless_range.getState());
    else if ( ie.getSource() == cbmi_display_protocol  )
    	simulation.network.displayNodeProtocol = cbmi_display_protocol.getState();
    else if ( ie.getSource() == cbmi_display_logical_addr  )
    	simulation.network.displayNodeLogicalAddress = cbmi_display_logical_addr.getState();
    else if ( ie.getSource() == cbmi_display_physical_addr )
    	simulation.network.displayNodePhysicalAddress = cbmi_display_physical_addr.getState();
    else if ( ie.getSource() == cbmi_display_security )
    	simulation.network.displaySecuritySetting = cbmi_display_security.getState();
    
    
	else if ( ie.getSource() == cb_st_wr  ) 
            {
                setNodeType(GUI_DisplayWindow.NODE_STATIC_WIRED);

            }
    else if ( ie.getSource() == cb_st_wl  ) 
            {
                setNodeType(GUI_DisplayWindow.NODE_STATIC_WIRELESS);

            }
    else if ( ie.getSource() == cb_st_both  ) 
            {
                setNodeType(GUI_DisplayWindow.NODE_STATIC_BOTH);

            }
    else if ( ie.getSource() == cb_mb_wl  ) 
            {
                setNodeType(GUI_DisplayWindow.NODE_MOBILE_WIRELESS);

            }
	else if ( ie.getSource() == cb_drag  ) 
            {
                setMode(GUI_DisplayWindow.EDIT_ARROW);

            }
            
	else if ( ie.getSource() == cb_move  ) 
				{
					setMode(GUI_DisplayWindow.EDIT_MOVE);

				}
	else if ( ie.getSource() == cb_add_nodes  ) 
            {
                setMode(GUI_DisplayWindow.EDIT_NODE);

            }
            else if ( ie.getSource() == cb_delete_nodes  ) 
            {
                setMode(GUI_DisplayWindow.EDIT_DENODE);

            }
          	else if ( ie.getSource() == cb_add_links  ) 
            {
                setMode(GUI_DisplayWindow.EDIT_LINE);

            }
            else if ( ie.getSource() == cb_delete_links  ) 
            {
                setMode(GUI_DisplayWindow.EDIT_DELINE);

            }
          else if ( ie.getSource() == cb_start_or_stop_protocol  ) 
            {
                setMode(GUI_DisplayWindow.EDIT_START);
            }
            else if ( ie.getSource() == cb_show_routes  ) 
            {
                setMode(GUI_DisplayWindow.EDIT_TREE);

            }
            else if (ie.getSource() == cb_src){
            	setMode(GUI_DisplayWindow.EDIT_SRC);

        	}else if (ie.getSource() == cb_dst){
            	setMode(GUI_DisplayWindow.EDIT_DST);

        	}
        	        	
      	
    else {
    	/* cbmi_packet_protocol is never accessed/modified anywhere in the program
		int i;
        for( i=0; i<cbmi_packet_protocol.length; i++){
            if( ie.getSource() == cbmi_packet_protocol[i] ) break;
        }

        if( i < cbmi_packet_protocol.length ){

	        System.out.println("find matched protocols");
	        //it is one of the protocols
	        for( int j=0; j<cbmi_packet_protocol.length; j++){
	            if( j== i) cbmi_packet_protocol[j].setState(true);
	            else cbmi_packet_protocol[j].setState(false);
	        }
	
	        simulation.network = (simHCNetwork)OverlayNetwork.createInst(i, simulation.network.returnTopo() );
	        simulation.network.the_gui = this;
	        //lotosDebug.errorln("Changed to "+OverlayNetwork.getProtocols()[i]);
	        dw.repaint();
	    }
	    */
    	int i;
	    for( i=0; i<cbmi_mobile_mode.length; i++){
            if( ie.getSource() == cbmi_mobile_mode[i] ) break;
        }
		if( i < cbmi_mobile_mode.length){
			for( int j=0; j<cbmi_mobile_mode.length; j++){
	            if( j== i) cbmi_mobile_mode[j].setState(true);
	            else cbmi_mobile_mode[j].setState(false);
	        }
			dw.setMobileMode(cbmi_mobile_mode[i].getLabel());
	        
		}
	    
		
	}
    
    
    dw.repaint();
         	
}
	void setNodeType(int nt){
		dw.setNodeType(nt);
		
		cb_st_wr.setState(nt == GUI_DisplayWindow.NODE_STATIC_WIRED);
		cb_st_wl.setState(nt == GUI_DisplayWindow.NODE_STATIC_WIRELESS);
		cb_st_both.setState(nt == GUI_DisplayWindow.NODE_STATIC_BOTH);
		cb_mb_wl.setState(nt == GUI_DisplayWindow.NODE_MOBILE_WIRELESS);
		
		setMode(GUI_DisplayWindow.EDIT_NODE);
		
	}

   void setMode(int m) 
    {
        dw.setEditMode(m);
        
        // CheckboxGroup unnecessary.
        cb_move.setState(m == GUI_DisplayWindow.EDIT_MOVE);
        cb_drag.setState(m== GUI_DisplayWindow.EDIT_ARROW);
        
        cb_add_nodes.setState(m == GUI_DisplayWindow.EDIT_NODE);
        cb_delete_nodes.setState(m == GUI_DisplayWindow.EDIT_DENODE);
        
        cb_add_links.setState(m == GUI_DisplayWindow.EDIT_LINE);
        cb_delete_links.setState(m == GUI_DisplayWindow.EDIT_DELINE);
        
        
        cb_start_or_stop_protocol.setState(m == GUI_DisplayWindow.EDIT_START);
        cb_show_routes.setState(m == GUI_DisplayWindow.EDIT_TREE);
        cb_src.setState(m == GUI_DisplayWindow.EDIT_SRC);
        cb_dst.setState(m == GUI_DisplayWindow.EDIT_DST);
    }
/**
 * This function gets the name of a file to be saved or loaded.
 * It does this by opening a FileDialog window.  If an error occurs
 * or cancel is pressed, null is returned.   The argument should
 * be either FileDialog.SAVE or FileDialog.LOAD.
 */
    private String getFileName(int save_or_load_state)
    {
        String win_title = (save_or_load_state == FileDialog.LOAD) ? "Load File" : "Save File";
        FileDialog fd = new FileDialog(theApplicationWindow, win_title, save_or_load_state);
        fd.show();

        String result;
        if (fd.getFile() == null)   // Cancel button was pressed.
            result = null;
        else
            result = fd.getDirectory() + fd.getFile();

        fd.dispose();
        return result;
    }


    PhysicalNetwork getOverlay() { return simulation.network;}
/*
    OverlayNetwork createCurrentProtocol(TopoAdaptor ta){
        int i;
        for( i=0; i<cbmi_packet_protocol.length; i++){
            if( cbmi_packet_protocol[i].getState() ) break;
            else continue;
        }
        if( null == ta ) return OverlayNetwork.createInst(i);
        return OverlayNetwork.createInst(i, ta);
    }
*/
    class Animation extends Thread {
        boolean to_exit;
        int numsteps;
        
        Animation() { super("animation"); to_exit=false; }
        public void run(){
            long this_time=0;
        	long start_time=System.currentTimeMillis();;
        	int i=0;
            while( !to_exit && animationRunning) {
            	/*
            	if(!simulation.network.hasMoreEvents()){
            		exit=true;
            		break;	
            	}*/
            	synchronized( simulation.network ) {
            		MobiReflectiveLinear.maxX = dw.getSize().width;
            		MobiReflectiveLinear.maxY = dw.getSize().height;
                    simulation.advance( (long)( advance_time * SimEventProcessor.TIME_PER_SEC) );
                }
				i++;
                dw.repaint();
                
                this_time=System.currentTimeMillis();
                int sleep_time=SimConfig.SIM_SLEEP-(int)(this_time-start_time-i*SimConfig.SIM_SLEEP);
                
				if(sleep_time<0)sleep_time=0;
				try{
                    Thread.sleep(sleep_time);
                }catch( Exception e) {}
            }
        }

        void stopAnimation( ) { 
        	to_exit = true;
        	
        }
    }
	

	/* (non-Javadoc)
	 * @see java.awt.KeyEventDispatcher#dispatchKeyEvent(java.awt.event.KeyEvent)
	 */
	public boolean dispatchKeyEvent(KeyEvent evt) {
		int keyCode = evt.getKeyCode();
		
        if (evt.getID() == KeyEvent.KEY_PRESSED) {
            if(keyCode == KeyEvent.VK_CONTROL){
            	this.dw.appendSelectedNode = true;
            }
        }else if(evt.getID() == KeyEvent.KEY_RELEASED){
        	if(keyCode == KeyEvent.VK_CONTROL){
        		this.dw.appendSelectedNode = false;
            }
        	
        }

        return false;
	}
	
	/**
     * Enables the close button on the main frame for an application;
     * grants initial focus to the text input area.
     */
    /* package */ void enableWindowListener() // called from main method
    {
        theApplicationWindow.addWindowListener(
            new WindowAdapter()
            {
                public void windowClosing (final WindowEvent ignored)
                {
                    System.exit(0);
                }

                public void windowOpened (final WindowEvent ignored)
                {
                	
                }
            }
        );
    }


}

