package lotos.gui;

import hypercast.I_Stats;
import hypercast.util.XmlUtil;

import java.awt.Button;
import java.awt.Dialog;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.Label;
import java.awt.TextArea;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.ByteArrayOutputStream;

import org.apache.xpath.XPath;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * This dialog can show or set the statistics of a hypcercast node
 * 
 * @author Guangyu Dong, Feb 11, 2004
 */
public class StatsWindow extends Dialog implements ActionListener {
	TextField tf_statsname;
	TextArea tf_statsvalue;
	Label l_msg;
	Button b_get;
	Button b_set;
	Button b_readschema;
	Button b_writeschema;
	Button b_close;

	I_Stats stats;

	public StatsWindow(I_Stats _stats) {
		super(new Frame(), "Access Statistics", false);

		stats = _stats;

		this.setSize(650,450);
		this.setLocation(300, 100);

		tf_statsname = new TextField(80);
		tf_statsvalue = new TextArea();
		tf_statsvalue.setColumns(80);
		tf_statsvalue.setRows(20);
		//scroll right
		l_msg=new Label("                                                                                                                                                                                                          ");
		
		this.setLayout(new FlowLayout());
		this.add(new Label("XPath"));
		this.add(tf_statsname);
		this.add(new Label("Value"));
		this.add(tf_statsvalue);
		b_get = new Button("getStats");
		b_set = new Button("setStats");
		b_readschema = new Button("getReadSchema");
		b_writeschema = new Button("getWriteSchema");
		b_close = new Button("close");
		
		this.add(b_get);
		this.add(b_set);
		this.add(b_readschema);
		this.add(b_writeschema);
		this.add(b_close);
		this.add(l_msg);
		
		b_close.addActionListener(this);
		b_get.addActionListener(this);
		b_set.addActionListener(this);
		b_readschema.addActionListener(this);
		b_writeschema.addActionListener(this);

	}
	
	public void setStats(I_Stats _stats){
		stats = _stats;
	}
	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		l_msg.setText("");
		
		if (e.getSource() == b_close) {
			this.dispose();
			return;
		}
		XPath xpath = XmlUtil.createXPath(this.tf_statsname.getText());
		
		if(xpath==null){
			this.tf_statsvalue.setText("Invalid XPath: "+this.tf_statsname.getText());
			return;
		}
		if (e.getSource() == b_get) {
			
			try {
				Document doc = XmlUtil.createDocument();
				Element [] results = stats.getStats(doc, xpath);
				String result_txt = "";
				
				if(results.length<=0)
					result_txt = "<Empty>";
				
				for(int i=0;i<results.length;i++){
					doc.appendChild(results[i]);
					ByteArrayOutputStream byteOutput = new ByteArrayOutputStream();
					XmlUtil.writeXml(doc, byteOutput, true);
					result_txt += byteOutput.toString() +"\n";
					doc.removeChild(results[i]);
				}
				
				this.tf_statsvalue.setText(result_txt);
					
			} catch (Exception e1) {
				l_msg.setText(e1.toString());
			} 
			
		} 
		else if (e.getSource() == b_readschema) {
			try {
				Document doc = XmlUtil.createDocument();
				Element [] results = stats.getReadSchema(doc, xpath);
				String result_txt = "";
				
				if(results.length<=0)
					result_txt = "<Empty>";
				
				for(int i=0;i<results.length;i++){
					doc.appendChild(results[i]);
					ByteArrayOutputStream byteOutput = new ByteArrayOutputStream();
					XmlUtil.writeXml(doc, byteOutput, true);
					result_txt += byteOutput.toString() +"\n";
					doc.removeChild(results[i]);
				}
				
				this.tf_statsvalue.setText(result_txt);
					
			} catch (Exception e1) {
				l_msg.setText(e1.toString());
			} 
			
		} 
		else if (e.getSource() == b_writeschema) {
			try {
				Document doc = XmlUtil.createDocument();
				Element [] results = stats.getWriteSchema(doc, xpath);
				String result_txt = "";
				
				if(results.length<=0)
					result_txt = "<Empty>";
				
				for(int i=0;i<results.length;i++){
					doc.appendChild(results[i]);
					ByteArrayOutputStream byteOutput = new ByteArrayOutputStream();
					XmlUtil.writeXml(doc, byteOutput, true);
					result_txt += byteOutput.toString() +"\n";
					doc.removeChild(results[i]);
				}
				
				this.tf_statsvalue.setText(result_txt);
					
			} catch (Exception e1) {
				l_msg.setText(e1.toString());
			} 
			
		} 
		else if (e.getSource() == b_set) {
			try {
				Document doc = XmlUtil.createDocumentFromString(tf_statsvalue.getText());
				if(doc.getChildNodes().getLength()>0){
					Element results [] = stats.setStats(
						doc,
						xpath,
						(Element)doc.getChildNodes().item(0));
					
					String result_txt = "";
					
					if(results.length<=0)
						result_txt = "<Empty>";
					
					for(int i=0;i<results.length;i++){
						doc.appendChild(results[i]);
						ByteArrayOutputStream byteOutput = new ByteArrayOutputStream();
						XmlUtil.writeXml(doc, byteOutput, true);
						result_txt += byteOutput.toString() +"\n";
						doc.removeChild(results[i]);
					}
					
					this.tf_statsvalue.setText(result_txt);
					
				}
				
			} catch (Exception e1) {
				l_msg.setText(e1.toString());
			} 
		}
		
	}

}
