package lotos.history;

import hypercast.HTTP_ServerUtility;
import lotos.event.SimEventMsg;
import lotos.SimEventProcessor;

import hypercast.util.XmlUtil;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class MessageHistoryEntry implements EventHistoryEntry{
	float timestamp;
	boolean sent;
	SimEventMsg eventMsg;
	
	String src, dst;
	
	byte payload[];
	
	String node;
	
	public MessageHistoryEntry(long timestamp, boolean sent, SimEventMsg msg, String src, String dst, String nodeID){
		this.timestamp = timestamp/SimEventProcessor.TIME_PER_SEC;
		this.sent = sent;
		this.eventMsg = msg;
		
		this.src = src;
		this.dst = dst;
		this.node = nodeID;
		
		payload = eventMsg.getHCMessage().toByteArray();
	}
	public float getTimestamp(){
		return timestamp;
	}
	public String getSummary(){
		if(sent){
			return "Message sent to "+dst; 
		}else{
			return "Message received from "+src;
		}
	}
	public String getDetails(){
		String details = "";
		
		details += "Src="+src+"    Dst="+dst+"    Size="+payload.length+"\n\n";
		
		details += "Message.toString() = "+eventMsg.getHCMessage().toString()+"\n\n";
		
		for(int i=0;i<payload.length;i++){
			byte []test = new byte[1];
			test[0]=payload[i];
			details += HTTP_ServerUtility.toHexString(test)+" ";
		}
		
		
		
		return details;
	}
	public String logEntry() {
		String entry = "<event node=\""+node+"\" type=\"";
		if(sent) {
			entry+="MessageSent\">\n";
			entry+="\t<timestamp>"+timestamp+"</timestamp>\n";
			entry+="\t<destination>"+dst+"</destination>\n";
			entry+="\t<messageToString>"+eventMsg.getHCMessage().toString()+"</messageToString>\n";
			entry+="\t<payload>";
			for(int i=0;i<payload.length;i++){
				byte []test = new byte[1];
				test[0]=payload[i];
				entry += HTTP_ServerUtility.toHexString(test)+" ";
			}
			entry+="</payload>\n";
		} else {
			entry+="MessageReceived\">\n";
			entry+="\t<timestamp>"+timestamp+"</timestamp>\n";
			entry+="\t<source>"+src+"</source>\n";
			entry+="\t<messageToString>"+eventMsg.getHCMessage().toString()+"</messageToString>\n";
			entry+="\t<payload>";
			for(int i=0;i<payload.length;i++){
				byte []test = new byte[1];
				test[0]=payload[i];
				entry += HTTP_ServerUtility.toHexString(test)+" ";
			}
			entry+="</payload>\n";			
		}
		entry+="</event>";
		
		return entry;
	}
}
