package lotos.nodeappl;

import hypercast.HyperCastConfig;
import hypercast.HyperCastStatsException;
import hypercast.I_OverlayMessage;
import hypercast.SimpleStats;
import hypercast.util.XmlUtil;
import hypercast.HyperCastConfigException;

import java.awt.Color;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.apache.xpath.XPath;

import lotos.LotosHyperCastAppl;
import lotos.SimConfig;
import lotos.SimEventProcessor;
import lotos.event.SimEvent;
import lotos.event.SimEventTimer;
/**
 * implementation of HelloWorld as an application in lotos. this 
 * implementation is essentially a finite state machine, with 
 * inputs from ReceiveCallback and receiveEvent.
 * 
 * the resend time in this application is changable through the 
 * configuration file.
 * @author Zhi An
 *
 */
public class HelloWorldAppl extends LotosHyperCastAppl {
	int resendTime = 5000000;
	int helloRec;
	int helloSent;
	boolean running;
	Document configDoc;
	
	/**
	 * This is the constructor that lotos calls when creating a node
	 * that runs this application. 
	 * @param config
	 * @param applConfig
	 */	
	public HelloWorldAppl(HyperCastConfig config, Document applConfig) {
		super(config); // the super constructor MUST be called.		
		configDoc = applConfig;
	}
	
	/**
	 * this is the color of the node in the gui.
	 * it is completely arbitrary.
	 */
	public Color getColor() {
		return Color.CYAN;
	}

	/**
	 * this is the shape of the node in the gui.
	 * it is completely arbitrary.
	 * the shape IDs are the same as those used in the
	 * protocol agents.
	 */
	public int getShape() {
		
		return shapeSquare;
	}

	public synchronized void  receiveEvent(SimEvent se) {
		if(!running)
			return;
		se.setTime(SimEventProcessor.getSimTime()+resendTime);
		SimEventProcessor.insertEvent(se);
		
		byte[] messagePayload = "Hello World".getBytes();
		I_OverlayMessage message = createMessage(messagePayload);
		sendToAll(message);
		helloSent++;
	}

	public void ReceiveCallback (final I_OverlayMessage message) {
//		 Extract the payload and the logical address of the source
		byte[] data = message.getPayload();
       String Src = message.getSourceAddress().toString();

       // Print the payload (Skip messages sent by this program) 
       if(!message.getSourceAddress().equals(getLogicalAddress()))
           println(getLogicalAddress().toString()+ " received <"+ new String(data) + "> from logical address: " + Src + ".");
		
		helloRec++;
	}

	public void restart() {
		helloRec=0;
		helloSent=0;

	}

	public void start() {
		try {
			Node nProperties = configDoc.getElementsByTagName("Properties").item(0);
			Node nResendTime = ((Element)nProperties).getElementsByTagName("ResendTime").item(0);
			String sResendTime = nResendTime.getFirstChild().getNodeValue();
			resendTime = Integer.valueOf(sResendTime).intValue();			
		} catch (Exception e) {
			println("resend time not specified, using default");
		}
		println("Starting HelloWorld, resend time: " +resendTime);
		
		helloRec=0;
		helloSent=0;
		running = true;
		SimEventTimer timer = new SimEventTimer(SimEventProcessor.getSimTime() + resendTime, this, new Object());
		SimEventProcessor.insertEvent(timer);
	}

	/**
	 * The application must stop when the node stops. It must be
	 * prevented from sending messages after it stops, or
	 * HyperCastAppl will throw an exception that stops the 
	 * simulation.
	 */
	public synchronized void stop() {
		running = false;

	}
	class stat_helloSent extends SimpleStats{
		protected String getStats()
		{
			return Integer.toString(helloSent);
		}

		protected String setStats(String newValue) throws HyperCastStatsException
		{
			throw new HyperCastStatsException(statisticsName, HyperCastStatsException.WriteToAReadOnlyValue);
		}

		public Element[] getReadSchema(Document doc, XPath xpath) throws HyperCastStatsException
		{
			return XmlUtil.createSchemaElement(doc/* document instance */, statisticsName/* element name */, "xsd:Integer"/* type */, 
												null/* restriction base */, null/* pattern value */);
		}
	}
	
	class stat_helloRec extends SimpleStats{
		protected String getStats()
		{
			return Integer.toString(helloRec);
		}

		protected String setStats(String newValue) throws HyperCastStatsException
		{
			throw new HyperCastStatsException(statisticsName, HyperCastStatsException.WriteToAReadOnlyValue);
		}

		public Element[] getReadSchema(Document doc, XPath xpath) throws HyperCastStatsException
		{
			return XmlUtil.createSchemaElement(doc/* document instance */, statisticsName/* element name */, "xsd:Integer"/* type */, 
												null/* restriction base */, null/* pattern value */);
		}
	}
}
