package lotos.nodeappl;

import hypercast.HyperCastStatsException;
import hypercast.HyperCastConfig;
import hypercast.I_OverlayMessage;
import hypercast.SimpleStats;
import hypercast.util.XmlUtil;
import hypercast.StatsProcessor;

import java.awt.Color;

import org.apache.xpath.XPath;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import lotos.LotosHyperCastAppl;
import lotos.SimEventProcessor;
import lotos.event.SimEvent;
import lotos.event.SimEventTimer;

public class HelloWorldAppl_Thread extends LotosHyperCastAppl implements Runnable {
	Thread myThread;
	int resendTime = 5000000;
	int helloRec;
	int helloSent;
	/*
	 * 
	 */
	public HelloWorldAppl_Thread(HyperCastConfig config, Document applConfig) {
		super(config); // the super constructor MUST be called.		
		
	}

	public synchronized void run() {
		myThread = Thread.currentThread();
		while(true) {
			try {
				wait();
			} catch (InterruptedException ie) {
				break;
			}
			byte[] messagePayload = "Hello World".getBytes();
			I_OverlayMessage message = createMessage(messagePayload);
			sendToAll(message);
			helloSent++;
			
			notifyAll();
		}
		notifyAll();
	}

	public void ReceiveCallback (final I_OverlayMessage message) {
//		 Extract the payload and the logical address of the source
		byte[] data = message.getPayload();
        String Src = message.getSourceAddress().toString();

        // Print the payload (Skip messages sent by this program) 
        if(!message.getSourceAddress().equals(getLogicalAddress()))
            System.out.println(getLogicalAddress().toString()+ " received <"+ new String(data) + "> from logical address: " + Src + ".");
		
		helloRec++;
	}

	public Color getColor() {
		return color;
	}

	public int getShape() {
		return 0;
	}

	public synchronized void receiveEvent(SimEvent event) {	
		event.setTime(SimEventProcessor.getSimTime()+resendTime);
		SimEventProcessor.insertEvent(event);	
		notify();
	}

	public void start() {
		helloRec = 0;
		helloSent = 0;
		statsProcessor.addStatsElement("HelloSent", new stat_helloSent(), 1, 1);
		statsProcessor.addStatsElement("HelloRec", new stat_helloRec(), 1, 1);
		
		if(myThread == null)
			new Thread(this, getName()).start();
		SimEventTimer timer = new SimEventTimer(SimEventProcessor.getSimTime() + resendTime, this, new Object());
		SimEventProcessor.insertEvent(timer);
		
	}

	public void restart() {
		helloSent = 0;
		helloRec = 0;		
	}

	public synchronized void stop() {
		if(myThread != null)
			myThread.interrupt();
	}
	
	class stat_helloSent extends SimpleStats{
		protected String getStats()
		{
			return Integer.toString(helloSent);
		}

		protected String setStats(String newValue) throws HyperCastStatsException
		{
			throw new HyperCastStatsException(statisticsName, HyperCastStatsException.WriteToAReadOnlyValue);
		}

		public Element[] getReadSchema(Document doc, XPath xpath) throws HyperCastStatsException
		{
			return XmlUtil.createSchemaElement(doc/* document instance */, statisticsName/* element name */, "xsd:Integer"/* type */, 
												null/* restriction base */, null/* pattern value */);
		}
	}
	
	class stat_helloRec extends SimpleStats{
		protected String getStats()
		{
			return Integer.toString(helloRec);
		}

		protected String setStats(String newValue) throws HyperCastStatsException
		{
			throw new HyperCastStatsException(statisticsName, HyperCastStatsException.WriteToAReadOnlyValue);
		}

		public Element[] getReadSchema(Document doc, XPath xpath) throws HyperCastStatsException
		{
			return XmlUtil.createSchemaElement(doc/* document instance */, statisticsName/* element name */, "xsd:Integer"/* type */, 
												null/* restriction base */, null/* pattern value */);
		}
	}
}
