/***************************************************************************/
/*       Lotos, an extensible overlay network simulator                    */
/*       MNG Group                                                         */
/*       University of Virginia                                            */
/*       July, 2002                                                        */
/***************************************************************************/
package lotos;

import lotos.*; 
import lotos.event.*;
import lotos.gateway.LotosGateway;
import lotos.gui.Evan;
import lotos.gui.EventWindow;
import lotos.gui.GUI;
import lotos.gui.StatsWindow;

import hypercast.*;
import hypercast.events.NODE_NEIGHBORHOODCHANGED;
import hypercast.events.NOTIFICATION_EVENT;
import hypercast.util.XmlUtil;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.text.ParseException;
import java.util.*;
import java.net.*;

import javax.swing.JFrame;

import org.apache.xpath.XPath;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 * This class implements a network running live Hypercast node
 * 
 * @author Guangyu Dong, Feb 3, 2004
 */

public class simHCNetwork extends PhysicalNetwork implements ActionListener, ItemListener{
	
	/** Config object */
	private SimConfig config;
	
	/** Adapter manger handling creating adapters */
	/*package*/ LotosAdapterManager adpt_mngr;
	
	/** Current protocol */
	private ProtocolEntry currentProtocol = null;
	private SecuritySettingEntry currentSecurity = null;
	private ApplicationEntry currentAppl = null;
	private GatewayEntry currentGateway = null;
	
	/** Display properties */
	public boolean displayNodeProtocol = true;
	public boolean displayNodeLogicalAddress = true;
	public boolean displayNodePhysicalAddress = true;
	public boolean displaySecuritySetting = true;
	
	private boolean loadSaveFlag_logical = true;
	private boolean loadSaveFlag_physical = true;
	
	public GUI the_gui;
	/**
	 * a vector containing all the protocols running in the network.
	 */
	public Vector protocols;
	
	MenuItem selectedProtocolMenu;
	CheckboxMenuItem security_menus[];
	
	/** Constructor
	 * build the network with some topology
	 * @param ta    The topolology adapter
	 * @throws IOException
	 * @throws ParseException
	 * @throws FileNotFoundException
	 */
	/*
	public simHCNetwork(TopoAdaptor ta){
		super(ta);
		
		initConfig();
		
	}
	*/
	/** Constructor
	 * build the network with some topology
	 * @throws IOException
	 * @throws ParseException
	 * @throws FileNotFoundException
	 */
	public simHCNetwork(){
		super();
		
		protocols = new Vector();
		
		initConfig();
	}
	private void initConfig(){
		try {
			config = new SimConfig(this, "SimHypercast.xml");
			adpt_mngr = new LotosAdapterManager();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		}
		if(config!=null && config.getProtocolCount()>0){
			//currentProtocol = config.getProtocolByName(config.getProtocolNames()[0]);
		}
		
	}
	/**
	 * Activate a protocol node
	 * @param o			the physical node
	 * @param type		the type of hypercast node
	 * @return		true if activation succeeded.
	 */
	
	private boolean activateNode(PhysicalNode o, ProtocolEntry pEntry, SecuritySettingEntry sEntry, ApplicationEntry aEntry, String info) {
		if(pEntry == null)
			return false;
		o.nodeInfo = new NodeInfo();
		o.nodeInfo.protocol = pEntry;
		o.nodeInfo.security = sEntry;
		o.nodeInfo.application = aEntry;		
		
		//if( dtServer == null ) return false;
		if (o == null)
			return false;

		if (active_nodes.contains(o)) {
			//return false;
			deactivateNode(o);
		}
		//create adapter
		if (adpt_mngr == null) {
			adpt_mngr = new LotosAdapterManager();
		}
		if (config == null) {
			initConfig();
		}

		//create protocol node
		//set up the configuration for the socket
		Document configdoc = pEntry.getConfig();
		configdoc = (Document)configdoc.cloneNode(true);
		HyperCastConfig config = new HyperCastConfig(configdoc);
		SimPhysicalAddress addr = LotosAdapterManager.createAddress(o.id, (InetAddress)LOCAL_ADDRESS.get(0));
		if(SimConfig.hybrid) {
			for(int i=0;i<LOCAL_ADDRESS.size();i++) {
				addr = LotosAdapterManager.createAddress(o.id, (InetAddress)LOCAL_ADDRESS.get(i));
				lotosDebug.println("registering address "+addr + " - " + o);
				LotosAdapterManager.addr_list.put(addr.toString(), o);
			}
		} else {
			LotosAdapterManager.addr_list.put(addr.toString(), o);
		}
		/*
		 * set the physical addresses for the lotos adapters based on the physical node id.
		 */
		config.setTextAttribute(XmlUtil.createXPath("/Public/NodeAdapter/NodeAdptLotos/PhysicalAddress"), addr.toString());
		config.setTextAttribute(XmlUtil.createXPath("/Public/SocketAdapter/ScktAdptLotos/PhysicalAddress"), addr.toString());
		
		//add security to the configuration if necessary
		if(sEntry!=null) {
			HyperCastConfig sec_config = sEntry.config;
			config.MergeConfig(sec_config.getDocument());
			/*
			 * MergeConfig only merges the public attributes. the private attributes must be manually copied.
			 * - Because the HyperCastAppl cannot set any of these private attributes, security cannot be done
			 * with applications based on HyperCastAppl.
			 * 
			 */
			
			String ksPassword =
				sec_config.getPrivateTextAttribute (XmlUtil.createXPath("/Private/KeyStorePassword"));							
			String pkAlias =
				sec_config.getPrivateTextAttribute (XmlUtil.createXPath("/Private/PrivateKeyAlias"));
			String pkPassword =
				sec_config.getPrivateTextAttribute (XmlUtil.createXPath("/Private/PrivateKeyPassword"));
			String groupKey = 
				sec_config.getPrivateTextAttribute (XmlUtil.createXPath("/Private/GroupKey"));
			
			config.setPrivateTextAttribute(XmlUtil.createXPath("/Private/KeyStorePassword"), ksPassword);
			config.setPrivateTextAttribute (XmlUtil.createXPath("/Private/PrivateKeyAlias"), pkAlias);
			config.setPrivateTextAttribute (XmlUtil.createXPath("/Private/PrivateKeyPassword"), pkPassword);
			config.setPrivateTextAttribute (XmlUtil.createXPath("/Private/GroupKey"), groupKey);
		}
		try {
			//apply any protocol specific configuration changes, e.g. for DT set the logical address coordinates
			I_SimHyperCastAgent agent = (I_SimHyperCastAgent)pEntry.getAgentClass().newInstance();
			agent.finishConfig(config, o);
						
			//configuration done, create the agent, protocol, and socket.
			if(aEntry == null) { //no application running, create a normal protocol
				
				//Protocol protocol = createProtocolNode(o, entry, security, config, info);
				Protocol protocol = new Protocol(this, o, agent);
				protocol.protocolName = pEntry.getName();
				if(sEntry!=null)
					protocol.securitySetting = sEntry.name;
				else
					protocol.securitySetting = "None";
				
				o.setProtocol(protocol);
				/*
					A_NodeApplication appl = (A_NodeApplication)applInfo.applClass.newInstance();
					appl.initAppl(applInfo.getConfig());
					protocol.setHCNode(agent.createSocket(o.getAddress(), config, info, addr, appl.getCallback(), appl.getNotificationHandler(), appl.getInterceptionCallback()));
					protocol.application = appl;
					appl.setSocket(protocol.hc_socket);
				} else*/
				//create the hypercast socket using the agent.
				protocol.setHCNode(agent.createSocket(config, info, addr, new DummyCallback(), null, null));	
				active_nodes.addElement(o);
				
				//if there is a lotos gateway, register the node with the gateway.
				if(gateway_nodes.size()>0) {
					LotosGateway gateway = (LotosGateway)((PhysicalNode)gateway_nodes.get(0)).getProtocol();
					gateway.registerAdapters(protocol);
				}
				
				//if (this.isRunning())
					protocol.startProtocol();
				
				protocols.add(protocol);
				
				return true;
				
			} else { //there will be an application, create a LotosHyperCastAppl
				
				try {
					/*
					 * the lotos adapters use the static variables in LotosAdapterManager to make handles to
					 * themselves in the protocol of the physical node they're on. But setting the application
					 * as the protocol on that node is impossible since the adapters are created on application
					 * initialization, which creates the actual socket. So the dummy protocol is just a temporary
					 * holder for the adapters, whose handles can then be passed to the application.
					 */
					DummyProtocol dummy = new DummyProtocol();
					o.setProtocol(dummy);
										
					LotosHyperCastAppl appl = (LotosHyperCastAppl)aEntry.getApplClass().getConstructor(new Class[] {HyperCastConfig.class, Document.class}).newInstance(new Object[] {config, aEntry.getConfig()});
					appl.setNodeAdapter(dummy.getNodeAdapter());
					appl.setScktAdapter(dummy.getScktAdapter());
					appl.setNetwork(this);
					
					o.setProtocol(appl);
					
					active_nodes.add(o);
					
					if(gateway_nodes.size()>0) {
						LotosGateway gateway = (LotosGateway)((PhysicalNode)gateway_nodes.get(0)).getProtocol();
						gateway.registerAdapters(appl);
					}
					//if (this.isRunning())
						appl.startProtocol();
					protocols.add(appl);

					return true;
					
				} catch(Exception e) {
					e.printStackTrace();
				}
				
				return true;
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return false;
	}
	
	/**
	 * Deactivate the protocol node
	 * @param o		the physical node
	 * @return		true if deactivation succeeded.
	 */
	public boolean deactivateNode(PhysicalNode o) {
		try {
		if (active_nodes.contains(o)) {
			
			I_ProtocolInterface protocol = o.getProtocol();
			if(protocol==null)
				return false;
			if(protocol instanceof Protocol)
				((Protocol)protocol).agent.nodeRemoved(((Protocol)protocol).hc_socket);
			protocols.remove(protocol);
			
			//get adapter
			LotosAdapter adapter = o.getProtocol().getNodeAdapter();
			/*
			 * free up the port at the gateways
			 */
			if(SimConfig.hybrid && gateway_nodes.size()>0) {
				PhysicalNode n = (PhysicalNode) gateway_nodes.get(0);
				LotosGateway gateway = (LotosGateway) n.protocol;
				gateway.closeAddress((SimPhysicalAddress)adapter.getPhysicalAddress());
			}
			
			o.getProtocol().clearLogicalEdges();
			o.getProtocol().stopProtocol();

			o.setProtocol(null);
			o.forwarding = true;

			//remove adapter from the manager
			adpt_mngr.removeAdapter(adapter);

			active_nodes.remove(o);
					
			return true;

		} else if(gateway_nodes.contains(o)){
			LotosGateway gateway = (LotosGateway)o.getProtocol();
			if(gateway==null)
				return false;
			gateway_nodes.remove(o);
			/*
			 * check if there aren't any gateways left. if no gateways, then
			 * close all the external ports left open.
			 */
			if(gateway_nodes.size()==0)
				gateway.reset();
			o.setProtocol(null);
			o.forwarding = true;
			return true;
		}
		}catch (Exception e) {
			e.printStackTrace();
		}
		return false;
	}

	/* (non-Javadoc)
	 * @see PhysicalNetwork#getLogicalNextHop(PhysicalNode, PhysicalNode)
	 */
	public PhysicalNode getLogicalNextHop(PhysicalNode src, PhysicalNode dst) {
		I_ProtocolInterface srcp, dstp;
		srcp = src.getProtocol();
		dstp = dst.getProtocol();

		if (srcp != null && dstp != null) {
			I_LogicalAddress la = srcp.getParent(dstp.getLogicalAddress());
			if (la != null) {
				PhysicalNode dstn =(PhysicalNode)adpt_mngr.searchAddress((SimPhysicalAddress)getPhysicalAddress(la));
				return dstn;
			}
		}

		return null;
	}

	/* (non-Javadoc)
	 * @see PhysicalNetwork#start()
	 */
	public boolean start() {
		this.simRunning = true;
		for (int i = 0; i < active_nodes.size(); i++) {
			PhysicalNode onode = (PhysicalNode)active_nodes.elementAt(i);
			onode.getProtocol().startProtocol();
		}
		return true;
	}

	/* (non-Javadoc)
	 * @see PhysicalNetwork#restart()
	 */
	public void restart() {
		super.restart();
		this.simRunning = false;
		for (int i = 0; i < active_nodes.size(); i++) {
			PhysicalNode onode = (PhysicalNode)active_nodes.elementAt(i);
			onode.getProtocol().restartProtocol();
		}
	}

	/* (non-Javadoc)
	 * @see PhysicalNetwork#buildOverlay()
	 */
	/*
	public boolean buildOverlay() {
		return false;
	}
	*/
	/*
	private final String[] treeMenu;
	=
		{
			"Activate",
			"Deactivate",
			"Stats",
			"-----------",
			"SPT",
			"DTServerJoin",
			"DTServer",
			"DTMulticast Node",
			"DTBuddyList Node",
			"HyperCube Node" };
			*/
	//private String currentProtocol = null;
	//private String currentSecuritySetting = null;
	/* (non-Javadoc)
	 * @see PhysicalNetwork#getMenus()
	 */
	PopupMenu protocolMenu = null;
	PopupMenu securityMenu;
	PopupMenu applMenu = null;
	PopupMenu gatewayMenu = null; 
	
	public PopupMenu getApplMenu(){
		if(applMenu==null){
			applMenu = new PopupMenu("Applications");
			String applications[] = config.getApplicationNames();
			CheckboxMenuItem pm = new CheckboxMenuItem("None");
			applMenu.add(pm);
			pm.addItemListener(this);
			
			pm.setState(true);
			for(int i=0;i<applications.length;i++){
				pm = new CheckboxMenuItem(applications[i]);
				applMenu.add(pm);
				pm.addItemListener(this);
					
			}
		}
		
		return applMenu;
	}
	
	public PopupMenu getGatewayMenu(){
		if(gatewayMenu==null){
			gatewayMenu = new PopupMenu("GatewayAdapters");
			String adapters[] = config.getGatewayNames();
			CheckboxMenuItem pm;
			for(int i=0;i<adapters.length;i++){
				pm = new CheckboxMenuItem(adapters[i]);
				gatewayMenu.add(pm);
				pm.addItemListener(this);
					
			}
		}
		
		return gatewayMenu;
	}
	
	public PopupMenu getProtocolMenu(){
		if(protocolMenu==null){
			protocolMenu = new PopupMenu("Protocols");
			String protocols[] = config.getProtocolNames();
			CheckboxMenuItem pm = new CheckboxMenuItem("None");
			protocolMenu.add(pm);
			pm.addItemListener(this);
			
			pm.setState(true);
			for(int i=0;i<protocols.length;i++){
				pm = new CheckboxMenuItem(protocols[i]);
				protocolMenu.add(pm);
				pm.addItemListener(this);
					
			}
		}
		
		return protocolMenu;
	}
	public PopupMenu getSecurityMenu(){
		if(securityMenu==null){
			securityMenu = new PopupMenu("Security");
			CheckboxMenuItem secSubMenu = new CheckboxMenuItem("None");
			secSubMenu.addItemListener(this);
			if(currentSecurity == null)
				secSubMenu.setState(true);
			securityMenu.add(secSubMenu);
			String secNames[] = config.getSecuritySettingNames();
			security_menus = new CheckboxMenuItem[secNames.length+1];
			security_menus[0] = secSubMenu;
			
			for(int i=0;i<secNames.length;i++){
				secSubMenu =  new CheckboxMenuItem(secNames[i]);
				secSubMenu.addItemListener(this);
				securityMenu.add(secSubMenu);
				security_menus[i+1]=secSubMenu;
				if(currentSecurity!=null && currentSecurity.name.equals(secNames[i])){
					secSubMenu.setState(true);
				}
			}
		}
		
		return securityMenu;
	}
	
	public PopupMenu getMenus() {
		/* initiate menu */
		PhysicalNode selectedNode = this.getFirstSelectedNode();
		
		String[] protocolNames = config.getProtocolNames();
		
		PopupMenu allmenu = new PopupMenu("SimHypercast");
		MenuItem mi;
		
		
		mi = new MenuItem("Activate");
		if(selectedNode==null || selectedNode.getProtocol()!=null)
			mi.setEnabled(false);
		
		mi.addActionListener(this);
		allmenu.add(mi);
		mi = new MenuItem("Deactivate");
		if(selectedNode!=null){
			I_ProtocolInterface hcNode = selectedNode.getProtocol();
			if(hcNode==null)
				mi.setEnabled(false);
		}else
			mi.setEnabled(false);
		
		mi.addActionListener(this);
		allmenu.add(mi);
		mi = new MenuItem("Node Statistics");
		/*
		if(selectedNode!=null){
			Protocol hcNode = (Protocol)selectedNode.getProtocol();
			if(hcNode==null)
				mi.setEnabled(false);
		}else
			mi.setEnabled(false);
		*/
		if(selectedNode==null)
			mi.setEnabled(false);
		mi.addActionListener(this);
		allmenu.add(mi);
		
		mi = new MenuItem("Event History");
		if(selectedNode!=null){
			I_ProtocolInterface hcNode = selectedNode.getProtocol();
			if(hcNode==null)
				mi.setEnabled(false);
		}else
			mi.setEnabled(false);
		
		mi.addActionListener(this);
		allmenu.add(mi);
				
		mi = new MenuItem("Set Gateway");
		if(selectedNode==null || selectedNode.getProtocol()!=null || currentGateway == null || gateway_nodes.size()>0)
			mi.setEnabled(false);
		
		mi.addActionListener(this);
		allmenu.add(mi);
		/*
		if(selectedNodes.size()==1 && selectedNode.getProtocol()!=null && selectedNode.getProtocol() instanceof Protocol){
			Protocol pNode = (Protocol)selectedNode.getProtocol();
			
			
			
			for(int i=0;i<protocolNames.length;i++){
				ProtocolEntry entry = config.getProtocolByName(protocolNames[i]);
				if(entry.getAgent().getName().equals(pNode.agent.getName())){
					
					String protocolMenus[] = entry.getAgent().getMenuItems();
					
					if(protocolMenus!=null && protocolMenus.length>0){
					
						//mi = new MenuItem("------");
						//allmenu.add(mi);
						
						Menu pmenu = new Menu(entry.getAgent().getName());
						selectedProtocolMenu = pmenu;
						
						allmenu.add(pmenu);
						
						for(int j=0;j<protocolMenus.length;j++){
							mi = new MenuItem(protocolMenus[j]);
							mi.addActionListener(this);
							pmenu.add(mi);
						}
						
						allmenu.add(pmenu);
					}
				}
			}
		}
		*/
		
		return allmenu;
	}

	public void showStats(PhysicalNode onode) {
		
		if (onode == null)
			return;
		/*
		if (onode.getProtocol() == null)
			return;

		I_Node hcnode = ((Protocol)onode.getProtocol()).getHCNode();
		if (hcnode == null)
			return;
			
		StatsWindow statswin = new StatsWindow(hcnode);
		*/
		/*
		if(onode.getProtocol()!=null && onode.getProtocol() instanceof Protocol) {
			I_Stats hcnode = ((Protocol)onode.getProtocol()).getHCNode();
			if (hcnode != null)
				onode.registerHCNode(hcnode);
		}
		
		StatsWindow statswin = new StatsWindow(onode);
		statswin.show();
		*/
		JFrame frame = new JFrame ("HyperCast XML viewer");
		
		/*
		 * Create a new HypercastConfigurator instance, which is
		 * an implementation of JPanel.
		 */
		Evan hc = new Evan(frame, onode, onode.getStatsName());
		
		frame.getContentPane().add (hc, BorderLayout.CENTER);
		frame.setSize(Evan.FRAME_WIDTH,Evan.FRAME_HEIGHT);
		frame.setVisible (true);
	}
	
	public void showEventHistory(PhysicalNode onode) {
		
		if (onode == null)
			return;
		if (onode.getProtocol() == null)
			return;

		I_ProtocolInterface hcnode = onode.getProtocol();
		if (hcnode == null)
			return;
		
		EventWindow eventwin = new EventWindow(hcnode);
		eventwin.show();
	}

	public void setLoadSaveFlag(boolean logical, boolean physical){
		
		if(!logical && !physical){
			throw new IllegalArgumentException("Need load/save either logical or physical network.");
		}
		
		this.loadSaveFlag_logical = logical;
		this.loadSaveFlag_physical = physical;
	}
	
	/* (non-Javadoc)
	 * @see PhysicalNetwork#protocolReader(java.lang.String)
	 */
	public boolean protocolReader(String fname) {
		Document doc;
		InputStream xmlinput = FileResourceManager.openFile(fname);
		try {
			doc = XmlUtil.createDocument(xmlinput, fname);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			return false;
		} catch (IOException e) {
			e.printStackTrace();
			return false;
		}
		
		if(this.loadSaveFlag_physical){
			nodes.clear();
		}
		
		active_nodes.clear();
		adpt_mngr.reset();
		
		Element LoTosProtocolSetting = (Element)doc.getElementsByTagName("PhysicalNetwork").item(0);
		NodeList nl = LoTosProtocolSetting.getElementsByTagName("PhysicalNode");
		
		for(int i=0;i<nl.getLength();i++){
			Element ovlynode = (Element)nl.item(i);
			String xyaddress = ovlynode.getElementsByTagName("XYAddress").item(0).getChildNodes().item(0).getNodeValue();
			PhysicalNode onode =null;
			
			if(this.loadSaveFlag_physical && ovlynode.getElementsByTagName("Physical").getLength()>0){
				Element phyNode = (Element)ovlynode.getElementsByTagName("Physical").item(0);
				Element mobiMode = (Element)phyNode.getElementsByTagName("MobilityMode").item(0);
				String mobiStr = mobiMode.getChildNodes().item(0).getNodeValue();
				float speed = 0;

				Element wired = (Element)ovlynode.getElementsByTagName("WiredConnection").item(0);
				String wiredStr = wired.getChildNodes().item(0).getNodeValue().trim();
				
				Element wireless = (Element)ovlynode.getElementsByTagName("WirelessConnection").item(0);
				String wirelessStr = wireless.getChildNodes().item(0).getNodeValue().trim();
				XYAddress addr = new XYAddress(xyaddress);
				
				if(mobiStr.equals("None"))
					mobiStr = null;
				else {
					Element mobiSpeed = (Element)phyNode.getElementsByTagName("MobilitySpeed").item(0);
					String mobiSpeedStr = mobiSpeed.getChildNodes().item(0).getNodeValue();
					speed = Float.parseFloat(mobiSpeedStr);
				}
				
				PhyNodeProp props = new PhyNodeProp(addr.x, addr.y, mobiStr, speed, wiredStr.equalsIgnoreCase("yes"), wirelessStr.equalsIgnoreCase("yes"));
					
				onode = this.addNode(props);
				if(wirelessStr.equalsIgnoreCase("yes") && ovlynode.getElementsByTagName("WirelessMAC").getLength() > 0) {
					Element mac = (Element)ovlynode.getElementsByTagName("WirelessMAC").item(0);
					String senserange = mac.getElementsByTagName("SenseRange").item(0).getFirstChild().getNodeValue();
					String transrange = mac.getElementsByTagName("TransRange").item(0).getFirstChild().getNodeValue();
					String bandwidth = mac.getElementsByTagName("Bandwidth").item(0).getFirstChild().getNodeValue();
					
					onode.wl_MAC.SENSE_RANGE = Float.valueOf(senserange).floatValue();
					onode.wl_MAC.SIGNAL_RANGE = Float.valueOf(transrange).floatValue();
					onode.wl_MAC.bandwidth = Integer.valueOf(bandwidth).intValue();
					
				}
									
			}else{	
				onode = findClosest(new XYAddress(xyaddress), 0);
			}
			
			if(onode==null)
				onode = this.addNode(new XYAddress(xyaddress));
			
			if(onode!=null && this.loadSaveFlag_logical && ovlynode.getElementsByTagName("Protocol").getLength()>0){
				Element protocol = (Element)ovlynode.getElementsByTagName("Protocol").item(0);
				
				String protocolName = protocol.getElementsByTagName("ProtocolName").item(0).getChildNodes().item(0).getNodeValue();
				String securitySetting = protocol.getElementsByTagName("SecuritySetting").item(0).getChildNodes().item(0).getNodeValue();
				String protocolInfo = protocol.getElementsByTagName("ProtocolInfo").item(0).getChildNodes().item(0).getNodeValue();
				String physicalAddress = protocol.getElementsByTagName("PhysicalAddress").item(0).getChildNodes().item(0).getNodeValue();
				
				ProtocolEntry pEntry = config.getProtocolByName(protocolName);
				SecuritySettingEntry sEntry = config.getSecuritySettingByName(securitySetting);
				
				//this.activateNode(onode, pEntry, sEntry, protocolInfo, physicalAddress);
				this.activateNode(onode, pEntry, sEntry, null, protocolInfo);
				
				if(pEntry!=null){
					
				}else{
					lotosDebug.println("Invalid protocol: "+protocolName);
				}
				
			}else if(onode!=null && this.loadSaveFlag_logical && ovlynode.getElementsByTagName("Application").getLength()>0){
				Element application = (Element)ovlynode.getElementsByTagName("Application").item(0);
				String applicationName = application.getElementsByTagName("ApplicationName").item(0).getChildNodes().item(0).getNodeValue();
				String protocolName = application.getElementsByTagName("ProtocolName").item(0).getChildNodes().item(0).getNodeValue(); 
				String securityName="None";
				if(application.getElementsByTagName("SecurityName").getLength()>0)
					securityName = application.getElementsByTagName("SecurityName").item(0).getChildNodes().item(0).getNodeValue();
				
				ProtocolEntry pEntry = config.getProtocolByName(protocolName);
				ApplicationEntry aEntry = config.getApplicationByName(applicationName);
				SecuritySettingEntry sEntry = config.getSecuritySettingByName(securityName);
				
				this.activateNode(onode, pEntry, sEntry, aEntry, new String(""));
				
			} else if(onode!=null && this.loadSaveFlag_logical && ovlynode.getElementsByTagName("Gateway").getLength()>0){
				Element gateway = (Element)ovlynode.getElementsByTagName("Gateway").item(0);
				String type = gateway.getElementsByTagName("GatewayType").item(0).getChildNodes().item(0).getNodeValue();
				
				this.currentGateway = config.getGatewayByName(type);
				setGateway(onode);
			}
		}
		nl = LoTosProtocolSetting.getElementsByTagName("PhysicalNode");
		//read links
		for(int i=0;i<nl.getLength();i++){
			Element ovlynode = (Element)nl.item(i);
			String xyaddress = ovlynode.getElementsByTagName("XYAddress").item(0).getChildNodes().item(0).getNodeValue();
			PhysicalNode onode = findClosest(new XYAddress(xyaddress), 0);;
			
			//physical links
			if(onode!=null && this.loadSaveFlag_physical && ovlynode.getElementsByTagName("Physical").getLength()>0){
				onode.physical_neighbors.clear();
				Element phyNode = (Element)ovlynode.getElementsByTagName("Physical").item(0);
				NodeList phyLinks = phyNode.getElementsByTagName("WiredLink");
				for(int j=0;j<phyLinks.getLength();j++){
					String endxyaddress = ((Element)phyLinks.item(j)).getElementsByTagName("EndPointXYAddress").item(0).getChildNodes().item(0).getNodeValue();
					PhysicalNode endNode = findClosest(new XYAddress(endxyaddress), 0);
					String bandwidth;
					String buffer;
					if(((Element)phyLinks.item(j)).getElementsByTagName("Bandwidth").item(0) != null)
						bandwidth = ((Element)phyLinks.item(j)).getElementsByTagName("Bandwidth").item(0).getChildNodes().item(0).getNodeValue();
					else
						bandwidth=Integer.toString(SimConfig.WIRE_DEFAULT_BANDWIDTH);
					if(((Element)phyLinks.item(j)).getElementsByTagName("Buffer").item(0) != null)
						buffer = ((Element)phyLinks.item(j)).getElementsByTagName("Buffer").item(0).getChildNodes().item(0).getNodeValue();
					else
						buffer = Integer.toString(SimConfig.WIRE_DEFAULT_BUFFER);
					if(endNode!=null){
					
						float dist = onode.getDirectDistanceTo( endNode );
						
						//this.addEdge(onode, endNode, dist);
						WiredEdge edge = new WiredEdge(onode, endNode, dist, Integer.parseInt(bandwidth), Integer.parseInt(buffer));
						this.addEdge(edge);
					}
				}
			}
			
			I_ProtocolInterface pnode = onode.getProtocol();
			if(onode!=null && pnode!=null && this.loadSaveFlag_logical && ovlynode.getElementsByTagName("Protocol").getLength()>0){
				onode.getProtocol().clearLogicalEdges();
				Element protocol = (Element)ovlynode.getElementsByTagName("Protocol").item(0);
				NodeList logLinks = protocol.getElementsByTagName("LogicalLink");
				for(int j=0;j<logLinks.getLength();j++){
					String endxyaddress = ((Element)logLinks.item(j)).getElementsByTagName("EndPointXYAddress").item(0).getChildNodes().item(0).getNodeValue();
					PhysicalNode endNode = findClosest(new XYAddress(endxyaddress), 0);;
					
					float dist = onode.getDirectDistanceTo( endNode );
					
					onode.getProtocol().insertLogicalEdge(endNode, Color.yellow, 2);
				
				}
			}
			
		}
		updateWirelessAdjacency();
		return true;
	}
	final static String formatTag = "hypercast";
	/* (non-Javadoc)
	 * @see PhysicalNetwork#protocolWriter(java.lang.String)
	 */
	public boolean protocolWriter(String fname) {
		Document doc = XmlUtil.createDocument();
		Element LoTosProtocolSetting = doc.createElement("PhysicalNetwork");
		doc.appendChild(LoTosProtocolSetting);
		
//TODO Remove experimental addons
//		int number_of_links = 0;
//		float distance = 0;
//		float sum_distance = 0;
//		float min_distance = 100000000F;
//		float max_distance = 0;
//
		
		for(int i=0;i<this.nodes.size();i++){
			PhysicalNode onode = (PhysicalNode)nodes.get(i);
			
			Element ovlynode = doc.createElement("PhysicalNode");
			
			
			Element xyaddress = doc.createElement("XYAddress");
			xyaddress.appendChild(doc.createTextNode(""+onode.getAddress()));
			ovlynode.appendChild(xyaddress);
			
			if(this.loadSaveFlag_physical){
				LoTosProtocolSetting.appendChild(ovlynode);
				
				Element phynode = doc.createElement("Physical");
				ovlynode.appendChild(phynode);
				
				String mobiStr = onode.getMobilityStr();
				if(mobiStr==null)
					mobiStr = "None";
				
				Element mobiMode = doc.createElement("MobilityMode");
				mobiMode.appendChild(doc.createTextNode(mobiStr));
				phynode.appendChild(mobiMode);
				
				if(!mobiStr.equals("None")){
					Element mobiSpeed = doc.createElement("MobilitySpeed");
					mobiSpeed.appendChild(doc.createTextNode(""+onode.mobile_speed));
					phynode.appendChild(mobiSpeed);
				}
				
				String wiredStr = onode.wired_link?"Yes":"No";
				String wirelessStr = onode.wireless_link?"Yes":"No";
				
				Element wired = doc.createElement("WiredConnection");
				wired.appendChild(doc.createTextNode(wiredStr));
				phynode.appendChild(wired);
				
				Element wireless = doc.createElement("WirelessConnection");
				wireless.appendChild(doc.createTextNode(wirelessStr));
				phynode.appendChild(wireless);
				
				//save the physical links
				for(int j=0;j<onode.physical_neighbors.size();j++){
					PhysicalNode endNode = (PhysicalNode)onode.physical_neighbors.get(j);
					WiredEdge edge = (WiredEdge)onode.physical_edges.get(j);
					Element phyEdge = doc.createElement("WiredLink");
					Element endXy = doc.createElement("EndPointXYAddress");
					endXy.appendChild(doc.createTextNode(endNode.getAddress().toString()));;
					phyEdge.appendChild(endXy);
					Element bandwidth = doc.createElement("Bandwidth");
					bandwidth.appendChild(doc.createTextNode(Integer.toString(edge.bandwidth)));
					phyEdge.appendChild(bandwidth);
					Element buffer = doc.createElement("Buffer");
					buffer.appendChild(doc.createTextNode(Integer.toString(edge.bufferSize)));
					phyEdge.appendChild(buffer);
					
					phynode.appendChild(phyEdge);
				}
				
				if(onode.wireless_link) {
					Element mac = doc.createElement("WirelessMAC");
					AdhocMAC wmac = onode.wl_MAC;
					
					Element senserange = doc.createElement("SenseRange");
					senserange.appendChild(doc.createTextNode(Float.toString(wmac.SENSE_RANGE)));
					mac.appendChild(senserange);
					Element transrange = doc.createElement("TransRange");
					transrange.appendChild(doc.createTextNode(Float.toString(wmac.SIGNAL_RANGE)));
					mac.appendChild(transrange);
					Element bandwidth = doc.createElement("Bandwidth");
					bandwidth.appendChild(doc.createTextNode(Integer.toString(wmac.bandwidth)));
					mac.appendChild(bandwidth);
					
					phynode.appendChild(mac);
					
				}
				
				
			}
			if(onode.getProtocol() instanceof Protocol) {
				Protocol pnode = (Protocol)onode.getProtocol();
				if(this.loadSaveFlag_logical && pnode!=null){
					if(!this.loadSaveFlag_physical)
						LoTosProtocolSetting.appendChild(ovlynode);
					
					Element protocol = doc.createElement("Protocol");
					ovlynode.appendChild(protocol);
					
					Element protocolName = doc.createElement("ProtocolName");
					protocolName.appendChild(doc.createTextNode(""+pnode.protocolName));
					protocol.appendChild(protocolName);
					
					Element securitySetting = doc.createElement("SecuritySetting");
					securitySetting.appendChild(doc.createTextNode(""+pnode.securitySetting));
					protocol.appendChild(securitySetting);
					
					Element info = doc.createElement("ProtocolInfo");
					info.appendChild(doc.createTextNode(""+pnode.agent.getInfoForNode(pnode.hc_socket)));
					protocol.appendChild(info);
					
					Element pa = doc.createElement("PhysicalAddress");
					pa.appendChild(doc.createTextNode(""+pnode.nodeAdapter.createPhysicalAddress()));
					protocol.appendChild(pa);
					
					//save the logical links
					//This only makes the network look nice when it's just loaded. the logical links aren't actually put into the nodes.
					for(int j=0;j<onode.getProtocol().getLogicalNeighbors().size();j++){
						PhysicalNode endNode = (PhysicalNode)onode.getProtocol().getLogicalNeighbors().get(j);
						Element logEdge = doc.createElement("LogicalLink");
						Element endXy = doc.createElement("EndPointXYAddress");
						endXy.appendChild(doc.createTextNode(endNode.getAddress().toString()));;
						logEdge.appendChild(endXy);
						protocol.appendChild(logEdge);
//						TODO Remove experimental addons
//						number_of_links++;
//						distance = onode.getAddress().distanceTo(endNode.getAddress());
//						sum_distance += distance;
//						min_distance = min_distance<distance ? min_distance : distance;
//						max_distance = max_distance>distance ? max_distance : distance;
//
					}
					
				}
			} else if (onode.getProtocol() instanceof LotosHyperCastAppl) {
				LotosHyperCastAppl appl = (LotosHyperCastAppl) onode.getProtocol();
				if(this.loadSaveFlag_logical && appl != null) {
					Element eAppl = doc.createElement("Application");
					ovlynode.appendChild(eAppl);
					
					Element eApplName = doc.createElement("ApplicationName");
					eApplName.appendChild(doc.createTextNode(onode.nodeInfo.application.name));
					eAppl.appendChild(eApplName);
					
					Element eProtocol = doc.createElement("ProtocolName");
					eProtocol.appendChild(doc.createTextNode(onode.nodeInfo.protocol.name));
					eAppl.appendChild(eProtocol);
					
					if(onode.nodeInfo.security != null) {
						Element eSecurity = doc.createElement("SecurityName");
						eSecurity.appendChild(doc.createTextNode(onode.nodeInfo.security.name));
						eAppl.appendChild(eSecurity);		
					}
				}
			} else if (onode.getProtocol() instanceof LotosGateway) {
				LotosGateway gateway = (LotosGateway) onode.getProtocol();
				if(this.loadSaveFlag_logical && gateway != null) {
					Element eGateway = doc.createElement("Gateway");
					ovlynode.appendChild(eGateway);
					
					Element eGName = doc.createElement("GatewayType");
					eGName.appendChild(doc.createTextNode(gateway.type));
					eGateway.appendChild(eGName);
				}
			}
		}
		
//		TODO Remove experimental addons
//		lotosDebug.println("Average/Min/Max: \n"+(sum_distance/number_of_links)+"\n"+min_distance+"\n"+max_distance);
//		
		try {
			FileOutputStream output = new FileOutputStream(fname);
			XmlUtil.writeXml(doc, output);
		} catch (Exception e) {
			lotosDebug.errorln("Error when saving network to file: "+fname);
	        return false;
		}
		return true;
	}
	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public synchronized void actionPerformed(ActionEvent arg0) {
		if(arg0.getSource() instanceof MenuItem){
			MenuItem mi = (MenuItem)arg0.getSource();
			String menu_name = mi.getLabel();
			
			Enumeration e = selectedNodes.elements();
			
			while(e.hasMoreElements()){
				PhysicalNode selectedNode = (PhysicalNode)e.nextElement();
				
				if(menu_name.equals(("Activate"))){
					activateNode(selectedNode);
				}else if(menu_name.equals(("Deactivate"))){
					deactivateNode(selectedNode);
				}else if(menu_name.equals("Node Statistics")){
					showStats(selectedNode);
				}
				else if(menu_name.equals("Event History")){
					showEventHistory(selectedNode);
				} else if(menu_name.equals("Set Gateway")) {
					setGateway(selectedNode);
				}
			}
			
		}
		if(the_gui!=null){
			the_gui.repaint();
			if(the_gui.dw!=null)
				the_gui.dw.repaint();
		}
		selectedNodes.clear();
		
	}
	/**
	 * this sets a physical node's protocol to the gateway protocol. 
	 * @param n the idle physical node
	 */
	
	void setGateway(PhysicalNode n) {
		if (n == null)
			return;

		if (active_nodes.contains(n)) {
			return;
		}
		//create adapter
		if (adpt_mngr == null) {
			adpt_mngr = new LotosAdapterManager();
		}
		
		LotosGateway gateway = new LotosGateway(this, n, adpt_mngr, currentGateway.config, currentGateway.name);
		n.setProtocol(gateway);
		gateway_nodes.add(n);
	}
	
	/* (non-Javadoc)
	 * @see PhysicalNetwork#activateNode(PhysicalNode)
	 */
	public boolean activateNode(PhysicalNode o) {
		if(o==null)return false;
		if(currentProtocol!=null)
			return activateNode(o, currentProtocol, currentSecurity, currentAppl, null);
		else
			return false;
	}
	/* (non-Javadoc)
	 * @see java.awt.event.ItemListener#itemStateChanged(java.awt.event.ItemEvent)
	 */
	public synchronized void itemStateChanged(ItemEvent arg0) {
		
		CheckboxMenuItem mi = (CheckboxMenuItem)arg0.getSource();
		String menu_name = mi.getLabel();
		Object parentMenu = mi.getParent();
		
		if(selectedProtocolMenu!=null && parentMenu == selectedProtocolMenu){
			
			Enumeration e = selectedNodes.elements();
			while(e.hasMoreElements()){
				PhysicalNode selectedNode = (PhysicalNode)e.nextElement();
			
				Protocol pnode = (Protocol)selectedNode.getProtocol();
				if(pnode!=null){
					pnode.agent.menuFired(menu_name);
				}
			}
		}else if(parentMenu == securityMenu){
			chooseSecurity(menu_name);
			
		}
		
		else if(parentMenu == protocolMenu){
			chooseProtocol(menu_name);
		}
		
		else if(parentMenu == applMenu) {
			chooseApplication(menu_name);
		}
		else if(parentMenu == gatewayMenu) {
			chooseGateway(menu_name);
		}
	}
	public String getCurrentProtocolName(){
		if(this.currentProtocol!=null){
			return currentProtocol.getName();
		}else{
			return "None";
		}
	}
	public String getCurrentApplicationName(){
		if(this.currentAppl!=null){
			return currentAppl.getName();
		}else{
			return "None";
		}
	}
	public String getCurrentSecurity(){
		if(this.currentSecurity!=null){
			return this.currentSecurity.name;
		}else{
			return "None";
		}
		
	}
	public String getCurrentGateway() {
		if(currentGateway!=null)
			return currentGateway.name;
		else
			return "None";
	}
	
	public void chooseGateway(String gateway_name) {
		getGatewayMenu();
		currentGateway = config.getGatewayByName(gateway_name); 
		for(int i=0;i<gatewayMenu.getItemCount();i++){
			CheckboxMenuItem pmi = (CheckboxMenuItem)gatewayMenu.getItem(i);
			if(pmi.getLabel().equals(gateway_name)){
				pmi.setState(true);
			}else{
				pmi.setState(false);
			}
		}
		
		Enumeration e = selectedNodes.elements();
		if(e.hasMoreElements()) {
			PhysicalNode selectedNode = (PhysicalNode)e.nextElement();
			deactivateNode(selectedNode);
			if(gateway_nodes.size()==0)
				setGateway(selectedNode);
		}
		if(the_gui!=null){
			the_gui.repaint();
			if(the_gui.dw!=null)
				the_gui.dw.repaint();
		}
	}
	
	public void chooseSecurity(String menu_name){
		getSecurityMenu();
		
		if(menu_name.equals("None")){
			currentSecurity = null;
		}else{
		
			String[] secNames = config.getSecuritySettingNames();
			
			currentSecurity = config.getSecuritySettingByName(menu_name);
			
		}
		for(int i=0;i<security_menus.length;i++){
			if(security_menus[i].getLabel().equals(menu_name)){
				security_menus[i].setState(true);
			}else{
				security_menus[i].setState(false);
			}
		}
		
		Enumeration e = selectedNodes.elements();
		while(e.hasMoreElements()){
			PhysicalNode selectedNode = (PhysicalNode)e.nextElement();
			
			if(selectedNode.getProtocol() == null) {
				activateNode(selectedNode);
			} else {
				deactivateNode(selectedNode);
				activateNode(selectedNode, selectedNode.nodeInfo.protocol, currentSecurity, selectedNode.nodeInfo.application, null);
			}
		}
		if(the_gui!=null){
			the_gui.repaint();
			if(the_gui.dw!=null)
				the_gui.dw.repaint();
		}
	}
	
	public void chooseProtocol(String menu_name){
		getProtocolMenu();
		ProtocolEntry entry=null;
		
		if(menu_name.equals("None")){
			entry = null;
		}else{
		
			String[] protocolNames = config.getProtocolNames();
			
			entry = config.getProtocolByName(menu_name);
		}
		currentProtocol = entry;
		Enumeration e = selectedNodes.elements();
		while(e.hasMoreElements()){
			PhysicalNode selectedNode = (PhysicalNode)e.nextElement();
			if(entry == null && selectedNode.getProtocol() != null) {
				deactivateNode(selectedNode);
				continue;
			} else if(entry !=null && selectedNode.getProtocol() == null){
				activateNode(selectedNode);
				continue;
			} else if(entry != null && selectedNode.getProtocol() != null) {
				deactivateNode(selectedNode);
				activateNode(selectedNode, entry, selectedNode.nodeInfo.security, selectedNode.nodeInfo.application, null);
			}
		}
				
		for(int i=0;i<protocolMenu.getItemCount();i++){
			CheckboxMenuItem pmi = (CheckboxMenuItem)protocolMenu.getItem(i);
			if(pmi.getLabel().equals(menu_name)){
				pmi.setState(true);
			}else{
				pmi.setState(false);
			}
		}
		if(the_gui!=null){
			the_gui.repaint();
			if(the_gui.dw!=null)
				the_gui.dw.repaint();
		}
		
	}
	
	public void chooseApplication(String menu_name) {
		getApplMenu();
		ApplicationEntry entry=null;
		
		if(menu_name.equals("None")){
			entry = null;
		}else{
			entry = config.getApplicationByName(menu_name);
		}
		currentAppl = entry;
		Enumeration e = selectedNodes.elements();
		while(e.hasMoreElements()){
			PhysicalNode selectedNode = (PhysicalNode)e.nextElement();

			//if the node is already running the correct application, skip it.
			if(entry != null && entry.getApplClass().isInstance(selectedNode.getProtocol()))
				continue;
			//if nothing is running, activate it.
			if(entry != null && selectedNode.getProtocol()==null)
				activateNode(selectedNode);
			
			this.deactivateNode(selectedNode);
			this.activateNode(selectedNode, selectedNode.nodeInfo.protocol, selectedNode.nodeInfo.security, entry, null);
			
		}
				
		for(int i=0;i<applMenu.getItemCount();i++){
			CheckboxMenuItem pmi = (CheckboxMenuItem)applMenu.getItem(i);
			if(pmi.getLabel().equals(menu_name)){
				pmi.setState(true);
			}else{
				pmi.setState(false);
			}
		}
		if(the_gui!=null){
			the_gui.repaint();
			if(the_gui.dw!=null)
				the_gui.dw.repaint();
		}
	}
	
	public I_PhysicalAddress getPhysicalAddress(I_LogicalAddress laddr) {
		//String knownLA = "";
		for(int i=0;i<protocols.size();i++) {
			I_ProtocolInterface protocol = (I_ProtocolInterface)protocols.get(i);
			//knownLA+=agent.getLA()+" ";
			if(protocol.getLogicalAddress().toString().equals(laddr.toString())) {
				//lotosDebug.println(" | "+knownLA);
				return protocol.getPhysicalAddress();
			}
		}

		//lotosDebug.println(" | "+knownLA);
		return null;
	}
	public void stop() {
		this.simRunning = false;
		while(active_nodes.size()>0) {
			deactivateNode((PhysicalNode)active_nodes.get(0));
		}
		if(gateway_nodes.size()>0) {
			LotosGateway gateway = (LotosGateway)((PhysicalNode)gateway_nodes.get(0)).getProtocol();
			gateway.reset();
		}
	}
}
